// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: CalendarBase.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:25:23  jens
 *  Initial import
 *
 *  Revision 1.6  2002/08/30 18:21:10  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.5  2002/08/07 19:14:59  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.4  2002/07/24 17:04:24  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.3  2002/07/24 14:42:42  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Globalization;
using System.Runtime.InteropServices;
using Stochastix.Utilities;
using Stochastix.Business;

namespace Stochastix.Business.Calendars
{

	/// <summary>
	/// This class provides methods for determining whether a date is a business 
	/// day or a holiday for a given market, and for incrementing/decrementing a 
	/// date of a given number of business days.
	/// </summary>
	[ RcsId("$Id: CalendarBase.cs,v 1.1.1.1 2003/02/01 18:25:23 jens Exp $") ] 
	[ ComVisible(false) ]
	public abstract class CalendarBase : IBusinessCalendar 
	{
		/// <summary>
		/// A date corresponding to the OLE Automation 0.0 date value.
		/// </summary>
		public static DateTime NullDate = DateTime.FromOADate(0.0);

		#region Constructors

		/// <summary>
		/// Construct a new BusinessCalendar based on a localized 
		/// culture and its default calendar.
		/// </summary>
		/// <param name="culture">The associated <see cref="System.Globalization.CultureInfo"/>.</param>
		/// <param name="name">The name of this business calendar.</param>
		protected CalendarBase(String name, CultureInfo culture) 
			: this(name, culture, culture.Calendar) 
		{}
			
		/// <summary>
		/// Construct a new BusinessCalendar based on a localized 
		/// culture and non-default calendar.
		/// </summary>
		/// <param name="name">The name of this business calendar.</param>
		/// <param name="culture">The associated <see cref="System.Globalization.CultureInfo"/>.</param>
		/// <param name="calendar">The underlying <see cref="System.Globalization.Calendar"/>.</param>
		protected CalendarBase( String name, CultureInfo culture, Calendar calendar )
		{
			this.name = name;
			this.calendar = calendar;
			this.culture = culture;
		}

		#endregion

		#region Properties

		private String name;

		/// <summary>
		/// A stringified representation of the Calendar.
		/// </summary>
		/// <returns>A String representing the object.</returns>
		override public String ToString() 
		{ 
			return name; 
		}


		/// <summary>
		/// The underlying <see cref="System.Globalization.Calendar"/>.
		/// </summary>
		private Calendar calendar;

		/// <summary>
		/// The underlying <see cref="System.Globalization.Calendar"/>.
		/// </summary>
		public Calendar Calendar
		{
			get { return calendar; }
		}

		/// <summary>
		/// The associated <see cref="System.Globalization.CultureInfo"/>.
		/// </summary>
		private CultureInfo culture;

		/// <summary>
		/// The associated <see cref="System.Globalization.CultureInfo"/>.
		/// </summary>
		public CultureInfo Culture
		{
			get { return culture; }
		}

		/// <summary>
		/// The associated <see cref="System.Globalization.RegionInfo"/>.
		/// </summary>
		public RegionInfo Region
		{
			get { return new RegionInfo(culture.LCID); }
		}

		#endregion

		#region Business days

		/// <summary>
		/// Returns <c>true</c> iff the date is a business day for the given market.
		/// </summary>
		/// <param name="date">The <see cref="DateTime"/> to check.</param>
		/// <returns><c>true</c> iff the date is a business day for the given market.</returns>
		public abstract bool IsBusinessDay(DateTime date);

		/// <summary>
		/// Returns <c>true</c> iff the date is a holiday for the given market.
		/// </summary>
		/// <param name="date">The <see cref="DateTime"/> to check.</param>
		/// <returns><c>true</c> iff the date is a holiday for the given market.</returns>
		public bool IsHoliday(DateTime date) 
		{
			return !IsBusinessDay(date);
		}

		#endregion

		#region Rolling and advancing

		/// <summary>
		/// Find the next business day with respect to the given date and
		/// rolling convention.
		/// </summary>
		/// <overloads>
		/// Find the next business day.
		/// </overloads>
		/// <param name="date">The <see cref="DateTime"/> to roll.</param>
		/// <param name="convention">The <see cref="RollingConvention"/> to use.</param>
		/// <returns>The resulting business day.</returns>
		public DateTime Roll(DateTime date, RollingConvention convention) 
		{
			// QL_REQUIRE(d!=Date(), "Calendar::roll : null date");
			DateTime d1 = date;
			if (convention == RollingConvention.Following || convention == RollingConvention.ModifiedFollowing) 
			{
				while (IsHoliday(d1))
					d1 = Calendar.AddDays(d1,1);
				if (convention == RollingConvention.ModifiedFollowing && d1.Month != date.Month) 
					return Roll(date, RollingConvention.Preceding);
			} 
			else if (convention == RollingConvention.Preceding || convention == RollingConvention.ModifiedPreceding) 
			{
				while (IsHoliday(d1))
					d1 = Calendar.AddDays(d1,-1);
				if (convention == RollingConvention.ModifiedPreceding && d1.Month != date.Month) 
					return Roll(date, RollingConvention.Following);
			} 
			else if ( convention != RollingConvention.Actual ) 
			{
				throw new ArgumentOutOfRangeException("Unknown rolling convention.");
			}
			return d1;
		}

		/// <summary>
		/// Find the next business day with respect to the given date.
		/// </summary>
		/// <remarks>
		/// This overload defaults to the Following <see cref="RollingConvention"/>
		/// to choose the first business day after the given holiday.
		/// </remarks>
		/// <param name="date">The <see cref="DateTime"/> to roll.</param>
		/// <returns>The resulting business day.</returns>
		public DateTime Roll(DateTime date) 
		{
			return Roll(date, RollingConvention.Following);
		}

		/// <summary>
		/// Advances the given <see cref="DateTime"/> the given
		/// <paramref name="length" /> of time <paramref name="units"/>.
		/// </summary>
		/// <overloads>
		/// Advances the given <see cref="DateTime"/>.
		/// </overloads>
		/// <param name="date">The <see cref="DateTime"/> that is to be advanced.</param>
		/// <param name="length">The number of time units to advance.</param>
		/// <param name="units">The <see cref="TimeUnit"/> used.</param>
		/// <param name="convention">The <see cref="RollingConvention"/> to use.</param>
		/// <returns>Returns the resulting <see cref="DateTime"/>.</returns>
		public DateTime Advance(DateTime date, int length, TimeUnit units,
			RollingConvention convention) 
		{
			// QL_REQUIRE(d!=Date(), "Calendar::roll : null date");
			if (length == 0) return Roll(date, convention);
			else if (units == TimeUnit.Days) 
			{
				DateTime d1 = date;
				if (length > 0) 
				{
					while (length > 0) 
					{
						d1 = d1.AddDays(1);
						while (IsHoliday(d1))
							d1 = d1.AddDays(1);
						length--;
					}
				} 
				else 
				{
					while (length < 0) 
					{
						d1 = d1.AddDays(-1);
						while(IsHoliday(d1))
							d1 = d1.AddDays(-1);
						length++;
					}
				}
				return d1;
			} 
			else 
			{
				return Roll(date + new Period(length, units), convention);
			}
		}

		/// <summary>
		/// Advances the given <see cref="DateTime"/> the given
		/// <paramref name="length" /> of time <paramref name="units"/>.
		/// </summary>
		/// <remarks>
		/// This overload defaults to the Following <see cref="RollingConvention"/>
		/// to choose the first business day after the given holiday.
		/// </remarks>
		/// <param name="date">The <see cref="DateTime"/> that is to be advanced.</param>
		/// <param name="length">The number of time units to advance.</param>
		/// <param name="units">The <see cref="TimeUnit"/> used.</param>
		/// <returns>Returns the resulting <see cref="DateTime"/>.</returns>
		public DateTime Advance(DateTime date, int length, TimeUnit units) 
		{
			return Advance(date, length, units, RollingConvention.Following);
		}

		/// <summary>
		/// Advances the given <see cref="DateTime"/> the given <see cref="Period"/>.
		/// </summary>
		/// <param name="date">The <see cref="DateTime"/> that is to be advanced.</param>
		/// <param name="period">The <see cref="Period"/> that the given date is advanced.</param>
		/// <param name="convention">The <see cref="RollingConvention"/> to use.</param>
		/// <returns>Returns the resulting <see cref="DateTime"/>.</returns>
		public DateTime Advance(DateTime date, Period period, RollingConvention convention) 
		{
			return Advance(date, period.Length, period.Units, convention);
		}

		/// <summary>
		/// Advances the given <see cref="DateTime"/> the given <see cref="Period"/>.
		/// </summary>
		/// <remarks>
		/// This overload defaults to the Following <see cref="RollingConvention"/>
		/// to choose the first business day after the given holiday.
		/// </remarks>
		/// <param name="date">The <see cref="DateTime"/> that is to be advanced.</param>
		/// <param name="period">The <see cref="Period"/> that the given date is advanced.</param>
		/// <returns>Returns the resulting <see cref="DateTime"/>.</returns>
		public DateTime Advance(DateTime date, Period period) 
		{ 
			return Advance(date, period.Length, period.Units, RollingConvention.Following);
		}

		/// <summary>
		/// Advances the given <see cref="DateTime"/> the days given in a <see cref="TimeSpan"/>.
		/// </summary>
		/// <param name="date">The <see cref="DateTime"/> that is to be advanced.</param>
		/// <param name="interval">The <see cref="TimeSpan"/> that the given date is advanced.</param>
		/// <param name="convention">The <see cref="RollingConvention"/> to use.</param>
		/// <returns>Returns the resulting <see cref="DateTime"/>.</returns>
		public DateTime Advance(DateTime date, TimeSpan interval, RollingConvention convention)
		{
			return Advance(date, interval.Days, TimeUnit.Days, convention);
		}

		/// <summary>
		/// Advances the given <see cref="DateTime"/> the days given in a <see cref="TimeSpan"/>.
		/// </summary>
		/// <remarks>
		/// This overload defaults to the Following <see cref="RollingConvention"/>
		/// to choose the first business day after the given holiday.
		/// </remarks>
		/// <param name="date">The <see cref="DateTime"/> that is to be advanced.</param>
		/// <param name="interval">The <see cref="TimeSpan"/> that the given date is advanced.</param>
		/// <returns>Returns the resulting <see cref="DateTime"/>.</returns>
		public DateTime Advance(DateTime date, TimeSpan interval) 
		{
			return Advance(date, interval.Days, TimeUnit.Days, RollingConvention.Following);
		}

		#endregion

	}
}
