// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: IBusinessCalendar.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:25:24  jens
 *  Initial import
 *
 *  Revision 1.4  2002/08/30 18:21:10  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.3  2002/07/24 17:04:24  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.2  2002/07/24 14:42:42  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Globalization;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Utilities;
using Stochastix.Business;

namespace Stochastix.Business {

	/// <summary>
	/// This interface provides methods for determining whether a date is a business 
	/// day or a holiday for a given market, and for incrementing/decrementing a 
	/// date of a given number of business days.
	/// </summary>
	/// <remarks>
	/// This interface is mainly for COM interop.
	/// </remarks>
	[ RcsId("$Id: IBusinessCalendar.cs,v 1.1.1.1 2003/02/01 18:25:24 jens Exp $") ]
	[ ComVisible(true) ]
	[ InterfaceTypeAttribute(ComInterfaceType.InterfaceIsDual) ]
	[
		Description( "This interface provides methods for determining whether a date is a business" +
					 " day or a holiday for a given market, and for incrementing/decrementing a" +
					 " date of a given number of business days." )
	]
	public interface IBusinessCalendar {

		/// <summary>
		/// Returns a <see cref="String"/> that represents the current Object.
		/// </summary>
		/// <remarks>
		/// <para>
		/// This method returns a human-readable string that is culture-sensitive.
		/// </para>
		/// <para>
		/// The default implementation returns the fully qualified name of the type
		/// of the Object. This method can be overridden in a derived
		/// class to return values that are meaningful for that type. 
		/// </para>
		/// </remarks>
		/// <returns>
		/// A <see cref="String"/> that represents the current Object.
		/// </returns>
		[ Description("A stringified representation of this object.") ]
		[ DispId(0) ]	// DISPID_VALUE
		String ToString();

		/// <summary>
		/// Returns <c>true</c> iff the date is a business day for the given market.
		/// </summary>
		/// <param name="date">The <see cref="DateTime"/> to check.</param>
		/// <returns><c>true</c> iff the date is a business day for the given market.</returns>
		[ Description( "Returns true iff the date is a business  day for the given market.") ]
		bool IsBusinessDay(DateTime date);

		/// <summary>
		/// Returns <c>true</c> iff the date is a holiday for the given market.
		/// </summary>
		/// <param name="date">The System.DateTime to check.</param>
		/// <returns><c>true</c> iff the date is a holiday for the given market.</returns>
		[ Description( "Returns true iff the date is a holiday for the given market.") ]
		bool IsHoliday(DateTime date);

		/// <summary>
		/// Find the next business day with respect to the given date and
		/// rolling convention.
		/// </summary>
		/// <overloads>
		/// Find the next business day.
		/// </overloads>
		/// <param name="date">The <see cref="DateTime"/> to roll.</param>
		/// <param name="convention">The <see cref="RollingConvention"/> to use.</param>
		/// <returns>The resulting business day.</returns>
		[ Description( "Find the next business day with respect to the given date") ]
		DateTime Roll(DateTime date, RollingConvention convention);

		/// <summary>
		/// Find the next business day with respect to the given date.
		/// </summary>
		/// <remarks>
		/// This overload defaults to the Following <see cref="RollingConvention"/>
		/// to choose the first business day after the given holiday.
		/// </remarks>
		/// <param name="date">The <see cref="DateTime"/> to roll.</param>
		/// <returns>The resulting business day.</returns>
		[ ComVisible(false) ]
		DateTime Roll(DateTime date);

		/// <summary>
		/// Advances the given <see cref="DateTime"/> the given
		/// <paramref name="length" /> of time <paramref name="units"/>.
		/// </summary>
		/// <overloads>
		/// Advances the given <see cref="DateTime"/>.
		/// </overloads>
		/// <param name="date">The <see cref="DateTime"/> that is to be advanced.</param>
		/// <param name="length">The number of time units to advance.</param>
		/// <param name="units">The <see cref="TimeUnit"/> used.</param>
		/// <param name="convention">The <see cref="RollingConvention"/> to use.</param>
		/// <returns>Returns the resulting <see cref="DateTime"/>.</returns>
		[ Description( "Advances the given System.DateTime the given number of time units.") ]
		DateTime Advance(DateTime date, int length, TimeUnit units, RollingConvention convention);

		/// <summary>
		/// Advances the given <see cref="DateTime"/> the given
		/// <paramref name="length" /> of time <paramref name="units"/>.
		/// </summary>
		/// <remarks>
		/// This overload defaults to the Following <see cref="RollingConvention"/>
		/// to choose the first business day after the given holiday.
		/// </remarks>
		/// <param name="date">The <see cref="DateTime"/> that is to be advanced.</param>
		/// <param name="length">The number of time units to advance.</param>
		/// <param name="units">The <see cref="TimeUnit"/> used.</param>
		/// <returns>Returns the resulting <see cref="DateTime"/>.</returns>
		[ ComVisible(false) ]
		DateTime Advance(DateTime date, int length, TimeUnit units);

		/// <summary>
		/// Advances the given <see cref="DateTime"/> the given <see cref="Period"/>.
		/// </summary>
		/// <param name="date">The <see cref="DateTime"/> that is to be advanced.</param>
		/// <param name="period">The <see cref="Period"/> that the given date is advanced.</param>
		/// <param name="convention">The <see cref="RollingConvention"/> to use.</param>
		/// <returns>Returns the resulting <see cref="DateTime"/>.</returns>
		[ ComVisible(false) ]
		DateTime Advance(DateTime date, Period period, RollingConvention convention);

		/// <summary>
		/// Advances the given <see cref="DateTime"/> the given <see cref="Period"/>.
		/// </summary>
		/// <remarks>
		/// This overload defaults to the Following <see cref="RollingConvention"/>
		/// to choose the first business day after the given holiday.
		/// </remarks>
		/// <param name="date">The <see cref="DateTime"/> that is to be advanced.</param>
		/// <param name="period">The <see cref="Period"/> that the given date is advanced.</param>
		/// <returns>Returns the resulting <see cref="DateTime"/>.</returns>
		[ ComVisible(false) ]
		DateTime Advance(DateTime date, Period period);

		/// <summary>
		/// Advances the given <see cref="DateTime"/> the days given in a <see cref="TimeSpan"/>.
		/// </summary>
		/// <param name="date">The <see cref="DateTime"/> that is to be advanced.</param>
		/// <param name="interval">The <see cref="TimeSpan"/> that the given date is advanced.</param>
		/// <param name="convention">The <see cref="RollingConvention"/> to use.</param>
		/// <returns>Returns the resulting <see cref="DateTime"/>.</returns>
		[ ComVisible(false) ]
		DateTime Advance(DateTime date, TimeSpan interval, RollingConvention convention);

		/// <summary>
		/// Advances the given <see cref="DateTime"/> the days given in a <see cref="TimeSpan"/>.
		/// </summary>
		/// <remarks>
		/// This overload defaults to the Following <see cref="RollingConvention"/>
		/// to choose the first business day after the given holiday.
		/// </remarks>
		/// <param name="date">The <see cref="DateTime"/> that is to be advanced.</param>
		/// <param name="interval">The <see cref="TimeSpan"/> that the given date is advanced.</param>
		/// <returns>Returns the resulting <see cref="DateTime"/>.</returns>
		[ ComVisible(false) ]
		DateTime Advance(DateTime date, TimeSpan interval);

		/// <summary>
		/// The underlying <see cref="System.Globalization.Calendar"/>.
		/// </summary>
		[ ComVisible(false) ]
		Calendar Calendar { get ; }

		/// <summary>
		/// The associated <see cref="System.Globalization.CultureInfo"/>.
		/// </summary>
		[ ComVisible(false) ]
		CultureInfo Culture { get ; }

		/// <summary>
		/// The associated <see cref="System.Globalization.RegionInfo"/>.
		/// </summary>
		[ ComVisible(false) ]
		RegionInfo Region {	get ; }

	}
}