// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: JointCalendar.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:25:24  jens
 *  Initial import
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Globalization;
using System.Runtime.InteropServices;
using Stochastix.Utilities;
using Stochastix.Business;
using Stochastix.Business.Resources;

namespace Stochastix.Business.Calendars
{

	/// <summary>
	/// Calendar combining holidays or business days from two other calendars.
	/// </summary>
	/// <remarks>
	/// <para>
	///	To combine holidays from Tokyo, London and New York use the following example:
	///	<example>
	///	[C#]
	///	<code>
	///	using Stochastix.Business;
	///	using Stochastix.Business.Calendars;
	///	
	///	IBusinessCalendar gbpUsdCalendar = new JointCalendar(
	///			London.Instance, NewYork.Instance, "GBP/USD"
	///	);
	///	IBusinessCalendar jointCalendar = new JointCalendar(
	///		Tokyo.Instance, gbpUsdCalendar, "YEN/GBP/USD"
	///	);
	///	jointCalendar = JointCalendar.Create( new IBusinessCalendar[] {
	///		Tokyo.Instance, London.Instance, NewYork.Instance }, 
	///		"YEN/GBP/USD"
	///	);
	///	</code>
	///	Regarding globalization settings, the resulting <c>jointCalendar</c> 
	///	behaves like the original <c>Tokyo</c> calendar.
	///	</example>
	///	</para>
	/// </remarks>
	[ RcsId("$Id: JointCalendar.cs,v 1.1.1.1 2003/02/01 18:25:24 jens Exp $") ] 
	[ ComVisible(false) ]
	public sealed class JointCalendar : CalendarBase
	{

		/// <summary>
		/// Calendar combination rule.
		/// </summary>
		public enum Rule 
		{
			/// <summary>
			/// The resulting calendar has holidays from both calendars. 
			/// That is, in other words, only common business days.
			/// </summary>
			Holidays,
			/// <summary>The resulting calendar has business days from both calendars. 
			/// That is, in other words, only common holidays.</summary>
			BusinessDays
		};

		#region Constructors

		/// <summary>
		/// Combine a main calendar with holidays from another calendar.
		/// </summary>
		/// <param name="mainCalendar">
		/// The main calendar (also used for globalization settings).</param>
		/// <param name="otherCalendar">
		/// The calendar which adds holidays to the main calendar.
		/// </param>
		/// <remarks>
		/// The combined calendars use the <c>Holidays</c> combination <see cref="Rule"/>.
		/// The resulting name can be localized in the resource file.
		/// </remarks>
		public JointCalendar(IBusinessCalendar mainCalendar, IBusinessCalendar otherCalendar)
			: this(mainCalendar, otherCalendar, 
			Strings.Format(null, "CalJoinHol", mainCalendar, Rule.Holidays, otherCalendar),
			Rule.Holidays)
		{}

		/// <summary>
		/// Combine a main calendar with holidays from another calendar.
		/// </summary>
		/// <param name="mainCalendar">
		/// The main calendar (also used for globalization settings).</param>
		/// <param name="otherCalendar">
		/// The calendar which adds holidays to the main calendar.
		/// </param>
		/// <param name="name">
		/// Name of the combined calendar.
		/// </param>
		/// <remarks>
		/// The combined calendars use the <c>Holidays</c> combination <see cref="Rule"/>.
		/// </remarks>
		public JointCalendar(IBusinessCalendar mainCalendar, IBusinessCalendar otherCalendar,
			string name)
			: this(mainCalendar, otherCalendar, name, Rule.Holidays)
		{}

		/// <summary>
		/// Combine a main calendar with another calendar.
		/// </summary>
		/// <param name="mainCalendar">
		/// The main calendar (also used for globalization settings).</param>
		/// <param name="otherCalendar">
		/// The calendar which is combined with
		///  the main calendar.
		/// </param>
		/// <param name="name">
		/// Name of the combined calendar.
		/// </param>
		/// <param name="rule">
		/// The combination <see cref="Rule"/> of the two calendars.
		/// </param>
		/// <remarks>
		/// The calendar combination <see cref="Rule"/> is used to join
		/// calendars on common business days or holidays:
		/// <list type="table">
		/// <listheader>
		/// <term>Calendar combination <see cref="Rule"/>.</term>
		/// <description>Description</description>
		/// </listheader>
		/// <item>
		/// <term><c>Holidays</c></term>
		/// <description>
		/// The resulting calendar has holidays from both calendars. 
		/// That is, in other words, only common business days.
		/// </description>
		/// </item>
		/// <item>
		/// <term><c>BusinessDays</c></term>
		/// <description>
		/// The resulting calendar has business days from both calendars. 
		/// That is, in other words, only common holidays.
		/// </description>
		/// </item>
		/// </list>
		/// </remarks>
		/// <exception cref="NullReferenceException">
		/// Thrown when a null reference has been passed as calendar.
		/// </exception>
		public JointCalendar(IBusinessCalendar mainCalendar, IBusinessCalendar otherCalendar,
			string name, Rule rule)
			: base(name, mainCalendar.Culture, mainCalendar.Calendar)
		{
			if( /* mainCalendar == null || */ otherCalendar == null)
				throw new NullReferenceException(
					Strings.Get(GetType(), "CalNullRef") );
			this.mainCalendar = mainCalendar;
			this.otherCalendar = otherCalendar;
			this.combineBusinessDays = (rule == Rule.BusinessDays);
		}

		#endregion

		private IBusinessCalendar mainCalendar;
		private IBusinessCalendar otherCalendar;
		private bool combineBusinessDays;

		/// <summary>
		/// Returns <c>true</c> iff the date is a business day for the given market.
		/// </summary>
		/// <param name="date">The <see cref="DateTime"/> to check.</param>
		/// <returns><c>true</c> iff the date is a business day for the given market.</returns>
		override public bool IsBusinessDay(DateTime date) 
		{
			// at least two of three conditions must be true
			// return (m&&o) || (m&&combineBusinessDays) || (o&&combineBusinessDays);
			bool m = mainCalendar.IsBusinessDay(date);
			if( combineBusinessDays && m ) // sufficient
				return true;
			bool o = otherCalendar.IsBusinessDay(date);
			return (m&&o) || (combineBusinessDays&&o);
		}

		/// <summary>
		/// Create a <see cref="JointCalendar"/> from multiple <see cref="IBusinessCalendar"/>s.
		/// </summary>
		/// <remarks>
		/// This static method recursively constructs a <see cref="JointCalendar"/> from the passed
		/// in calendars. The result contains holidays from all calendars. 
		/// See the class constructor for more information.
		/// <para>
		/// You must at least pass in two calendars two join. 
		/// Passing in one calendar does not result in an exception. Instead the calendar
		/// is returned under it's original name.
		/// </para>
		/// <para>
		///	To combine holidays from Tokyo, London and New York use the following example:
		///	<example>
		///	[C#]
		///	<code>
		///	using Stochastix.Business;
		///	using Stochastix.Business.Calendars;
		///	
		///	IBusinessCalendar jointCalendar = JointCalendar.Create( 
		///		new IBusinessCalendar[] {
		///			Tokyo.Instance, London.Instance, NewYork.Instance 
		///		}, "YEN/GBP/USD" );
		///	</code>
		///	Regarding globalization settings, the resulting <c>jointCalendar</c> 
		///	behaves like the original <c>Tokyo</c> calendar.
		///	</example>
		///	</para>
		/// </remarks>
		/// <param name="calendars">
		/// An <see cref="Array"/> of <see cref="IBusinessCalendar"/>s. 
		/// </param>
		/// <param name="name">
		/// The name for the <see cref="JointCalendar"/>.
		/// </param>
		/// <exception cref="ArgumentException">
		/// Thrown when an empty array has been given to <paramref name="calendars"/>.
		/// </exception>
		/// <returns>
		/// The resulting calendar has holidays from all calendars. 
		/// That is, in other words, only common business days.
		/// </returns>
		public static IBusinessCalendar Create(IBusinessCalendar[] calendars, string name)
		{
			return Create(calendars, name, Rule.Holidays);
		}

		/// <summary>
		/// Create a <see cref="JointCalendar"/> from multiple <see cref="IBusinessCalendar"/>s
		/// using the given combination rule.
		/// </summary>
		/// <remarks>
		/// This static method recursively constructs a <see cref="JointCalendar"/> from the passed
		/// in calendars. The result contains holidays from all calendars. 
		/// See the class constructor for more information.
		/// <para>
		/// You must at least pass in two calendars two join. 
		/// Passing in one calendar does not result in an exception. Instead the calendar
		/// is returned under it's original name.
		/// </para>
		/// </remarks>
		/// <param name="calendars">
		/// An <see cref="Array"/> of <see cref="IBusinessCalendar"/>s. 
		/// </param>
		/// <param name="name">
		/// Name of the combined calendar.
		/// </param>
		/// <param name="rule">
		/// The combination <see cref="Rule"/> of the two calendars.
		/// </param>
		/// <remarks>
		/// The calendar combination <see cref="Rule"/> is used to join
		/// calendars on common business days or holidays:
		/// <list type="table">
		/// <listheader>
		/// <term>Calendar combination <see cref="Rule"/>.</term>
		/// <description>Description</description>
		/// </listheader>
		/// <item>
		/// <term><c>Holidays</c></term>
		/// <description>
		/// The resulting calendar has holidays from all calendars. 
		/// That is, in other words, only common business days.
		/// </description>
		/// </item>
		/// <item>
		/// <term><c>BusinessDays</c></term>
		/// <description>
		/// The resulting calendar has business days from all calendars. 
		/// That is, in other words, only common holidays.
		/// </description>
		/// </item>
		/// </list>
		/// <para>
		///	To combine <strong>business days</strong> from Tokyo, London and New York 
		///	(that is: only account for common holidays) use the following example:
		///	<example>
		///	[C#]
		///	<code>
		///	using Stochastix.Business;
		///	using Stochastix.Business.Calendars;
		///	
		///	IBusinessCalendar jointCalendar = JointCalendar.Create( 
		///		new IBusinessCalendar[] {
		///			Tokyo.Instance, London.Instance, NewYork.Instance 
		///		}, "YEN/GBP/USD", JointCalendar.Rule.BusinessDays );
		///	</code>
		///	Regarding globalization settings, the resulting <c>jointCalendar</c> 
		///	behaves like the original <c>Tokyo</c> calendar.
		///	</example>
		///	</para>
		/// </remarks>
		/// <exception cref="ArgumentException">
		/// Thrown when an empty array has been given to <paramref name="calendars"/>.
		/// </exception>
		/// <returns>
		/// A calendar joined with the given rule, either having all holidays or only
		/// common holidays.
		/// </returns>
		public static IBusinessCalendar Create(IBusinessCalendar[] calendars,
			string name, Rule rule)
		{
			int l = calendars.Length;
			if( l == 0 )
				throw new ArgumentException(
					Strings.Get( typeof(JointCalendar), "CalJoinOne"), "calendars");
			if( l == 1 )
				return calendars[0];
			IBusinessCalendar cal = calendars[l-1];
			for(int i=l-2; i>0; i--)
				cal = new JointCalendar(calendars[i], cal, String.Empty, rule);
			return new JointCalendar(calendars[0], cal, name, rule);
		}
	}
}
