// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: ActualActualISMA.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:25:25  jens
 *  Initial import
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Utilities;
using Stochastix.Business.Resources;

namespace Stochastix.Business.DayCounters
{
	/// <summary>
	/// Actual/Actual (ISMA/Bond) day count convention.
	/// </summary>
	/// <seealso href="http://www.isda.org/c_and_a/pdf/mktc1198.pdf">
	/// ISDA: EMU and market conventions
	/// </seealso>
	[ RcsId("$Id: ActualActualISMA.cs,v 1.1.1.1 2003/02/01 18:25:25 jens Exp $") ]
	[ ComVisible(true), ClassInterface(ClassInterfaceType.None) ]
	[ FactoryItem("act/act (Bond)"), FactoryItem("act/act(b)") ]
	[ FactoryItem("act/act"), FactoryItem("actact") ]
	public sealed class ActualActualISMA : DayCounterBase, IDayCounter 
	{
		/// <summary>
		/// Parameterless COM constructor - use <see cref="Instance"/> member instead.
		/// </summary>
		/// <remarks>
		/// This constructor is provided for COM compatibility only.
		/// Use the static member <see cref="Instance"/> to get an object
		/// of this type.
		/// </remarks>
		[ Obsolete("Use the static Instance member in .NET applications.") ]
		public ActualActualISMA() : this("act/act (Bond)") 
		{}

		internal ActualActualISMA(String name) : base(name) 
		{}

		#region FactoryItem pattern

		[ Obsolete() ] // just to ignore the CS0618 warning below
		static ActualActualISMA()
		{
			Instance = new ActualActualISMA();	// CS0618
		}

		/// <summary>
		/// A static instance of this type.
		/// </summary>
		static public readonly ActualActualISMA Instance;
		
		#endregion

		override protected double YearFractionImpl(
			DateTime startDate, DateTime endDate, 
			DateTime refPeriodStart, DateTime refPeriodEnd) 
		{
			if( refPeriodStart.Ticks == 0) refPeriodStart = startDate;
			if( refPeriodEnd.Ticks == 0) refPeriodEnd = endDate;

			// normalize reference period
			refPeriodStart = refPeriodStart.Date;
			refPeriodEnd = refPeriodEnd.Date;

			if( ! ( refPeriodEnd > refPeriodStart && refPeriodEnd > startDate ) )
				throw new ArgumentException(
					// Invalid reference period: startDate: {0}, endDate: {1}, 
					// Reference period Start: {2}, Reference period end: {3}.
					String.Format( Strings.Get( GetType(), "DcRefDate"),
					new object[] { startDate, endDate, refPeriodStart, refPeriodEnd } ) 
					);

			// estimate roughly the length in months of a period
			int months = (int)(0.5+12*ActualDays(refPeriodStart, refPeriodEnd)/365.0);
			if( months == 0 )
				throw new ArgumentException(
					// TODO: i do not understand this error message
					// number of months does not divide 12 exactly
					Strings.Get( GetType(), "DcMonths12") );
			double period = (double)months/12.0;

			if (endDate <= refPeriodEnd) 
			{
				// here refPeriodEnd is a future (notional?) payment date
				if (startDate >= refPeriodStart)
					// here refPeriodStart is the last (maybe notional)
					// payment date.
					// refPeriodStart <= startDate <= endDate <= refPeriodEnd
					// [maybe the equality should be enforced, since
					// refPeriodStart < startDate <= endDate < refPeriodEnd
					// could give wrong results] ???
					return period*ActualDays(startDate,endDate) /
						ActualDays(refPeriodStart,refPeriodEnd);
				else 
				{
					// here refPeriodStart is the next (maybe notional)
					// payment date and refPeriodEnd is the second next
					// (maybe notional) payment date.
					// startDate < refPeriodStart < refPeriodEnd
					// AND endDate <= refPeriodEnd
					// this case is long first coupon

					// the last notional payment date
					DateTime previousRef = refPeriodStart.AddMonths(-months);
					if (endDate > refPeriodStart)
						return YearFractionImpl(startDate, refPeriodStart, previousRef,
							refPeriodStart) +
							YearFractionImpl(refPeriodStart, endDate, refPeriodStart,
							refPeriodEnd);
					else
						return YearFractionImpl(startDate,endDate,previousRef,refPeriodStart);
				}
			} 
			else 
			{
				// here refPeriodEnd is the last (notional?) payment date
				// startDate < refPeriodEnd < endDate AND refPeriodStart < refPeriodEnd
				if( refPeriodStart > startDate )
					throw new ArgumentException(
						// TODO: now it is: refPeriodStart <= startDate < refPeriodEnd < endDate
						// Invalid dates: startDate < refPeriodStart < refPeriodEnd < endDate.
						Strings.Get(GetType(), "DcInvRefDates") );

				// the part from startDate to refPeriodEnd
				double sum = YearFractionImpl(startDate, refPeriodEnd, refPeriodStart, refPeriodEnd);

				// the part from refPeriodEnd to endDate
				// count how many regular periods are in [refPeriodEnd, endDate],
				// then add the remaining time
				int i=0;
				DateTime newRefStart, newRefEnd;
				do 
				{
					newRefStart = refPeriodEnd.AddMonths(months*i);
					newRefEnd   = refPeriodEnd.AddMonths(months*(i+1));
					if (endDate < newRefEnd) 
					{
						break;
					} 
					else 
					{
						sum += period;
						i++;
					}
				} while (true);
				sum += YearFractionImpl(newRefStart,endDate,newRefStart,newRefEnd);
				return sum;
			}
		}

	}

}
