// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: DayCounterBase.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:25:25  jens
 *  Initial import
 *
 *  Revision 1.7  2002/08/30 18:21:10  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.6  2002/08/07 19:14:59  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.5  2002/07/24 17:04:24  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.4  2002/07/24 14:42:42  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Diagnostics;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Utilities;
using Stochastix.Business.Resources;

namespace Stochastix.Business.DayCounters
{

	/// <summary>
	/// This class provides methods for determining the length of a time
	/// period according to given market convention, both as a number
	/// of days and as a year fraction.
	/// </summary>
	/// <unittest>Stochastix.Business.Tests.TestDayCounters.TestActualActual</unittest>
	[ RcsId("$Id: DayCounterBase.cs,v 1.1.1.1 2003/02/01 18:25:25 jens Exp $") ] 
	[ ComVisible(false) ]
	public abstract class DayCounterBase : IDayCounter 
	{
		/// <summary>
		/// Conventions for act/n and 30/n types of day counters.
		/// </summary>
		protected enum Convention 
		{
			/// <summary>act/basis convention</summary>
			Actual, 
			/// <summary>30/basis convention (US)</summary>
			US, 
			/// <summary>30/basis convention (EU)</summary>
			EU, 
			/// <summary>30/basis convention (Italy)</summary>
			Italian
		};

		private String name;
		private Convention countConvention;
		private double basis;

		/// <summary>
		/// Constructor for act/n and 30/n types of day counters.
		/// </summary>
		/// <param name="name"></param>
		/// <param name="countConvention"></param>
		/// <param name="basis"></param>
		protected DayCounterBase(String name, 
			Convention countConvention, int basis ) 
		{
			this.name = name;
			this.countConvention = countConvention;
			this.basis = (double)basis;
		}

		/// <summary>
		/// Constructor for act/act types of day counters.
		/// </summary>
		/// <param name="name"></param>
		protected DayCounterBase(String name) 
		{
			this.name = name;
			this.countConvention = Convention.Actual;
			this.basis = 365.25; // just to have it initialized somehow
		}

		/// <summary>
		/// The literal name of this day counter.
		/// </summary>
		/// <returns></returns>
		override public String ToString()
		{
			return name;
		}

		/// <summary>
		/// The number of actual days between two dates.
		/// </summary>
		/// <param name="startDate"></param>
		/// <param name="endDate"></param>
		/// <returns></returns>
		internal protected static int ActualDays(DateTime startDate, DateTime endDate) 
		{
			return (endDate.Date-startDate.Date).Days;
		}

		/// <summary>
		/// Returns the number of days between two dates.
		/// </summary>
		/// <param name="startDate"></param>
		/// <param name="endDate"></param>
		/// <returns></returns>
		public int DayCount(DateTime startDate, DateTime endDate) 
		{
			if(countConvention==Convention.Actual)
				return ActualDays(startDate, endDate);

			// otherwise we count 30 somethings

			int dd1 = startDate.Day, dd2 = endDate.Day;
			int mm1 = startDate.Month, mm2 = endDate.Month;

			switch(countConvention) 
			{
				case Convention.US:
					if (dd2 == 31 && dd1 < 30) { dd2 = 1; mm2++; }
					break;

				case Convention.Italian:
					if (mm1 == 2 && dd1 > 27) dd1 = 30;
					if (mm2 == 2 && dd2 > 27) dd2 = 30;
					break;

				case Convention.EU:
					break;

				default:
					Trace.Fail("Unknown Option " + countConvention.ToString(), 
						"Assuming EU convention");
					break;

			}
			return 360*(endDate.Year-startDate.Year) + 30*(mm2-mm1-1) +
				Math.Max(0,30-dd1) + Math.Min(30,dd2);
		}

		/// <summary>
		/// Returns the period between two dates as a fraction of year.
		/// </summary>
		/// <param name="start"></param>
		/// <param name="end"></param>
		/// <returns></returns>
		public double YearFraction(DateTime start, DateTime end) 
		{
			return YearFraction(start, end, 
				// TODO: can we replace zero dates with start, end ??
				// see ActualActualISMA for a test on these 0 values
				// use CalendarBase.NullDate here???
				new DateTime(0), new DateTime(0));
		}
		
		/// <summary>
		/// Returns the period between two dates as a fraction of year.
		/// </summary>
		/// <param name="startDate"></param>
		/// <param name="endDate"></param>
		/// <param name="refPeriodStart"></param>
		/// <param name="refPeriodEnd"></param>
		/// <returns></returns>
		public double YearFraction(
			DateTime startDate, DateTime endDate, 
			DateTime refPeriodStart, DateTime refPeriodEnd) 
		{
			// normalize to 00:00 am
			startDate = startDate.Date;
			endDate = endDate.Date;

			if( startDate > endDate )
				throw new ArgumentException(
					// Invalid dates: startDate={0}, endDate={1}.
					Strings.Format(GetType(), "DcInvDates", startDate, endDate) 
				);

			if (startDate == endDate) return 0.0;

			return YearFractionImpl(startDate, endDate, 
				refPeriodStart.Date, refPeriodEnd.Date);
		}

		/// <summary>
		/// Returns the period between two dates as a fraction of year.
		/// </summary>
		/// <remarks>
		/// This is the default implementation, which implements year fractions
		/// on a fixed basis. act/act day counters need to override this method.
		/// </remarks>
		/// <param name="startDate"></param>
		/// <param name="endDate"></param>
		/// <param name="refPeriodStart"></param>
		/// <param name="refPeriodEnd"></param>
		/// <returns></returns>
		virtual protected double YearFractionImpl(DateTime startDate, DateTime endDate, 
			DateTime refPeriodStart, DateTime refPeriodEnd) 
		{
			return DayCount(startDate, endDate) / basis;
		}

	}
}

