// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: CumulativeNormalDistribution.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:42:10  jens
 *  Initial import
 *
 *  Revision 1.4  2002/12/05 15:57:52  stochastix
 *  synced with my current sandbox
 *
 *  Revision 1.3  2002/10/15 12:43:00  stochastix
 *  updated license
 *
 *  Revision 1.2  2002/08/28 15:35:51  stochastix
 *  changes due to FxCop
 *
 *  Revision 1.1.1.1  2002/08/15 11:44:10  stochastix
 *  Initial revision
 *
 *  Revision 1.2  2002/08/07 19:14:59  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.1  2002/07/26 00:25:00  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Utilities;
using Stochastix.Numerics.Resources;

namespace Stochastix.Numerics.Distributions
{
	/// <summary>
	/// Cumulative normal distribution class.
	/// </summary>
	/// <remarks>
	/// Given x it provides an approximation to the
	/// integral of the gaussian normal distribution:
	/// formula here ...
	/// 
	/// For this implementation see M. Abramowitz and I. Stegun,
	/// Handbook of Mathematical Functions,
	/// Dover Publications, New York (1972)
	/// </remarks>
	[ RcsId("$Id: CumulativeNormalDistribution.cs,v 1.1.1.1 2003/02/01 18:42:10 jens Exp $") ] 
	[ ComVisible(false) ]
	public class CumulativeNormalDistribution : IUnaryFunction
	{
		public CumulativeNormalDistribution() : this(0.0, 1.0)
		{}
		public CumulativeNormalDistribution(double average) 
			: this(average, 1.0)
		{}
		public CumulativeNormalDistribution(double average,
			double sigma)
		{
			if( sigma <= 0.0 )
				throw new ArgumentOutOfRangeException( "sigma", sigma,
					Strings.Format( GetType(), "DistSigmaGr0", sigma) );
			this.average = average;
			this.sigma = sigma;
		}

		private double average, sigma;
		private NormalDistribution gaussian = new NormalDistribution();

		// IUnaryFunction
		public double Value(double x)
		{

			// 23.12.2002 jT
			// copied from changes in QL: not sure if this is necessary or even makes sense
#if DEBUG
			if( (x >= average) && (2.0*average-x > average) )
				throw new InvalidOperationException( 
					Strings.Get( GetType(), "TODO: not a real number.") );
#endif
			if (x >= average) 
			{
				double xn = (x - average) / sigma;
				double k = 1.0 / (1.0 + gamma*xn);
				double temp = gaussian.Value(xn) * k *
					(a1 + k*(a2 + k*(a3 + k*(a4 + k*a5))));
				if (temp < precision) 
					return 1.0;
				temp = 1.0 - temp;
				if (temp < precision) 
					return 0.0;
				return temp;
			} 
			else 
			{
				return 1.0 - Value(2.0*average - x);
			}
		}

		/// <summary>
		/// A lightweight static delegate for average=0.0, sigma=1.0.
 		/// You can also instatiate this class and then use
		/// instance.Value as well.
		/// </summary>
		public static double Function(double x)
		{
			if (x >= 0.0) 
			{
				double k = 1.0 / (1.0 + gamma*x);
				double temp = NormalDistribution.Function(x) * k *
					(a1 + k*(a2 + k*(a3 + k*(a4 + k*a5))));
				if ( temp < precision ) 
					return 1.0;
				temp = 1.0 - temp;
				if ( temp < precision ) 
					return 0.0;
				return temp;
			} 
			else 
			{
				return 1.0 - Function(-x);
			}
		}

		static public double FunctionDerivative(double x)
		{
			return NormalDistribution.Function(x);
		}

		public double Derivative(double x)
		{
			double xn = (x - average) / sigma;
			return gaussian.Value(x);
		}

		private const double a1 =  0.319381530;
		private const double a2 = -0.356563782;
		private const double a3 =  1.781477937;
		private const double a4 = -1.821255978;
		private const double a5 =  1.330274429;

		private const double gamma     = 0.2316419;
		private const double precision = 1e-6;
	}
}
