// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: Matrix.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:42:09  jens
 *  Initial import
 *
 *  Revision 1.6  2002/12/05 15:57:51  stochastix
 *  synced with my current sandbox
 *
 *  Revision 1.5  2002/10/15 12:55:41  stochastix
 *  updated license
 *
 *  Revision 1.4  2002/08/28 15:41:41  stochastix
 *  changes due to FxCop
 *
 *  Revision 1.3  2002/08/28 15:00:05  stochastix
 *  merging work
 *
 *  Revision 1.2  2002/08/28 14:42:06  stochastix
 *  merging work
 *
 *  Revision 1.1.1.1  2002/08/15 11:44:10  stochastix
 *  Initial revision
 *
 *  Revision 1.2  2002/08/07 19:14:59  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using System.Globalization;
using System.Text;
using Stochastix.Numerics;
using Stochastix.Numerics.LA;
using Stochastix.Numerics.Resources;
using Stochastix.Utilities;

namespace Stochastix.Numerics
{
	/// <summary>
	/// A Matrix class using optimized BLAS and LAPACK libraries.
	/// </summary>
	[ RcsId("$Id: Matrix.cs,v 1.1.1.1 2003/02/01 18:42:09 jens Exp $") ] 
	[ ComVisible(false) ]
	[ Description("A Matrix class using optimized BLAS and LAPACK libraries.") ]
	public class Matrix : IFormattable, ICloneable
	{
		#region Constructors

		public Matrix(int rows, int columns)
		{
			this.rows = rows;
			this.columns = columns;
			this.offset = 0;
			this.ld = columns;
			this.data = new double[rows*columns];
		}

		public Matrix(int rows, int columns, double[] data)
			: this(rows, columns)
		{
			if(rows*columns > data.Length)
				throw new ArgumentException( 
					Strings.Get(GetType(), "MatNoData"), "data" );
			Array.Copy(data, data.GetLowerBound(0), 
				this.data, 0, this.data.Length);
		}

		public Matrix( double[,] data)
			: this(data.GetLength(0), data.GetLength(1))
		{
			unsafe 
			{
				fixed( double* d = &this.data[0], 
						   s = &data[data.GetLowerBound(0),data.GetLowerBound(1)])	
				{
					array_copy(d, s, this.data.Length);
				}
			}
		}

		public Matrix(Matrix other)
		{
			// we could also compact to columns=ld and offset=0 here!
			this.rows = other.rows;
			this.columns = other.columns;
			this.offset = other.offset;
			this.ld = other.ld;
			this.data = (double[])other.data.Clone();
		}

		#endregion

		#region Private data

		internal double[] data;
		private int rows, columns;
		internal int offset, ld;

		#endregion

		#region Properties and indexers

		public int Rows
		{
			get { return rows; }
		}

		public int Columns
		{
			get { return columns; }
		}

		public double this[int row, int column]
		{
			get
			{
				return data[offset+row*ld+column];
			}
			set
			{
				data[offset+row*ld+column] = value;
			}
		}

		public Vector this[int row]
		{
			get
			{
				// TODO: check args
				return new AttachedVector(data, Columns, row*ld, 1);
			}
		}

		#endregion

		#region Columns, Rows and Diagonal

		private class AttachedVector : Vector
		{
			public AttachedVector( double[] data, int count, int offset, int inc)
				: base(data, count, offset, inc)
			{}
		}

		public Vector Row(int i)
		{
			return this[i];
		}		

		public Vector Column(int i)
		{
			// check args
			return new AttachedVector(data, Rows, i, ld);
		}		

		public Vector Diagonal
		{
			get
			{
				return new AttachedVector(data, 
					Math.Min(Rows, Columns), 0, ld+1);
			}
		}		

		#endregion

		#region Matrix add/sub - prefer in-place operation

		public void Add( Matrix other) 
		{
			Add(other, 1.0);
		}

		public void Add( Matrix other, double scalar) 
		{
			if( this.Columns != other.Columns || this.Rows != other.Rows )
				throw new ArgumentException( Strings.Get( GetType(), "MatDimNeq"));

			if(this.ld == this.columns && other.ld == other.columns) 
				// add all rows in one call
				BLAS.axpy(rows*columns, scalar, 
					ref other.data[other.offset], 1, 
					ref this.data[this.offset], 1
					);
			else	
			{	
				// add each row separately
				int thisoff  = this.offset, 
					otheroff = other.offset;
				for(int i=rows; i!=0; i--) 
				{
					BLAS.axpy(columns, scalar, 
						ref other.data[otheroff], 1, 
						ref this.data[thisoff], 1
						);
					thisoff  += this.ld;
					otheroff += other.ld;
				}
			}
		}

		public static Matrix operator+(Matrix x, Matrix y) 
		{
			Matrix temp = x.Clone();
			temp.Add(y, 1.0);
			return temp;
		}

		public void Subtract(Matrix other)
		{
			Add(other, -1.0);
		}

		public static Matrix operator-(Matrix x, Matrix y)
		{
			Matrix temp = x.Clone();
			temp.Add(y, -1.0);
			return temp;
		}

		#endregion

		#region Matrix mult/div with scalar -- prefer in-place

		public void Multiply(double scalar)
		{
			if( ld == columns ) // continous array
			{
				if( scalar == 0.0 )
					Array.Clear(this.data, offset, rows*columns);
				else
					BLAS.scal(rows*columns, scalar, ref data[offset], 1);
			} 
			else 
			{
				for( int i=0; i<Rows; i++)
					Row(i).Multiply(scalar);
			}
		}
		public static Matrix operator*(Matrix matrix, double scalar)
		{
			Matrix temp = matrix.Clone();
			temp.Multiply(scalar);
			return temp;
		}
		public static Matrix operator*(double scalar, Matrix matrix)
		{
			Matrix temp = matrix.Clone();
			temp.Multiply(scalar);
			return temp;
		}
		public void Divide(double scalar)
		{
			Multiply(1.0/scalar);
		}
		public static Matrix operator/(Matrix matrix, double scalar)
		{
			return matrix*(1.0/scalar);
		}

		#endregion

		#region vectorial products -- prefer operator

		public static Vector operator*(Vector v, Matrix m)
		{
			if( m.rows != v.Count )
				throw new ArgumentException( Strings.Get( typeof(Matrix), "MatDimNeq"));
			Vector temp = new Vector(m.Rows);
			BLAS.gemv('T', m.Rows, m.Columns,
				1.0, ref m.data[m.offset], m.ld, 
				ref v.data[v.offset], v.inc, 0.0, 
				ref temp.data[0], temp.inc);
			return temp;
		}

		public static Vector operator*(Matrix matrixA, Vector vectorX)
		{
			if( matrixA.Columns != vectorX.Count )
				throw new ArgumentException( Strings.Get( typeof(Matrix), "MatDimNeq"));
			Vector temp = new Vector(matrixA.Rows);
			BLAS.gemv('N', matrixA.Rows, matrixA.Columns,
				1.0, ref matrixA.data[matrixA.offset], matrixA.ld, 
				ref vectorX.data[vectorX.offset], vectorX.inc, 0.0, 
				ref temp.data[0], temp.inc);
			return temp;
		}

		#endregion

		#region matrix products -- prefer AddProduct or operator*

		public void AddProduct(Matrix ma, Matrix mb)
		{
			AddProduct(ma, mb, 1.0, 1.0);
		}
		
		public void AddProduct(Matrix ma, Matrix mb, double alpha, double beta)
		{
			if( this.Rows != ma.Rows || this.Columns != mb.Columns ||
				ma.Columns != mb.Rows )
				throw new ArgumentException( Strings.Get( GetType(), "MatDimNeq"));
			// Matrix result(m1.rows(),m2.columns());
			BLAS.gemm('N', 'N',	this.rows, this.columns, ma.columns, 
				alpha, ref ma.data[ma.offset], ma.ld, 
				ref mb.data[mb.offset], mb.ld,
				beta, ref this.data[this.offset], this.ld);
		}

		public static Matrix operator*(Matrix ma, Matrix mb)
		{
			Matrix temp = new Matrix(ma.Rows, mb.Columns);
			temp.AddProduct(ma, mb, 1.0, 0.0); 
			return temp;
		}

		#endregion

		#region Misc. operations

		public static Matrix Transpose(Matrix m) 
		{
			int cols = m.Columns;
			int rows = m.Rows;
			Matrix temp = new Matrix(cols, rows);
			unsafe 
			{
				fixed( double* t = &temp.data[0] )
				{
					double* tp = t;
					for( int i=0; i<cols; i++ )
						for( int j=0; j<rows; j++ )
							*tp++ = m[j,i];	// TODO: = m.data[m.offset+row*m.ld+column]
				}

			}
			return temp;
		}

		/// <summary>
		/// Returns the square root of a real symmetric matrix.
		/// </summary>
		/// <param name="realSymmetricMatrix"></param>
		/// <returns></returns>
		public static Matrix Sqrt(Matrix realSymmetricMatrix)
		{
			//! eigenvalues smaller than tolerance are considered zero
			const double tolerance = 1e-15;
			SymmetricSchurDecomposition jd = 
				new SymmetricSchurDecomposition(realSymmetricMatrix);
			Matrix evectors = jd.Eigenvectors;	// do not copy -- consume!
			Vector evalues = jd.Eigenvalues;

			double maxEvTolerance = evalues.Max() * tolerance;
			int size = realSymmetricMatrix.Rows;
			Matrix D = new Matrix(size, size);
			Vector diagonal = D.Diagonal;
			for(int i = 0; i < size;i++)
			{
				if( Math.Abs(evalues[i]) <= maxEvTolerance)
					evalues[i] = 0.0;
				else if( evalues[i] < 0.0 )
					throw new ApplicationException( Strings.Get( typeof(Matrix),
						"TODO: Some eigenvalues are negative."));
				diagonal[i] = Math.Sqrt(evalues[i]);
			}

			// return evectors*D*Matrix.Transpose(evectors)
			//  - use gemm direct so we can skip transpose()
			//  - use D to buffer return matrix
			Matrix temp = evectors * D;
			BLAS.gemm('N', 'T',	size, size, size, 
				1.0, ref temp.data[temp.offset], temp.ld, 
				ref evectors.data[evectors.offset], evectors.ld,
				0.0, ref D.data[D.offset], D.ld);
			return D;
		}

		#endregion

//		#region Implementation (IDisposable)
//
//		public void Dispose()
//		{
//			data = null;
//		}
//
//		#endregion

		#region Type casts

		// making this implicit brings us foreach() and stuff
		// but remember the europython motto: 
		//          explicit is better than implicit
		public static explicit operator double[,](Matrix m)
		{
			double[,] arr = new double[m.Rows,m.Columns];
			unsafe 
			{
				fixed( double* ap = &arr[0,0], mp = &m.data[m.offset]) 
				{
					if( m.ld == m.Columns)
						array_copy(ap, mp, arr.Length);
						//  BLAS.copy(arr.Length, mp, 1, ap, 1)
					else
					{
						double* app=ap, mpp=mp;
						int gap = m.ld - m.Columns;
						for(double* end=ap+arr.Length; app<end; mpp+=gap)
							for(double* rend=app+m.ld; app<rend;)
								*app++ = *mpp++;
					}
				}
			}
			return arr;
		}

		#endregion

		#region Implementation (IFormattable and ToString)

		override public string ToString()
		{
			return ToString("f6", CultureInfo.InvariantCulture.NumberFormat);
		}

		public string ToString( string format, IFormatProvider formatProvider)
		{
			StringBuilder s = new StringBuilder(this.Rows * this.Columns * 15);
			s.AppendFormat("{{ // {0}x{1} {2}\n", 
				this.Rows, this.Columns, base.ToString());

			for(int i=0; i< this.Rows;)
			{
				s.Append("  { ");
				for(int j=0; j<this.Columns;)
				{
					s.Append( this[i,j].ToString(format, formatProvider) );
					if( ++j < this.Columns ) 
						s.Append(", ");
				}
				if( ++i < this.Rows ) s.Append(" },\n");
				else s.Append(" }\n");
			}

			s.Append("}\n");
			return s.ToString();
		}

		#endregion

		#region private helpers

		static private unsafe void array_copy(double* d, double* s, int length) 
		{
			for(;length!=0;length--)
				*d++ = *s++;
		}

		#endregion
		
		#region Implementation (ICloneable)

		public Matrix Clone()
		{
			return new Matrix(this);
		}

		object ICloneable.Clone()
		{
			return this.Clone();
		}

		#endregion

	}
}
