// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: ISolver1D.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:42:12  jens
 *  Initial import
 *
 *  Revision 1.4  2002/12/05 15:57:52  stochastix
 *  synced with my current sandbox
 *
 *  Revision 1.3  2002/10/15 12:44:45  stochastix
 *  updated license
 *
 *  Revision 1.2  2002/08/28 15:36:34  stochastix
 *  changes due to FxCop
 *
 *  Revision 1.1.1.1  2002/08/15 11:44:11  stochastix
 *  Initial revision
 *
 *  Revision 1.4  2002/08/07 19:15:00  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.3  2002/07/24 17:04:24  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.2  2002/07/24 14:42:41  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Utilities;

namespace Stochastix.Numerics.Solvers1D
{

	/// <summary>
	/// Interface for 1-D solvers.
	/// </summary>
	[ RcsId("$Id: ISolver1D.cs,v 1.1.1.1 2003/02/01 18:42:12 jens Exp $") ] 
	[ ComVisible(true) ]
	[ Description("Interface for 1-D solvers.") ]
	[ InterfaceTypeAttribute(ComInterfaceType.InterfaceIsDual) ]
	public interface ISolver1D
	{
		/// <summary>
		/// A stringified representation of the solver.
		/// </summary>
		/// <returns>A String representing the object.</returns>
		[
		Description("A stringified representation of this object."),
		DispId(0)	// DISPID_VALUE
		]
		string ToString();

		/// <summary>
		/// This method returns the zero of the <see cref="IObjectiveFunction"/>
		/// <pramref name="objectiveFunction"/>, determined with the given accuracy.
		/// </summary>
		/// <remarks>
		/// <i>x</i> is considered a zero if <i>|f(x)| &lt; accuracy</i>.
		/// This method contains a bracketing routine to which an initial
		/// guess must be supplied as well as a step used to scan the range
		/// of the possible bracketing values.
		/// </remarks>
		/// <param name="objectiveFunction">The <see cref="IObjectiveFunction"/>.</param>
		/// <param name="xAccuracy">Given accuracy.</param>
		/// <param name="guess">Initial guess used to scan the range
		/// of the possible bracketing values.</param>
		/// <param name="step">Initial step used to scan the range
		/// of the possible bracketing values.</param>
		/// <returns>The zero of the objective function.</returns>
		/// <exception cref="ApplicationException">
		/// Thrown when a bracket is not found after the maximum
		/// number of evaluations (see <see cref="MaxEvaluations"/>).
		/// </exception>
		[ Description("This method returns the zero of the ObjectiveFunction f.") ]
		double Solve( IObjectiveFunction objectiveFunction, 
			double xAccuracy, double guess, double step);

		/// <summary>
		/// This method returns the zero of the <see cref="IObjectiveFunction"/>
		/// <pramref name="objectiveFunction"/>, determined with the given accuracy.
		/// </summary>
		/// <remarks>
		/// <i>x</i> is considered a zero if <i>|f(x)| &lt; accuracy</i>.
		/// An initial guess must be supplied, as well as two values which 
		/// must bracket the zero (i.e., either 
		/// <i>f(x<sub>min</sub>) &gt; 0</i> &amp;&amp; 
		/// <i>f(x<sub>max</sub>) &lt; 0</i>, or 
		/// <i>f(x<sub>min</sub>) &lt; 0</i> &amp;&amp; 
		/// <i>f(x<sub>max</sub>) &gt; 0</i> must be true). 
		/// </remarks>
		/// <param name="objectiveFunction">The <see cref="IObjectiveFunction"/>.</param>
		/// <param name="xAccuracy">Given accuracy.</param>
		/// <param name="guess">Initial guess used to scan the range
		/// of the possible bracketing values.</param>
		/// <param name="xMin">Lower bracket.</param>
		/// <param name="xMax">Upper bracket.</param>
		/// <returns>The zero of the objective function.</returns>
		/// <exception cref="ApplicationException">
		/// Thrown when a bracket is not found after the maximum
		/// number of evaluations (see <see cref="MaxEvaluations"/>).
		/// </exception>
		[ Description("This method returns the zero of the ObjectiveFunction f.") ]
		double Solve( IObjectiveFunction objectiveFunction, double xAccuracy,
			double guess, double xMin, double xMax);


		// TODO: add a "BracketedSolve" for COM export?


		/// <summary>
		/// Maximum number of function evaluations for the bracketing routine. 
		/// An Error is thrown if a bracket is not found after this number of 
		/// evaluations.
		/// </summary>
		[ Description("Maximum number of function evaluations for the bracketing routine.") ]
		int MaxEvaluations
		{
			get; set;
		}
		
		/// <summary>
		/// The lower bound for the function domain.
		/// </summary>
		[ Description("The lower bound for the function domain.") ]
		double LowerBound
		{
			get; set;
		}

		/// <summary>
		/// The upper bound for the function domain.
		/// </summary>
		[ Description("The upper bound for the function domain.") ]
		double UpperBound
		{
			get; set;
		}

	};

}
