// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: Newton.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:42:12  jens
 *  Initial import
 *
 *  Revision 1.4  2002/12/05 15:57:52  stochastix
 *  synced with my current sandbox
 *
 *  Revision 1.3  2002/10/15 12:44:45  stochastix
 *  updated license
 *
 *  Revision 1.2  2002/08/28 15:36:34  stochastix
 *  changes due to FxCop
 *
 *  Revision 1.1.1.1  2002/08/15 11:44:11  stochastix
 *  Initial revision
 *
 *  Revision 1.3  2002/07/24 17:04:24  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.2  2002/07/24 14:42:41  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Utilities;
using Stochastix.Numerics.Resources;

namespace Stochastix.Numerics.Solvers1D
{
	/// <summary>
	/// Newton 1-D solver.
	/// </summary>
	/// <remarks>
	///	<para>
	///	The implementation of the algorithms was inspired by 
	///	<see href="http://www.library.cornell.edu/nr/bookcpdf.html">Numerical Recipes in C</see>, 
	///	2nd edition, Cambridge University Press, Teukolsky, Vetterling, Flannery - 
	///	<see href="http://www.library.cornell.edu/nr/bookcpdf/c9-2.pdf">
	///	Chapter 9.4 Newton-Raphson Method Using Derivative
	///	</see>.
	///	</para>
	/// </remarks>
	///	<seealso href="http://www.library.cornell.edu/nr/bookcpdf.html">
	///	Numerical Recipes in C (free online book)
	///	</seealso>
	[ RcsId("$Id: Newton.cs,v 1.1.1.1 2003/02/01 18:42:12 jens Exp $") ] 
	[ ComVisible(true) ]
	[ ClassInterface(ClassInterfaceType.None) ]
	public sealed class Newton : Solver1D, ISolver1D
	{
		/// <summary>
		/// Implementation of the actual code to search for the zeroes of 
		/// the <see cref="IObjectiveFunction"/>.
		/// </summary>
		/// <remarks>
		/// It assumes that:
		/// <list type="bullet">
		/// <item>
		///   <description>
		///		<see cref="Solver1D.xMin"/> and <see cref="Solver1D.xMax"/> form a valid bracket; 
		///	  </description>
		/// </item>
		/// <item>
		///   <description>
		///     <see cref="Solver1D.fxMin"/> and <see cref="Solver1D.fxMax"/> contain the values of the
		///		function in <see cref="Solver1D.xMin"/> and <see cref="Solver1D.xMax"/>; 
		///	  </description>
		/// </item>
		/// <item>
		///   <description>
		///     <see cref="Solver1D.root"/> was initialized to a valid initial guess.
		///	  </description>
		/// </item>
		/// </list>
		/// </remarks>
		/// <param name="objectiveFunction">The <see cref="IObjectiveFunction"/>.</param>
		/// <param name="xAccuracy">Given accuracy.</param>
		/// <returns>The zero of the objective function.</returns>
		/// <exception cref="ApplicationException">
		/// Thrown when a bracket is not found after the maximum
		/// number of evaluations (see <see cref="Solver1D.MaxEvaluations"/>).
		/// </exception>
		/// <exception cref="NotImplementedException">
		/// Thrown by the <see cref="ObjectiveFunction"/> base class
		/// when the function's derivative has not been implemented.
		/// </exception>
		override protected double SolveImpl(IObjectiveFunction objectiveFunction, 
			double xAccuracy)
		{
            while (evaluationNumber++ < MaxEvaluations) {
				double froot  = objectiveFunction.Value(root);
				double dfroot = objectiveFunction.Derivative(root);
				// the objectiveFunction throws  System.NotImplementedException
				// QL_REQUIRE(dfroot != Null<double>(), "Newton requires function's derivative");

				double dx = froot/dfroot;
                root -= dx;
                // jumped out of brackets, switch to NewtonSafe
                if ( (xMin-root) * (root-xMax) < 0.0 ) {
                    ISolver1D newtonSafe = new NewtonSafe();
                    newtonSafe.MaxEvaluations -= evaluationNumber;
                    return newtonSafe.Solve(objectiveFunction, xAccuracy, 
						root+dx, xMin, xMax);
                }
                if (Math.Abs(dx) < xAccuracy)
                    return root;
            }

			throw new ApplicationException( Strings.Format(
				GetType(), "SlvMaxEval", MaxEvaluations)
				);

		}
	}
}
