// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: Accumulator.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:42:13  jens
 *  Initial import
 *
 *  Revision 1.2  2002/12/05 15:57:48  stochastix
 *  synced with my current sandbox
 *
 *  Revision 1.1  2002/10/15 12:50:09  stochastix
 *  moved from QuantLib.NET
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Collections;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Numerics.Resources;
using Stochastix.Numerics.Distributions;
using Stochastix.Utilities;


namespace Stochastix.Numerics.Statistics
{
	/// <summary>
	/// Statistics accumulator tool.
	/// </summary>
	/// <remarks>
	/// It can accumulate a set of data and return statistic quantities
	/// as mean, variance, std. deviation, skewness, and kurtosis.
	/// </remarks>
	[ RcsId("$Id: Accumulator.cs,v 1.1.1.1 2003/02/01 18:42:13 jens Exp $") ] 
	[ ComVisible(false) ]
	public class Accumulator
	{
		public Accumulator()
		{
			// Reset();
		}

		private double min = Double.MaxValue;
		private double max = Double.MinValue;
		private int sampleNumber; // = 0;
		private double sampleWeight; //  = 0.0;
		private double sum; //  = 0.0;
		private double quadraticSum; //  = 0.0;
		private double downsideQuadraticSum; //  = 0.0;
		private double cubicSum; //  = 0.0;
		private double fourthPowerSum; //  = 0.0;

		/// <summary>
		/// Resets the data to a null set.
		/// </summary>
		public void Reset() 
		{
			min = Double.MaxValue;
			max = Double.MinValue;
			sampleNumber = 0;
			sampleWeight = 0.0;
			sum = 0.0;
			quadraticSum = 0.0;
			downsideQuadraticSum = 0.0;
			cubicSum = 0.0;
			fourthPowerSum = 0.0;
		}

		#region Adder

		/// <summary>
		/// Adds a datum to the set, weight is assumed to be 1.
		/// </summary>
		/// <param name="value"></param>
		public void Add(double value)
		{
			this.Add(value, 1.0);
		}

		/// <summary>
		/// Adds a weighted datum to the set.
		/// </summary>
		/// <param name="value"></param>
		/// <param name="weight">Weights must be positive or null.</param>
		public void Add(double value, double weight) 
		{
			if (weight<0.0) 
				throw new ArgumentOutOfRangeException( "weight", weight,
					Strings.Format(GetType(), "StatAddNeg", weight)
				);
			if(sampleNumber == int.MaxValue)
				throw new OverflowException(
					Strings.Format(GetType(), "StatMax", int.MaxValue)
				);

			sampleNumber++;
			sampleWeight += weight;

			double temp = weight*value;
			sum += temp;
			temp *= value;
			quadraticSum += temp;
			downsideQuadraticSum += value < 0.0 ? temp : 0.0;
			temp *= value;
			cubicSum += temp;
			temp *= value;
			fourthPowerSum += temp;
			min=Math.Min(value, min);
			max=Math.Max(value, max);
		}

		public void AddRange(double[] d)
		{
			for( int i=0; i<d.Length; i++)
				this.Add(d[i], 1.0);		// unsafe=faster?
		}

		/// <summary>
		/// Adds a weighted sample to the set.
		/// </summary>
		public void Add(Sample sample) 
		{
			this.Add((double)sample, sample.weight);
		}

		public void AddRange(Sample[] samples)
		{
			for( int i=0; i<samples.Length; i++)
				this.Add((double)samples[i], samples[i].weight);	// unsafe=faster?
		}

		/// <summary>
		/// Adds a sequence of data (doubles or Samples) to the set.
		/// See the other overloads for faster methods on arrays.
		/// </summary>
		/// <param name="sequence">A sequence of Samples or doubles.</param>
		public void AddRange(IEnumerable sequence) 
		{
			IEnumerator it = sequence.GetEnumerator();
			it.MoveNext();
			if( it.Current is Sample) 
			{
				foreach( Sample sample in sequence )
					this.Add((double)sample, sample.weight);
			} 
			else // assume doubles...
			{
				foreach( double d in sequence )
					this.Add(d, 1.0);
			}
		}

		#endregion

		#region Statistics

		/// <summary>
		/// Number of samples collected.
		/// </summary>
		public int Count
		{
			get { return sampleNumber; }
		}

		/// <summary>
		/// Sum of data weights.
		/// </summary>
		public double WeightSum  
		{
			get { return sampleWeight; }
		}

		/// <summary>
		/// Returns the mean.
		/// </summary>
		/// <latex>
		/// Defined as 
		/// \f[ \langle x \rangle = \frac{\sum w_i x_i}{\sum w_i}. \f]
		/// </latex>
		public double Mean 
		{
			get 
			{
				if(sampleNumber <= 0)
					throw new InvalidOperationException(
						Strings.Get(GetType(), "StatEmpty")
						);
				if(sampleWeight <= 0.0)
					throw new InvalidOperationException(
						Strings.Get(GetType(), "StatWeight0")
					);
				return sum/sampleWeight;
			}
		}

		/// <summary>
		/// The variance.
		/// </summary>
		/// <latex>
		/// defined as
		/// \f[ 
		///		\frac{N}{N-1} \left\langle \left(
		///		 x-\langle x \rangle \right)^2 \right\rangle. 
		///	\f]
		/// </latex>
		public double Variance
		{
			get
			{
				if(sampleNumber <= 0)
					throw new InvalidOperationException(
						Strings.Get(GetType(), "StatEmpty")
						);
				if(sampleWeight <= 0.0)
					throw new InvalidOperationException(
						Strings.Get(GetType(), "StatWeight0")
						);

				double v = (sampleNumber/(sampleNumber-1.0)) *
					(quadraticSum - sum*sum/sampleWeight)/sampleWeight;

				if (Math.Abs(v) <= 1.0e-6) 
					v = 0.0;
				else if(v < 0.0)
					throw new ArithmeticException(
						Strings.Format(GetType(), "StatNegVar", v)
						);
				return v;
			}
		}

		/// <summary>
		/// The standard deviation sigma.
		/// </summary>
		/// <latex>
		/// Standard deviation \f$ \sigma \f$, defined as the
		/// square root of the variance.
		/// </latex>
		public double StandardDeviation
		{
			get { return Math.Sqrt(Variance); }
		}
		
		/// <summary>
		/// The downside variance.
		/// </summary>
		/// <latex>
		/// Defined as
		/// \f[ \frac{N}{N-1} \times \frac{ \sum_{i=1}^{N}
		/// \theta \times x_i^{2}}{ \sum_{i=1}^{N} w_i} \f],
		/// where \f$ \theta \f$ = 0 if x > 0 and \f$ \theta \f$ =1 if x &lt;0
		/// </latex>
		public double DownsideVariance
		{
			get
			{
				if(sampleNumber <= 0)
					throw new InvalidOperationException(
						Strings.Get(GetType(), "StatEmpty")
						);
				if(sampleWeight <= 0.0)
					throw new InvalidOperationException(
						Strings.Get(GetType(), "StatWeight0")
						);

				return sampleNumber/(sampleNumber-1.0)*
					downsideQuadraticSum /sampleWeight;
			}
		}

		/// <summary>
		/// Downside deviation, defined as the square root of the 
		/// downside variance.
		/// </summary>
		public double DownsideDeviation
		{
			get { return Math.Sqrt(DownsideVariance); }
		}

		/// <summary>
		/// The error estimate epsilon, defined as the 
		/// square root of the ratio of the variance to 
		/// the number of samples.
		/// </summary>
		public double ErrorEstimate 
		{
			get { return Math.Sqrt(Variance/sampleNumber); }
		}

		/// <summary>
		/// Skewness.
		/// </summary>
		/// <latex>
		/// defined as
		/// \f[ 
		///		\frac{N^2}{(N-1)(N-2)} \frac{\left\langle \left(
		///		x-\langle x \rangle \right)^3 \right\rangle}{\sigma^3}. 
		///	\f]
		/// </latex>
		/// <remarks>
		/// The above evaluates to 0.0 for a Gaussian distribution.
		/// </remarks>
		public double Skewness
		{
			get
			{
				if(sampleNumber <= 2)
					throw new InvalidOperationException(
						Strings.Get(GetType(), "StatSkew2")
						);

				double s = StandardDeviation;
				if (s==0.0) return 0.0;
				double m = Mean;

				return sampleNumber*sampleNumber/
					((sampleNumber-1.0)*(sampleNumber-2.0)*s*s*s)*
					(cubicSum-3.0*m*quadraticSum+2.0*m*m*sum)/sampleWeight;
			}
		}

		/// <summary>
		/// Excess kurtosis.
		/// </summary>
		/// <latex>
		/// defined as
		/// \f[ \frac{N(N+1)}{(N-1)(N-2)(N-3)}
		/// \frac{\left\langle \left( x-\langle x \rangle \right)^4
		/// \right\rangle}{\sigma^4} - \frac{3(N-1)^2}{(N-2)(N-3)}. \f]
		/// </latex>
		/// <remarks>
		/// The above evaluates to 0 for a Gaussian distribution.
		/// </remarks>
		public double Kurtosis
		{
			get
			{
				if(sampleNumber <= 3)
					throw new InvalidOperationException(
						Strings.Get(GetType(), "StatKurt3")
						);

				double m = Mean;
				double v = Variance;

				if (v==0)
					return  - 3.0*(sampleNumber-1.0)*(sampleNumber-1.0) /
						((sampleNumber-2.0)*(sampleNumber-3.0));

				return sampleNumber*sampleNumber*(sampleNumber+1.0) /
					((sampleNumber-1.0)*(sampleNumber-2.0) *
					(sampleNumber-3.0)*v*v) *
					(fourthPowerSum - 4.0*m*cubicSum + 6.0*m*m*quadraticSum -
					3.0*m*m*m*sum)/sampleWeight -
					3.0*(sampleNumber-1.0)*(sampleNumber-1.0) /
					((sampleNumber-2.0)*(sampleNumber-3.0));
			}
		}

		/// <summary>
		/// The minimum sample value.
		/// </summary>
		public double Min
		{
			get 
			{
				if(sampleNumber <= 0)
					throw new InvalidOperationException(
						Strings.Get(GetType(), "StatEmpty")
						);
				return min;
			}
		}

		/// <summary>
		/// The maximum sample value.
		/// </summary>
		public double Max
		{
			get 
			{
				if(sampleNumber <= 0)
					throw new InvalidOperationException(
						Strings.Get(GetType(), "StatEmpty")
						);
				return max;
			}
		}

		#endregion

	}
}
