// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: TestDistributions.cs,v $
 *  Revision 1.1.1.1  2003/02/01 18:42:13  jens
 *  Initial import
 *
 *  Revision 1.3  2002/12/05 15:57:51  stochastix
 *  synced with my current sandbox
 *
 *  Revision 1.2  2002/10/15 12:45:28  stochastix
 *  updated license
 *
 *  Revision 1.1  2002/08/28 15:43:36  stochastix
 *  moved into dotNum module
 *
 *  Revision 1.1.1.1  2002/08/15 11:45:37  stochastix
 *  Initial revision
 *
 *  Revision 1.1  2002/08/07 19:15:21  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

namespace Stochastix.Numerics.Tests
{
	using System;
	using System.Collections;
	using NUnit.Framework;
	using Stochastix.Numerics.Distributions;


	/// <summary>
	/// 
	/// </summary>
	public class TestDistributions: TestCase 
	{
		/// <summary>
		/// Just a constructor. I think you can pass the name of the
		/// test (==method) to run.
		/// </summary>
		/// <param name="name">Name of the test to run.</param>
		public TestDistributions(String name): base(name) {}

		// declare private members here used for test setup

		/// <summary>
		/// Initialize this test case.
		/// </summary>
		protected override void SetUp() 
		{
			x = new Vector(N, xMin, h);
			y = x.Apply( new UnaryFunction(this.gaussian));
			yDerivative = x.Apply( 
				new UnaryFunction(this.gaussianDerivative));
			yIntegrated = x.Apply( cum );
		}

		private const double average = 0.0;
		private const double sigma = 1.0;
		private const double xMin = average - 4*sigma;
		private const double xMax = average + 4*sigma;
		private const int N = 10001;
		private const double h = (xMax-xMin)/(N-1);
		private Vector x, y, yDerivative, yIntegrated;

		/// <summary>
		/// Free all you resources.
		/// </summary>
		protected override void TearDown()
		{
			x = y = null;
			yIntegrated = null;
			yDerivative = null;
		}

		private double gaussian(double x)
		{
			double normFact = sigma * Math.Sqrt(2*Math.PI);
			double dx = x-average;
			return Math.Exp(-dx*dx/(2.0*sigma*sigma))/normFact;
		}

		private double gaussianDerivative(double x)
		{
			double normFact = sigma*sigma*sigma*Math.Sqrt(2*Math.PI);
			double dx = x-average;
			return -dx*Math.Exp(-dx*dx/(2.0*sigma*sigma))/normFact;
		}

		private double norm(Vector f, double h)
		{
			double sum = 0.0;
			foreach (double d in f) 
				sum += d*d;
			// numeric integral of f^2
			double res = h* ( sum - 0.5*f[0]*f[0] - 
				0.5*f[f.Count-1]*f[f.Count-1] );
			return Math.Sqrt(res);
		}

		private NormalDistribution normal = 
			new NormalDistribution(average, sigma);
		private CumulativeNormalDistribution cum = 
			new CumulativeNormalDistribution(average, sigma);
		private InvCumulativeNormalDistribution invCum = 
			new InvCumulativeNormalDistribution(average, sigma);
		private InvCumulativeNormalDistribution2 invCum2 = 
			new InvCumulativeNormalDistribution2(average, sigma);

		public void TestGaussian()
		{
			Vector yTemp = x.Apply( normal );
			yTemp.Subtract(y);
			Assertion.Assert( norm(yTemp,h) <= 1.0e-16);
		}

		public void TestGaussianDelegate()
		{
			NormalDistribution normal = new NormalDistribution();
			foreach( double xi in x)
				Assertion.Assert( Math.Abs( 
					NormalDistribution.Function(xi) - 
					normal.Value(xi) ) <= 1.0e-16 );
		}

		public void TestGaussianDerivative()
		{
			Vector yTemp = x.Apply(
				new UnaryFunction( normal.Derivative));
			yTemp.Subtract(yDerivative);
			Assertion.Assert( norm(yTemp, h) <= 1.0e-16);
		}

		public void TestCumulativeDerivative()
		{
			Vector yTemp = x.Apply(
				new UnaryFunction( cum.Derivative));
			yTemp.Subtract(y);
			Assertion.Assert( norm(yTemp, h) <= 1.0e-16);
		}

		public void TestCumulativeDelegate()
		{
			CumulativeNormalDistribution cum = 
				new CumulativeNormalDistribution();
			foreach( double xi in x)
				Assertion.Assert( Math.Abs( 
					CumulativeNormalDistribution.Function(xi) - 
					cum.Value(xi) ) <= 1.0e-16 );
		}

		public void TestInverseCumulative()
		{
			Vector yTemp = yIntegrated.Apply( invCum );
			yTemp.Subtract(x);
			Assertion.Assert( norm(yTemp, h) <= 1.0e-4);
		}

		public void TestInverseCumulative2()
		{
			Vector yTemp = yIntegrated.Apply( invCum2 );
			yTemp.Subtract(x);
			Assertion.Assert( norm(yTemp, h) <= 1.0e-3);
		}
	}
}
