// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: FloatingRateCoupon.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:26  jens
 *  Initial import
 *
 *  Revision 1.6  2002/08/30 18:21:10  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.5  2002/08/07 19:15:00  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.4  2002/07/24 17:04:26  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.3  2002/07/24 14:42:42  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.QuantLib.Indexes;
using Stochastix.Business;
using Stochastix.Business.Calendars;
using Stochastix.Utilities;

namespace Stochastix.QuantLib.CashFlows
{
	/// <summary>
	/// Coupon at par on a term structure.
	/// </summary>
	[ RcsId("$Id: FloatingRateCoupon.cs,v 1.1.1.1 2003/02/01 19:01:26 jens Exp $") ] 
	[ ComVisible(false) ]
	public class FloatingRateCoupon : Coupon, IFloatingRateCoupon
	{

		/// <summary>
		/// Construct a floating rate coupon. The coupon does not 
		/// roll the payment date which must already be a business day.
		/// </summary>
		/// <param name="nominal">Nominal</param>
		/// <param name="paymentDate">A pre-rolled business day.</param>
		/// <param name="index"></param>
		/// <param name="fixingDays"></param>
		/// <param name="spread"></param>
		/// <param name="startDate"></param>
		/// <param name="endDate"></param>
		public FloatingRateCoupon(
			double nominal, DateTime paymentDate, 
			// CHANGED the order of params in the constructor
			IXibor index, int fixingDays, double spread,
			DateTime startDate, DateTime endDate)
			: this(nominal, paymentDate, index, fixingDays, spread,
				startDate, endDate,	
				CalendarBase.NullDate, CalendarBase.NullDate)
		{}
		
		/// <summary>
		/// Construct a floating rate coupon. The coupon does not 
		/// roll the payment date which must already be a business day.
		/// </summary>
		/// <param name="nominal">Nominal</param>
		/// <param name="paymentDate">A pre-rolled business day.</param>
		/// <param name="index"></param>
		/// <param name="fixingDays"></param>
		/// <param name="spread"></param>
		/// <param name="startDate"></param>
		/// <param name="endDate"></param>
		/// <param name="refPeriodStart"></param>
		/// <param name="refPeriodEnd"></param>
		public FloatingRateCoupon(
			double nominal, DateTime paymentDate,
			// CHANGED the order of params in the constructor
			IXibor index, int fixingDays, double spread,
			DateTime startDate, DateTime endDate,
			DateTime refPeriodStart, DateTime refPeriodEnd )
			: base(nominal, paymentDate, startDate, endDate,
				refPeriodStart, refPeriodEnd)
		{
			this.index = index;
			this.fixingDays = fixingDays;
			this.spread = spread;
			RegisterWith(index);
		}

		private IXibor index;
		private int fixingDays;
		private double spread;

		override public string ToString()
		{
			return string.Format(
				"FloatingRateCoupon {0}; index={1}; spread={2:P}",
				base.ToString(), index, spread
				);
		}

		public IXibor Index
		{
			get { return index; }
		}

		public int FixingDays
		{
			get { return fixingDays; }
		}

		public double Spread
		{
			get { return spread; }
		}

		virtual public double Fixing
		{
			get { return Amount / (Nominal*AccrualPeriod); }
		}

		override public IDayCounter DayCounter
		{
			get	{ return index.TermStructure.DayCounter; }
		}

		override public double AccruedAmount(DateTime date)
		{
			date = date.Date;	// normalize
			if (date <= AccrualStartDate || date > /*payment*/Date) 
			{
				return 0.0;
			} 
			else 
			{
				return Nominal * Fixing * DayCounter.YearFraction(
					AccrualStartDate,
					date <= AccrualEndDate ? date : AccrualEndDate, // MIN()
					refPeriodStart, refPeriodEnd
					);
			}
		}

		override public double Amount
		{
			get
			{
				// QL_REQUIRE(!termStructure.isNull(),
				//	"null term structure set to par coupon");
				DateTime todaysDate = index.TermStructure.TodaysDate;
				DateTime fixingDate = index.Calendar.Advance(
					AccrualStartDate, -fixingDays, TimeUnit.Days,
					RollingConvention.Preceding);
//
//				// TODO: I think the following should be handled by 
//				//	Xibor.Fixing which already has similar code
//
				if (fixingDate < todaysDate) 
				{
					// must have been fixed
					return (XiborManager.History.Fixing(
						index.FamilyName, index.Tenor.ToString(), fixingDate
						) + spread) * AccrualPeriod * Nominal;;
				}

				if (fixingDate == todaysDate) 
				{
					// might have been fixed
					try 
					{
						return (XiborManager.History.Fixing(
							index.FamilyName, index.Tenor.ToString(), fixingDate
							) + spread) * AccrualPeriod * Nominal;
					} 
					catch 
					{
						// fall through and forecast
					}
				}

				DateTime fixingValueDate = index.Calendar.Advance(
					fixingDate, index.SettlementDays, TimeUnit.Days,
					// TODO: could be index.RollingConvention
					RollingConvention.Following);  

				// rollback fixing days
				// convention changed from Preceding to Following following changes
				// in QuantLib (2002-12-23)
				DateTime endDate = index.Calendar.Advance(AccrualEndDate, -fixingDays, 
					TimeUnit.Days, RollingConvention.Following);
				// advance settlement days
				endDate = index.Calendar.Advance(endDate, index.SettlementDays, 
					TimeUnit.Days, RollingConvention.Following);

				double startDiscount = 
					index.TermStructure.Discount(fixingValueDate, false);
				double endDiscount = 
					index.TermStructure.Discount(endDate, false);
				return ( (startDiscount/endDiscount-1.0) +
							spread * AccrualPeriod			) * Nominal;
			}
		}
	}
}


