// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: FiniteDifferenceModel.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:28  jens
 *  Initial import
 *
 *  Revision 1.4  2002/10/15 13:19:55  stochastix
 *  now using dotNum
 *
 *  Revision 1.3  2002/08/30 18:13:01  stochastix
 *  small changes for DotNum and license
 *
 *  Revision 1.2  2002/08/15 18:14:21  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.1  2002/08/07 19:14:59  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Numerics;
using Stochastix.Utilities;

namespace Stochastix.QuantLib.FiniteDifferences
{
	/// <summary>
	/// Generic finite difference model.
	/// </summary>
	// See sect. \ref findiff
	[ RcsId("$Id: FiniteDifferenceModel.cs,v 1.1.1.1 2003/02/01 19:01:28 jens Exp $") ] 
	[ ComVisible(false) ]
	[ Description("Generic finite difference model.") ]
	public class FiniteDifferenceModel 
	{
		public FiniteDifferenceModel(IEvolver evolver, double[] stoppingTimes)
		{
			this.evolver = evolver;
			if( stoppingTimes != null)
				this.stoppingTimes = stoppingTimes;

		}
		private IEvolver evolver;
		private double[] stoppingTimes = new double[0];

		/// <summary>
		/// Solves the problem between the given times, possibly
		/// applying a condition at every step.
		/// </summary>
		/// <remarks>
		/// being this a rollback, <paramref name="fromTime"/> must be a later
		/// time than <paramref name="toTime"/>.
		/// </remarks>
		/// <param name="condition">A step condition applied at every step or null.</param>
		public void Rollback(Vector a, double fromTime, double toTime,
			int steps, IStepCondition condition)
		{
			double dt = (fromTime-toTime)/steps;
			double t = fromTime;
			evolver.Dt = dt;
			for (int i=0; i<steps; i++, t -= dt) 
			{
				int j;
				for(j=0; j<stoppingTimes.Length; j++)
				{
					if(t-dt <= stoppingTimes[j] && stoppingTimes[j] < t)
						break;
				}
				if(j == stoppingTimes.Length)
				{   // No stopping time was hit
					evolver.Step(a,t);
					if (condition != null)
						condition.ApplyTo(a,t);
				}
				else  
				{   // A stopping time was hit

					// First baby step from t to stoppingTimes_[j]
					evolver.Dt = t-stoppingTimes[j];
					evolver.Step(a,t);
					if (condition != null)
						condition.ApplyTo(a,stoppingTimes[j]);

					// Second baby step from stoppingTimes_[j] to t-dt
					evolver.Dt = stoppingTimes[j] - (t-dt);
					evolver.Step(a,stoppingTimes[j]);
					if (condition != null)
						condition.ApplyTo(a,t-dt);

					evolver.Dt=dt;	// reset step size
				}

			}
		}
	}
}