// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: SimpleSwap.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:29  jens
 *  Initial import
 *
 *  Revision 1.4  2002/08/30 18:21:11  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.3  2002/08/07 19:15:00  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.2  2002/07/31 18:49:39  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.1  2002/07/24 17:04:27  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using System.Collections;
using Stochastix.Utilities;
using Stochastix.QuantLib.CashFlows;
using Stochastix.QuantLib.Indexes;
using Stochastix.Business;


namespace Stochastix.QuantLib.Instruments
{
	/// <summary>
	/// Simple fixed-rate vs Libor swap.
	/// </summary>
	[ RcsId("$Id: SimpleSwap.cs,v 1.1.1.1 2003/02/01 19:01:29 jens Exp $") ] 
	[ ComVisible(false) ]
	[ Description("Simple fixed-rate vs Libor swap.") ]
	public class SimpleSwap : Swap 
	{
		public SimpleSwap(bool payFixedRate,
			// dates
			DateTime startDate, int n, TimeUnit units,
			IBusinessCalendar calendar, RollingConvention rollingConvention,
			double nominal,
			// fixed leg
			int fixedFrequency, double fixedRate, 
			bool fixedIsAdjusted, IDayCounter fixedDayCount,
			// floating leg
			int floatingFrequency, IXibor index,
			int indexFixingDays, double spread,
			// hook to term structure
			ITermStructure termStructure) 
			: this(payFixedRate, startDate, n, units, calendar, 
			rollingConvention, nominal, fixedFrequency, fixedRate,
			fixedIsAdjusted, fixedDayCount, floatingFrequency, index,
			indexFixingDays, spread, termStructure, "", "")
		{}
		
		public SimpleSwap(bool payFixedRate,
			// dates
			DateTime startDate, int n, TimeUnit units,
			IBusinessCalendar calendar, RollingConvention rollingConvention,
			double nominal,
			// fixed leg
			int fixedFrequency, double fixedRate, 
			bool fixedIsAdjusted, IDayCounter fixedDayCount,
			// floating leg
			int floatingFrequency, IXibor index,
			int indexFixingDays, double spread,
			// hook to term structure
			ITermStructure termStructure,
			// description
			string isinCode, string description)
			: base(null, null, termStructure, isinCode, description)
		{
			this.payFixedRate = payFixedRate;
			this.fixedRate = fixedRate;
			this.spread = spread;
			this.nominal = nominal;
	                        
			this.maturityDate = calendar.Advance(startDate, 
				n, units, rollingConvention);
            
			ICollection iFixedLeg = Scheduler.FixedRateCouponVector(
				new double[1] {nominal}, new double[1] {fixedRate},
				startDate, maturityDate, fixedFrequency, calendar,
				rollingConvention, fixedIsAdjusted, fixedDayCount,
				fixedDayCount, /*stubDate=*/ startDate);

			ICollection iFloatLeg = Scheduler.FloatingRateCouponVector(
				new double[1] {nominal}, new double[1] {spread},
				startDate, maturityDate, floatingFrequency, calendar,
				rollingConvention, index, indexFixingDays, 
				/*stubDate=*/ startDate);

			RegisterWith(iFloatLeg);

			if (payFixedRate) 
			{
				firstLeg_  = iFixedLeg;
				secondLeg_ = iFloatLeg;
			} 
			else 
			{
				firstLeg_  = iFloatLeg;
				secondLeg_ = iFixedLeg;
			}
		}

		private bool payFixedRate;
		private double fixedRate;
		private double spread;
		private double nominal;
		private DateTime maturityDate;
		
		public double FairRate
		{
			get { return fixedRate - NPV/FixedLegBPS; }
		}

		public double FairSpread
		{
			get { return spread - NPV/FloatingLegBPS; }
		}

		public double FixedLegBPS
		{
			get 
			{ 
				return (payFixedRate ? FirstLegBPS : SecondLegBPS);
			}
		}

		public double FloatingLegBPS
		{
			get
			{
				return (payFixedRate ? SecondLegBPS : FirstLegBPS);
			}
		}

		public double FixedRate
		{
			get { return fixedRate; }
		}

		public double Spread
		{
			get { return spread; }
		}

		public double Nominal
		{
			get { return nominal; }
		}

		public DateTime Maturity
		{
			get { return maturityDate; }
		}

		public bool PayFixedRate
		{
			get { return payFixedRate; }
		}

		public IEnumerable FixedLeg
		{
			get { return (payFixedRate ? firstLeg_ : secondLeg_); }
		}

		public IEnumerable FloatingLeg
		{
			get { return (payFixedRate ? secondLeg_ : firstLeg_); }
		}
	}
}
