// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: BlackScholesLattice.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:30  jens
 *  Initial import
 *
 *  Revision 1.4  2002/08/30 18:21:11  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.3  2002/07/24 17:04:23  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.2  2002/07/24 14:42:41  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Utilities;
using Stochastix.QuantLib.NumericalMethods;

namespace Stochastix.QuantLib.Lattices
{
	/// <summary>
	/// Simple binomial lattice approximating the Black-Scholes model.
	/// </summary>
	/// <remarks>
	/// <para>
	/// The binomial method is the simplest numerical method that can be used to
	/// price path-independent derivatives. It is usually the preferred lattice
	/// method under the Black-Scholes-Merton model. 
	/// </para>
	/// <para>
	/// There are several approaches to build the underlying binomial tree,
	/// like <see cref="JarrowRudd">Jarrow-Rudd</see> or 
	/// <see cref="CoxRossRubinstein">Cox-Ross-Rubinstein</see>.
	/// </para>
	/// </remarks>
	[ RcsId("$Id: BlackScholesLattice.cs,v 1.1.1.1 2003/02/01 19:01:30 jens Exp $") ] 
	[ ComVisible(false) ]
	public class BlackScholesLattice : Lattice 
	{
		public BlackScholesLattice(Tree tree, 
			double riskFreeRate, double endTime, int steps)
			: base(new TimeGrid(endTime, steps), 2)
		{
			this.tree = tree;
			this.discount = Math.Exp(-riskFreeRate*(endTime/steps));
			this.pd = tree.Probability(0,0,0);
			this.pu = tree.Probability(0,0,1);
		}

		private Tree tree;
		private double discount;
		private double pd, pu;

		public Tree Tree
		{ 
			get { return tree; }
		}

		#region Lattice implementation
		override public int Count(int i) 
		{ 
			return tree.Count(i); 
		}
		override public double Discount(int i, int j) 
		{ 
			return discount; 
		}

		override protected void Stepback(int i, double[] values, double[] newValues)
		{
			for (int j=0; j<Count(i); j++)
				newValues[j] = (pd*values[j] + pu*values[j+1]) * discount;
		}

		override protected int Descendant(int i, int index, int branch)
		{
			return tree.Descendant(i, index, branch);
		}
		override protected double Probability(int i, int index, int branch)
		{
			return tree.Probability(i, index, branch);
		}
		#endregion
	}
}
