// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: Lattice.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:30  jens
 *  Initial import
 *
 *  Revision 1.5  2002/08/30 18:21:11  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.4  2002/07/26 00:25:00  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.3  2002/07/24 17:04:24  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.2  2002/07/24 14:42:41  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using System.Collections;
using System.Diagnostics;
using Stochastix.Utilities;
using Stochastix.QuantLib.NumericalMethods;
using Stochastix.QuantLib.Resources;

namespace Stochastix.QuantLib.Lattices
{
	/// <summary>
	/// Lattice-method abstract base class.
	/// </summary>
	/// <remarks>
	/// <para>
	/// A <see cref="Lattice"/> relies on one or several trees 
	/// (each one approximating a diffusion process) to price an
	/// instance of the <see cref="DiscretizedAsset"/> class. 
	/// Trees are instances of classes derived from <see cref="Tree"/>,
	/// and define the branching between nodes and transition probabilities.
	/// </para>
	/// </remarks>
	[ RcsId("$Id: Lattice.cs,v 1.1.1.1 2003/02/01 19:01:30 jens Exp $") ] 
	[ ComVisible(false) ]
	public abstract class Lattice : NumericalMethod 
	{
		public Lattice(ITimeGrid timeGrid, int n) : base(timeGrid)
		{
			this.n = n;
			if( n <= 0 ) 
				throw new ArgumentException( 
					Strings.Get( GetType(), "LatNotZero"), "n");

			statePrices = new ArrayList();
			statePrices.Add( new double[] { 1.0 } );
			statePricesLimit = 0;
		}

		private int n;
		// Arrow-Debrew state prices
		private ArrayList statePrices;
		private int statePricesLimit;

		/// <summary>
		/// Computes the present value of an asset using Arrow-Debrew prices.
		/// </summary>
		/// <param name="asset"></param>
		/// <returns></returns>
		public double PresentValue(IDiscretizedAsset asset)
		{
			int i = TimeGrid.FindIndex(asset.Time);
			if ( i > statePricesLimit )
				ComputeStatePrices(i);
			// return DotProduct(asset.Values, statePrices[i]);
			double[] prices = (double[])statePrices[i];
			Debug.Assert(
				asset.Values.Length == prices.Length,
				"arrays with different sizes cannot be multiplied.");
			double val = 0.0;
			for( int j=0; j<prices.Length; j++)
				val += asset.Values[j] * prices[j];
			return val;
			// return inner_product(asset.Values, statePriceValues(i), 0.0);
		}
/*
		private double inner_product(IEnumerable a, IEnumerable b, double val)
		{
			IEnumerator ita = a.GetEnumerator();
			IEnumerator itb = b.GetEnumerator();
			while( ita.MoveNext() && itb.MoveNext() )
				val += (double)ita.Current * (double)itb.Current;
			return val;
		}
*/
		#region NumericalMethod interface

		/// <summary>
		/// Initialize a DiscretizedAsset object.
		/// </summary>
		/// <param name="asset"></param>
		/// <param name="time"></param>
		override public void Initialize(IDiscretizedAsset asset, double time)
		{
			int i = TimeGrid.FindIndex(time);
			asset.Time = time;
			asset.Reset(Count(i));
		}

		/// <summary>
		/// Roll-back a DiscretizedAsset object until a certain time.
		/// </summary>
		/// <param name="asset"></param>
		/// <param name="toTime"></param>
		override public void Rollback(IDiscretizedAsset asset, double toTime)
		{
			double from = asset.Time;	// Can't Rollback to the future:
			if( from<toTime )
				throw new ArgumentException( Strings.Format( GetType(),
					"LatBackFut", toTime), "toTime");

			int iFrom = TimeGrid.FindIndex(from);
			int iTo = TimeGrid.FindIndex(toTime);

			for (int i=iFrom-1; i>=iTo; i--) 
			{
				double[] newValues = new double[ Count(i) ];
				Stepback(i, asset.Values, newValues);
				asset.Time = TimeGrid[i];
				asset.Values = newValues;
				asset.AdjustValues();
			}
		}

		#endregion


		public abstract int Count(int i);

		/// <summary>
		/// Discount factor at time <i>t<sub>i</sub></i> and 
		/// node indexed by <paramref name="index"/>.
		/// </summary>
		/// <param name="i"></param>
		/// <param name="index"></param>
		/// <returns></returns>
		public abstract double Discount(int i, int index);

		/// <summary>
		/// Descendant.
		/// </summary>
		/// <param name="i"></param>
		/// <param name="index"></param>
		/// <param name="branch"></param>
		/// <returns></returns>
		protected abstract int    Descendant(int i, int index, int branch);

		/// <summary>
		/// Probability.
		/// </summary>
		/// <param name="i"></param>
		/// <param name="index"></param>
		/// <param name="branch"></param>
		/// <returns></returns>
		protected abstract double Probability(int i, int index, int branch);


		public double[] StatePrices(int i)
		{
			if ( i > statePricesLimit )
				ComputeStatePrices(i);
			return (double[])statePrices[i];
		}

		private double[] statePriceValues(int i)
		{
			return (double[])statePrices[i];
		}

		protected void ComputeStatePrices(int until)
		{
			for (int i=statePricesLimit; i<until; i++) 
			{
				double[] v = new double[ Count(i+1) ];
				statePrices.Add(v);
				for (int j=0; j<Count(i); j++) 
				{
					double disc = Discount(i,j);
					double statePrice = statePriceValues(i)[j];
					for (int l=0; l<n; l++) 
					{
						statePriceValues(i+1)[Descendant(i,j,l)] +=
							statePrice*disc*Probability(i,j,l);
					}
				}
			}
			statePricesLimit = until;
		}

		protected virtual void Stepback(int i, double[] values, double[] newValues)
		{
			for (int j=0; j<Count(i); j++) 
			{
				double value = 0.0;
				for (int l=0; l<n; l++) 
					value += Probability(i,j,l) * values[Descendant(i,j,l)];
				value *= Discount(i,j);
				newValues[j] = value;
			}

		}


	}

}
