// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: TrinomialTree.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:30  jens
 *  Initial import
 *
 *  Revision 1.4  2002/08/30 18:21:11  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.3  2002/07/24 17:04:24  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.2  2002/07/24 14:42:41  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Utilities;
using Stochastix.QuantLib.NumericalMethods;


namespace Stochastix.QuantLib.Lattices
{
	/// <summary>
	/// Recombining trinomial tree class.
	/// </summary>
	/// <remarks>
	/// This class defines a recombining trinomial tree approximating a
	/// a diffusion.
	/// <para>
	/// Warning: The diffusion term of the SDE must be independent of
	///	         the underlying process.
	///	</para>
	///	<para>
	///	When the underlying stochastic process has a mean-reverting pattern,
	///	it is usually better to use a trinomial tree instead of a binomial tree. 
	///	<see cref="TrinomialTree"/> is constructed using a 
	///	<see cref="IDiffusionProcess">diffusion process</see> and a
	///	<see cref="ITimeGrid">time-grid</see>. 
	///	The goal is to build a recombining trinomial tree that will
	///	discretize, at a finite set of times, the possible evolutions of a random
	///	variable <i>y</i> satisfying <i>dy<sub>t</sub> = 
	///	mu(t, y<sub>t</sub>) dt + sigma(t, y<sub>t</sub>) dW<sub>t</sub></i>.
	///	At each node, there is a probability <i>p<sub>u</sub></i>, 
	///	<i>p<sub>m</sub></i> and <i>p<sub>d</sub></i> to go through respectively 
	///	the upper, the middle and the lower branch.
	///	</para>
	///	        <!--
	///        
	///			These probabilities must satisfy
	///			\f[
	///				p_{u}y_{i+1,k+1}+p_{m}y_{i+1,k}+p_{d}y_{i+1,k-1}=E_{i,j}
	///			\f]
	///			and
	///			\f[
	///				p_u y_{i+1,k+1}^2 + p_m y_{i+1,k}^2 + p_d y_{i+1,k-1}^2 = 
	///				V^2_{i,j}+E_{i,j}^2,
	///			\f]
	///			where k (the index of the node at the end of the middle branch)
	///			is the index of the node which is the nearest to the expected future
	///			value, \f$ E_{i,j}=\mathbf{E}\left( y(t_{i+1})|y(t_{i})=y_{i,j}\right) \f$
	///			and \f$ V_{i,j}^{2}=\mathbf{Var}\{y(t_{i+1})|y(t_{i})=y_{i,j}\} \f$.
	///			If we suppose that the variance is only dependant on time 
	///			\f$ V_{i,j}=V_{i} \f$ and set \f$ y_{i+1} \f$ to \f$ V_{i}\sqrt{3} \f$,
	///			we find that
	///			\f[
	///				p_{u} = \frac{1}{6}+\frac{(E_{i,j}-y_{i+1,k})^{2}}{6V_{i}^{2}} +
	///						\frac{E_{i,j}-y_{i+1,k}}{2\sqrt{3}V_{i}},
	///			\f]
	///			\f[
	///				p_{m} = \frac{2}{3}-\frac{(E_{i,j}-y_{i+1,k})^{2}}{3V_{i}^{2}},
	///			\f]
	///			\f[
	///				p_{d} = \frac{1}{6}+\frac{(E_{i,j}-y_{i+1,k})^{2}}{6V_{i}^{2}} -
	///						\frac{E_{i,j}-y_{i+1,k}}{2\sqrt{3}V_{i}}.
	///			\f]
	///
	///		-->
	/// </remarks>
	[ RcsId("$Id: TrinomialTree.cs,v 1.1.1.1 2003/02/01 19:01:30 jens Exp $") ] 
	[ ComVisible(false) ]
	public class TrinomialTree : Tree 
	{
		public TrinomialTree(IDiffusionProcess process,
			ITimeGrid timeGrid) : this(process, timeGrid, false)
		{}

		public TrinomialTree(IDiffusionProcess process,
			ITimeGrid timeGrid, bool isPositive) : base(timeGrid.Count)
		{
			this.timeGrid = timeGrid;
			this.x0 = process.X0;

			dx = new double[timeGrid.Count];
			// dx[0] = 0.0;

			int nTimeSteps = timeGrid.Count - 1;
			int jMin = 0;
			int jMax = 0;

			branchings = new TrinomialBranching[nTimeSteps];

			for (int i=0; i<nTimeSteps; i++) 
			{
				double  t = timeGrid[i];
				double dt = timeGrid.Dt(i);

				//Variance must be independent of x
				double v2 = process.Variance(t, 0.0, dt);
				double v  = Math.Sqrt(v2);
				dx[i+1] = v*Math.Sqrt(3.0);		// Add dx

				TrinomialBranching branching = 
					new TrinomialBranching(jMax-jMin+1);

				int jNewMin = int.MaxValue;
				int jNewMax = int.MinValue;
				for (int j=jMin; j<=jMax; j++) 
				{
					double x = x0 + j*dx[i];
					double m = process.Expectation(t, x, dt);
					int temp = (int)Math.Floor((m-x0)/dx[i+1] + 0.5);

					if (isPositive) 
						while (x0 + (temp-1)*dx[i+1] <= 0) 
							temp++;

					if( temp < jNewMin ) jNewMin = temp;
					if( temp > jNewMax ) jNewMax = temp;

					double e = m - (x0 + temp*dx[i+1]);
					double e2 = e*e;
					double e3 = e*Math.Sqrt(3.0);

					int j0 = j - jMin;
					branching.k[j0] = temp;
					branching.probs[0,j0] = (1.0 + e2/v2 - e3/v) / 6.0;
					branching.probs[1,j0] = (2.0 - e2/v2) / 3.0;
					branching.probs[2,j0] = (1.0 + e2/v2 + e3/v) / 6.0;
				}

				branching.jMin = jNewMin-1;
				branching.jMax = jNewMax+1;

				branchings[i] = branching;	// Add branching

				jMin = branching.jMin;
				jMax = branching.jMax;
			}

		}

		private ITimeGrid timeGrid;
		private double x0;
		private double[] dx;
		private TrinomialBranching[] branchings;

		public ITimeGrid TimeGrid 
		{ 
			get { return timeGrid; }
		}

		public double Dx(int i)
		{ 
			return dx[i]; 
		}

		override public double Underlying(int i, int index)
		{
			if (i==0) return x0;
			int jMin = branchings[i-1].jMin;
			return x0 + Dx(i)*(double)(jMin+index);		// todo: Dx(i) = dx[i] ??
		}

		override public int Count(int i)
		{
			if (i==0) return 1;
			return branchings[i-1].jMax - 
				branchings[i-1].jMin + 1;
		}

		override public int Descendant(int i, int index, int branch)
		{
			return branchings[i].Descendant(index, branch);
		}

		override public double Probability(int i, int index, int branch)
		{
			return branchings[i].Probability(index, branch);
		}

		/// <summary>
		/// Branching scheme for a trinomial node.
		/// </summary>
		/// <remarks>
		/// Each node has three descendants, with the middle branch linked
		/// to the node which is closest to the expectation of the variable.
		/// </remarks>
		private class TrinomialBranching 
		{
			public TrinomialBranching(int n)
			{
				k = new int[n];
				probs = new double[3,n];
			}

			public int[] k;
			public double[,] probs;
			public int jMin = int.MaxValue;
			public int jMax = int.MinValue;

			public int Descendant(int index, int branch)
			{
				return (k[index] - jMin) - 1 + branch;
			}
			public double Probability(int index, int branch)
			{
				return probs[branch,index];
			}
		}
	}
}
