// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: BasketPathPricer.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:30  jens
 *  Initial import
 *
 *
 *********************************************************************
 */
#endregion

using System;
using Stochastix.Numerics;
using Stochastix.QuantLib.Instruments;
using Stochastix.QuantLib.Resources;

namespace Stochastix.QuantLib.MonteCarlo
{
	/// <summary>
	/// Multipath pricer for European-type basket option.
	/// </summary>
	/// <remarks>
	/// The value of the option at expiration is given by the value
	/// of the underlying which has best performed.
	/// </remarks>
	public class BasketPathPricer : PathPricer
	{
		public BasketPathPricer(Option.Type optionType, Vector underlying, 
			double strike, double discountFactor, bool useAntitheticVariance) 
			: base( discountFactor, useAntitheticVariance)
		{
			if( underlying.Min() <= 0.0 )
				throw new ArgumentException( Strings.Get( GetType(),
					"TODO: Price of underlying(s) must be positive."), "underlying" );
			if( strike <= 0.0 )
				throw new ArgumentException( Strings.Get( GetType(),
					"TODO: Strike price must be positive."), "strike" );
			this.optionType = optionType;
			this.underlying = underlying;
			this.strike = strike;
		}	
		private Option.Type optionType;
		Vector underlying;
		private double strike;

		/// <summary>
		/// Given one or more pathes, the value of an option is returned.
		/// </summary>
		/// <param name="pathes"></param>
		/// <returns></returns>
		public override double Value(Path[] pathes)
		{
			// would throw anyways: QL_RequireArgNotNull(pathes, "pathes", null);
			if( pathes.Length != underlying.Count )
				throw new ArgumentException( Strings.Get( GetType(),
					"TODO: Not enough pathes." ), "pathes" );
			if( pathes[0].Count == 0 )
				throw new ArgumentException( Strings.Get( GetType(),
					"TODO: the path cannot be empty."), "pathes" );

			int numAssets = pathes.Length;
			Vector logDrift     = new Vector(numAssets);
			Vector logDiffusion = new Vector(numAssets);
			for(int j=0; j<numAssets; j++)
			{
				logDrift[j]     = pathes[j].Drift.Sum();
				logDiffusion[j] = pathes[j].Diffusion.Sum();
			}

			double basketPrice1 = Vector.DotProduct( underlying,
				Vector.Exp( logDrift + logDiffusion) );
			if (useAntitheticVariance)
			{
				double basketPrice2 = Vector.DotProduct( underlying,
					Vector.Exp( logDrift - logDiffusion) );
				return discount * ( 
					Option.ExercisePayoff( optionType, basketPrice1, strike) +
					Option.ExercisePayoff( optionType, basketPrice2, strike)
					) / 2.0;
			}
			else
			{
				return discount * Option.ExercisePayoff( 
					optionType, basketPrice1, strike);
			}
		}
	}
}
