// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: MonteCarloModel.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:30  jens
 *  Initial import
 *
 *  Revision 1.3  2002/10/15 13:18:58  stochastix
 *  now using dotNum
 *
 *  Revision 1.2  2002/08/30 18:19:35  stochastix
 *  small tweaks + license
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Numerics.Statistics;

namespace Stochastix.QuantLib.MonteCarlo
{
	/// <summary>
	/// General purpose Monte Carlo model for path samples.
	/// </summary>
	/// <remarks>
	/// This class encapsulates the general structure of a Monte Carlo 
	/// calculations, namely
	/// <list type="bullet">
	/// <item><description>
	/// the generation of a number of paths,
	/// </description></item>
	/// <item><description>
	/// the pricing of the derivative on each path and
	/// </description></item>
	/// <item><description>
	/// the averaging of the results to yield the actual derivative price.
	/// </description></item>
	/// </list>
	/// <para>
	/// As outlined above, the first two steps are delegated to a 
	/// <see cref="PathGenerator"/> and a <see cref="PathPricer"/>. 
	/// The third step is also delegated to an object which
	/// accumulates weighted values and returns the statistic properties of the
	/// set of such values. One such class is <see cref="RiskStatistics"/>
	/// provided by the <see cref="Stochastix.Numerics"/> assembly.
	/// </para>
	/// <para>
	/// The concern of the Monte Carlo model is therefore to act as a glue
	/// between such three components and can be expressed by the following
	/// pseudo-code:
	/// <code>
	/// given pathGenerator, pathPricer, accumulator;
	/// for i in number of samples {
	///		path,weight = pathGenerator.next();
	///		price = pathPricer(path);
	///		accumulator.add(price,weight);
	///	}
	///	</code>
	///	</para>
	///	<para>
	///	The Monte Carlo model also provides the user with the possibility to
	///	take advantage of control-variate techniques.
	///	Such techniques consist in pricing a portfolio from which the price of
	///	the derivative can be deduced, but with a lower variance than the
	///	derivative alone. 
	///	</para>
	///	<para>
	///	In our current implementation, static-hedge control variate is used,
	///	namely, the formed portfolio is long of the derivative we need to price
	///	and short of a similar derivative whose price can be calculated
	///	analytically. The value of the portfolio on a given path will of course
	///	be given by the difference of the values of the two derivatives on such
	///	path. However, due to the similarity between the derivatives, the
	///	portfolio price will have a lower variance than either derivative alone
	///	since any variation in the price of the latter will be partly compensated
	///	by a similar variation in the price of the other.
	///	Lastly, given the portfolio price, the price of the derivative we are
	///	interested in can be deduced by adding the analytic value of the other.
	///	</para>
	///	<para>
	///	In order to use such technique, the user must provide the model with a
	///	path pricer for the additional option and the value of the latter.
	///	The action of the Monte Carlo model is in this case expressed as
	///	in the following pseudo-code:
	///	<code>
	///	given pathGenerator, pathPricer, cvPathPricer, cvPrice, accumulator;
	///	for i in number of samples {
	///		path,weight  = pathGenerator.next();
	///		portfolioPrice = pathPricer(path) - cvPathPricer(path);
	///		accumulator.add(portfolioPrice+cvPrice,weight);
	///	}
	///	</code>
	/// </para>
	/// </remarks>
	public class MonteCarloModel 
	{
		/// <overloads>
		/// Initialize a new MonteCarloModel.
		/// </overloads>
		/// <summary>
		/// Initialize a new MonteCarloModel without using control variate techniques
		/// and using the default <see cref="RiskStatistics"/> accumulator.
		/// </summary>
		/// <param name="pathGenerator">The <see cref="PathGenerator"/> generating single or multiple random walks.</param>
		/// <param name="pathPricer">The <see cref="PathPricer"/> calculating the option price on each sample.</param>
		public MonteCarloModel(PathGenerator pathGenerator,
			PathPricer pathPricer)
			: this(pathGenerator, pathPricer, new RiskStatistics())
		{}

		/// <summary>
		/// Initialize a new MonteCarloModel without using control variate techniques.
		/// </summary>
		/// <param name="pathGenerator">The <see cref="PathGenerator"/> generating single or multiple random walks.</param>
		/// <param name="pathPricer">The <see cref="PathPricer"/> calculating the option price on each sample.</param>
		/// <param name="sampleAccumulator">A sample accumulator to record statistics.</param>
		public MonteCarloModel(PathGenerator pathGenerator,
			PathPricer pathPricer, RiskStatistics sampleAccumulator)
			: this(pathGenerator, pathPricer, sampleAccumulator, null, 0.0)
		{}

		/// <summary>
		/// Initialize a new MonteCarloModel optionally using control variate techniques.
		/// </summary>
		/// <remarks>
		/// In order to use the control variate technique, the user should provide the
		/// additional control option, namely the <paramref name="cvPathPricer"/> and
		/// the <paramref name="cvOptionValue"/>.
		/// </remarks>
		/// <param name="pathGenerator">
		/// The <see cref="PathGenerator"/> generating single or multiple random walks.
		/// </param>
		/// <param name="pathPricer">
		/// The <see cref="PathPricer"/> calculating the option price on each sample.
		/// </param>
		/// <param name="sampleAccumulator">
		/// A sample accumulator to record statistics.
		/// </param>
		/// <param name="cvPathPricer">
		/// The control variate <see cref="PathPricer"/> or 
		/// null (<b>Nothing</b> in Visual Basic).
		/// </param>
		/// <param name="cvOptionValue">
		/// The option value in case of using a <paramref name="cvPathPricer"/>.
		/// </param>
		public MonteCarloModel(PathGenerator pathGenerator,
			PathPricer pathPricer, RiskStatistics sampleAccumulator,
			PathPricer cvPathPricer, double cvOptionValue)
		{
			this.pathGenerator = pathGenerator;
			this.pathPricer = pathPricer;
			this.sampleAccumulator = sampleAccumulator;
			this.cvPathPricer = cvPathPricer;
			this.cvOptionValue = cvOptionValue;
		}

		private PathGenerator pathGenerator;
		private PathPricer pathPricer;
		private RiskStatistics sampleAccumulator;
		private PathPricer cvPathPricer;
		private double cvOptionValue;

		/// <summary>
		/// The sample acuumulator.
		/// </summary>
		public RiskStatistics SampleAccumulator
		{
			get { return sampleAccumulator; }
		}

		/// <summary>
		/// Generate samples and calculate option prices.
		/// </summary>
		/// <param name="samples">Number of samples to generate.</param>
		public void AddSamples(int samples)
		{
			for(int i=0; i<samples; i++) 
			{
				Sample path = pathGenerator.Next();
				double price = pathPricer.Value((Path[])path.value);
				if (cvPathPricer!=null)
					price += cvOptionValue - 
						cvPathPricer.Value((Path[])path.value);
				sampleAccumulator.Add(price, path.weight);
			}
		}
	}
}
