// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: Path.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:31  jens
 *  Initial import
 *
 *  Revision 1.2  2002/08/30 18:19:35  stochastix
 *  small tweaks + license
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Collections;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Numerics;
using Stochastix.QuantLib.Resources;

namespace Stochastix.QuantLib.MonteCarlo
{
	/// <summary>
	/// Single factor random walk.
	/// </summary>
	/// <remarks>
	/// This class stores the variation vector decomposed in its 
	/// <see cref="Drift"/> (determined) and <see cref="Diffusion"/> (random) 
	/// components. 
	/// <para>
	/// This allows the implementation of antithetic variance reduction techniques.
	/// In short, such techniques consist in pricing an option on both the
	/// given path and its antithetic, the latter being a path with the same
	/// drift and the opposite diffusion component. The value of the sample
	/// is defined as the average of the prices on the two paths.
	/// </para>
	/// <para>
	/// The term <em>path</em> will be used as meaning either path or multi-path
	/// depending on the context. The term <em>single path</em> is not to be taken
	/// as opposite to multi-path, but rather as meaning "a single instance of a 
	/// (multi)path" as opposed to the set of all generated (multi)paths.
	/// <see cref="PathPricer"/>s are used to price the generated path.
	/// </para>
	/// </remarks>
	public class Path
	{
		//			public Path(int size)
		//			{
		//				QL_RequireArgRange(size>0, "size", "TODO", size);
		//				this.times = new Vector(size);
		//				this.drift = new Vector(size);
		//				this.diffusion = new Vector(size);
		//			}

		/// <summary>
		/// Initialize a new Path.
		/// </summary>
		/// <param name="times">
		/// Discrete times at which the path was sampled.
		/// </param>
		/// <param name="drift">Deterministic components.</param>
		/// <param name="diffusion">Random components.</param>
		/// <exception cref="ArgumentException">
		/// Thrown when the times, drifts and diffusion vectors have
		/// not the same length.
		/// </exception>
		public Path(Vector times, Vector drift, Vector diffusion)
		{
			if( times.Count != drift.Count )
				throw new ArgumentException( Strings.Get( GetType(),
					"TODO: times and drift vectors have different size." ), "drift" );
			if( times.Count != diffusion.Count )
				throw new ArgumentException( Strings.Get( GetType(),
					"TODO: times and diffusion vectors have different size." ), "diffusion" );
			this.times = times;
			this.drift = drift;
			this.diffusion = diffusion;
		}

		private Vector times;
		private Vector drift;
		private Vector diffusion;

		/// <summary>
		/// The added value of the deterministic and random components, sampled
		/// at the discrete values in <see cref="Times"/>.
		/// </summary>
		public double this[int i]
		{
			get { return drift[i] + diffusion[i]; }
		}

		/// <summary>
		/// Number of time steps / samples.
		/// </summary>
		public int Count
		{
			get { return drift.Count; }
		}

		/// <summary>
		/// Discrete times <see cref="Vector"/> at which the path was sampled.
		/// </summary>
		public Vector Times
		{
			get { return times; }
			//				set { times = value; }
		}

		/// <summary>
		/// Drift <see cref="Vector"/>.
		/// </summary>
		public Vector Drift
		{
			get { return drift; }
			//				set { drift = value; }
		}

		/// <summary>
		/// Diffusion <see cref="Vector"/>.
		/// </summary>
		public Vector Diffusion
		{
			get { return diffusion; }
			//				set { diffusion = value; }
		}

	}
}
