// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: PathGenerator.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:31  jens
 *  Initial import
 *
 *  Revision 1.3  2002/10/15 13:18:58  stochastix
 *  now using dotNum
 *
 *  Revision 1.2  2002/08/30 18:19:35  stochastix
 *  small tweaks + license
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Collections;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Numerics;
using Stochastix.Numerics.Statistics;
using Stochastix.QuantLib.Resources;

namespace Stochastix.QuantLib.MonteCarlo
{
	/// <summary>
	/// Abstract base class for random path generators.
	/// </summary>
 	/// <remarks>
	/// <para>
	/// The Black-Scholes equation with the the risk-free rate <i>r</i>, 
	/// volatility of the underlying <i>s</i> and <i>v=r-s<super>2</super>/2</i>
	/// has the form of a diffusion process. According to a heuristic 
	/// interpretation, paths followed by the logarithm of the underlying would
	/// be Brownian random walks with a constant drift <i>v</i> per unit time
	/// and a standard deviation <i>s*sqrt(T)</i> over a time <i>T</i>.
	/// </para>
	/// <para>
	/// Therefore, the paths to be generated for a Monte Carlo model of the
	/// Black-Scholes equation will be vectors of successive variations of
	/// the logarithm of the underlying price over <i>M</i> consecutive time
	/// intervals <i>dt<sub>i</sub>, i = 0..M-1</i>.
	/// Each such variation will be drawn from a Gaussian distribution with
	/// average <i>v dt<sub>i</sub></i> and standard deviation
	/// <i>s sqrt(dt<sub>i</sub>)</i> - or possibly <i>v<sub>i</sub> dt<sub>i</sub></i>
	/// and <i>s<sub>i</sub> sqrt(dt<sub>i</sub>)</i> should <i>v</i> and 
	/// <i>s</i> vary in time.
	/// </para>
	/// <para>
	/// The <see cref="Path" /> class stores the variation vector decomposed in 
	/// its drift (determined) and diffusion (random) components.
	/// This allows the implementation of antithetic variance reduction techniques.
	/// </para>
	/// <para>
	/// Derived classes are provided which generate paths and multi-paths with the 
	/// desired drift and diffusion components, namely, 
	/// <see cref="SinglePathGenerator"/> and <see cref="MultiPathGenerator"/>.
	/// </para>
	/// <para>
	/// The <see cref="SinglePathGenerator"/> is initialized with a constant
	/// drift and variance or time dependent parameters. 
	/// The <see cref="MultiPathGenerator"/> is initialized with an array of
	/// constant drifts - one for each single asset - and a covariance matrix
	/// which encapsulates the relations between the diffusion components of
	/// the single assets.
	/// </para>
	/// <para>
	/// The time discretization of the (multi)paths can be specified either as
	/// a given number of equal time steps over a given time span, or as a
	/// vector of explicitly specified times at which the path will be sampled.
	/// </para>
	/// </remarks>
	public abstract class PathGenerator
	{
		/// <summary>
		/// Protected constructor for abstract base class.
		/// </summary>
		protected PathGenerator(int assets, int timeSteps)
		{
			if( assets <= 0) 
				throw new ArgumentException( Strings.Get( GetType(),
					"TODO: Number of assets must be > 0."), "assets");
			if( timeSteps <= 0) 
				throw new ArgumentException( Strings.Get( GetType(),
					"TODO: Time steps must be > 0."), "timeSteps" );
			this.assets = assets;
			this.timeSteps = timeSteps;
			this.drift = new Vector[assets];
		}

		/// <summary>
		/// Protected constructor for abstract base class.
		/// </summary>
		protected PathGenerator(Vector drifts, Vector times)
			: this(drifts.Count, times.Count)
		{
			if( times[0] < 0.0 ) 
				throw new ArgumentException( Strings.Get( GetType(),
				"TODO: First time must be >= 0."), "times" );
			this.times = times;
			this.timeDelays = adjacent_difference(times);
			for( int i=0; i<assets; i++)
				this.drift[i] = /* Vector */ timeDelays * drifts[i];
		}

		/// <summary>
		/// Protected constructor for abstract base class.
		/// </summary>
		protected PathGenerator(Vector drifts, double time, int timeSteps)
			: this(drifts.Count, timeSteps)
		{
			if( time <= 0.0) 
				throw new ArgumentException( Strings.Get( GetType(),
					"TODO: Time must be > 0."), "time" );

			double dt = time/TimeSteps;
			this.times = new Vector(TimeSteps, dt, dt);
			this.timeDelays = new Vector( TimeSteps, dt);
			for( int i=0; i<assets; i++)
				this.drift[i] = new Vector(TimeSteps, drifts[i]*dt);
		}

		private int assets, timeSteps;
		protected Vector times, timeDelays;
		protected Vector[] drift;
		protected RandomArrayGenerator arrayGenerator;

		/// <summary>
		/// Number of assets = number of paths generated by the path generator.
		/// </summary>
		public int Count
		{
			get { return assets; }
		}

		/// <summary>
		/// Number of times at which the path will be sampled.
		/// </summary>
		/// <remarks>
		/// Depending on the iinitialization, these could be equal time steps
		/// or a vector of explicitly specified times at which the path will
		/// be sampled.
		/// </remarks>
		public int TimeSteps
		{
			get { return timeSteps; }
		}

		// returns Sample<Path[]>
		/// <summary>
		/// Generate next path.
		/// </summary>
		/// <remarks>
		/// Derived path generators must override this method to generate one or more
		/// <see cref="Path"/>es.
		/// </remarks>
		/// <returns>
		/// An array containing a single <see cref="Path"/>, 
		/// wrapped in a (possibly weighted) <see cref="Sample"/>.
		/// </returns>
		public abstract Sample Next();

		protected static Vector adjacent_difference(Vector times)
		{
			int n = times.Count;
			Vector timeDelays = new Vector(n);
			double t = 0.0;
			for(int i=0; i<n; i++) 
			{
				double ti = times[i];
				if( t>ti )
					throw new ArgumentException( Strings.Get( typeof(PathGenerator),
						"TODO: Sample times are not in ascending order"), "times");
				timeDelays[i] = ti-t;
				t = ti;
			}
			return timeDelays;
		}

	}
}
