// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: PathPricer.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:31  jens
 *  Initial import
 *
 *  Revision 1.2  2002/08/30 18:19:35  stochastix
 *  small tweaks + license
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Collections;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Numerics;
using Stochastix.QuantLib.Resources;

namespace Stochastix.QuantLib.MonteCarlo
{
	/// <summary>
	/// Base class for path pricers.
	/// </summary>
	/// <remarks>
	/// This is the base class from which path pricers must inherit. 
	/// <para>
	/// The only method which subclasses are required to implement is
	/// <c>double Value(<see cref="Path"/>[] path);</c> where <c>path[]</c> 
	/// is an array of one or more <see cref="Path"/>s depending on the derivative
	/// whose value must be calculated.
	/// </para>
	/// <para>
	/// Similarly, the term <em>path</em> will be used in the following
	/// discussion as meaning either path or multi-path depending on the
	/// context.
	/// The term <em>single path</em> is not to be taken as opposite to
	/// multi-path, but rather as meaning "a single instance of a (multi)path"
	/// as opposed to the set of all generated (multi)paths.
	/// </para>
	/// <para>
	/// The above method encapsulates the pricing of the derivative on a single
	/// path and must return its value had the evolution of the underlying(s)
	/// followed the path passed as argument.
	/// </para>
	/// <para>
	/// For this reason, control variate techniques (see below) must not be
	/// implemented at this level since they would cause the returned value
	/// to differ from the actual price of the derivative on the path.
	/// </para>
	/// <para>
	/// Instead, antithetic variance-reduction techniques can be effectively
	/// implemented at this level and indeed are used in the pricers currently
	/// included in the library.
	/// </para>
	/// <para>
	/// In short, such techniques consist in pricing an option on both the
	/// given path and its antithetic, the latter being a path with the same
	/// drift and the opposite diffusion component. The value of the sample
	/// is defined as the average of the prices on the two paths.
	/// </para>
	/// <para>
	/// A generic implementation of antithetic techniques could consist of a
	/// path pricer class which takes a concrete path pricer upon construction
	/// and whose <c>Value</c> method (as described above) simply proxies two
	/// calls to the contained pricer, passing the given path and its antithetic, 
	/// and averages the result.
	/// However, this would not take full advantage of the technique.
	/// </para>
	/// <para>
	/// In fact, it must be noted that using antithetic paths not only reduces
	/// the variance <em>per se</em> but also allows to factor out
	/// calculations commons to a path and its antithetic, thus reducing greatly
	/// the computation time. Therefore, such techniques are best implemented
	/// inside the path pricer itself, whose algorithm can fully exploit such
	/// factorization.
	/// </para>
	/// <para>
	/// A number of path pricers are available in the library and are listed
	/// in the namespace reference.
	/// </para>
	/// </remarks>
	public abstract class PathPricer
	{
		/// <summary>
		/// Protected constructor for abstract base class.
		/// </summary>
		protected PathPricer(double discountFactor, bool useAntitheticVariance)
		{
			if( discountFactor <= 0.0 || discountFactor > 1.0 )
				throw new ArgumentOutOfRangeException( "discountFactor", discountFactor, 
					Strings.Get( GetType(), "TODO: Discount factor must be in (0%;100%].") );
			this.discount = discountFactor;
			this.useAntitheticVariance = useAntitheticVariance;
		}

		protected double discount;
		protected bool useAntitheticVariance;

		/// <summary>
		/// Given one or more pathes, the value of an option is returned.
		/// </summary>
		/// <remarks>
		/// This method must be overriden by derived classes. 
		/// </remarks>
		/// <param name="path">
		/// An array of one or more <see cref="Path"/>s depending on the derivative
		/// whose value must be calculated.
		/// </param>
		/// <returns>
		/// The price of the derivative had the evolution of the underlying(s)
		/// followed the path passed as argument.
		/// </returns>
		public abstract double Value(Path[] path);

	}
}