// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: SinglePathGenerator.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:31  jens
 *  Initial import
 *
 *  Revision 1.3  2002/10/15 13:18:58  stochastix
 *  now using dotNum
 *
 *  Revision 1.2  2002/08/30 18:19:35  stochastix
 *  small tweaks + license
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Collections;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Numerics;
using Stochastix.Numerics.Statistics;
using Stochastix.Random;
using Stochastix.QuantLib.Resources;

namespace Stochastix.QuantLib.MonteCarlo
{
	/// <summary>
	/// Generates a single random path from a continous RNG.
	/// </summary>
	/// <remarks>
	/// See introduction to <see cref="PathGenerator"/> base class for more
	/// information about path generators.
	/// <para>
	/// The <see cref="SinglePathGenerator"/> is initialized with a constant
	/// drift and variance or time dependent parameters. 
	/// </para>
	/// <para>
	/// The time discretization of the paths can be specified either as
	/// a given number of equal time steps over a given time span, or as a
	/// vector of explicitly specified times at which the path will be sampled.
	/// </para>
	/// </remarks>
	public class SinglePathGenerator : PathGenerator
	{
		/// <overloads>
		/// Initialize a new SinglePathGenerator.
		/// </overloads>
		/// <summary>
		/// Initialize a new SinglePathGenerator with constant drift and variance,
		/// sampled at the given number of equal time steps.
		/// </summary>
		/// <param name="drift">
		/// Constant drift of the Brownian random walks.
		/// </param>
		/// <param name="variance">
		/// Constant variance.
		/// </param>
		/// <param name="time">
		/// Time span which is sampled in <paramref name="timeSteps"/> equal time steps.
		/// </param>
		/// <param name="timeSteps">
		/// Number of equal time steps at which the path will be sampled.
		/// </param>
		public SinglePathGenerator(IContinousRng generator,
			double drift, double variance, double time, int timeSteps)
			: base(new Vector(1, drift), time, timeSteps)
		{
			if( variance < 0.0 )
				throw new ArgumentException( Strings.Get( GetType(),
					"TODO: Variance can not be negative."), "variance" );
			this.arrayGenerator = new RandomArrayGenerator( generator, 
				new Vector(TimeSteps, variance * time/TimeSteps ));
		}

		/// <summary>
		/// Initialize a new SinglePathGenerator with constant drift and variance.
		/// </summary>
		/// <remarks>
		/// The initial time is assumed to be zero and must 
		/// <b>not</b> be included in the passed vector.
		/// </remarks>
		/// <param name="drift">
		/// Constant drift of the Brownian random walks.
		/// </param>
		/// <param name="variance">
		/// Constant variance.
		/// </param>
		/// <param name="times">
		/// A <see cref="Vector"/> of explicitly specified times at which the
		/// path will be sampled. 
		/// The initial time is assumed to be zero and must 
		/// <b>not</b> be included in the passed vector.
		/// </param>
		public SinglePathGenerator(IContinousRng generator,
			double drift, double variance, Vector times) 
			: base(new Vector(1, drift), times)
		{
			if( variance < 0.0 )
				throw new ArgumentException( Strings.Get( GetType(),
					"TODO: Variance can not be negative."), "variance" );
			this.arrayGenerator = new RandomArrayGenerator( generator, 
				timeDelays * variance); 
		}

		/// <summary>
		/// Initialize a new SinglePathGenerator with a time dependent path.
		/// </summary>
		/// <remarks>
		/// The initial time is assumed to be zero and must 
		/// <b>not</b> be included in the passed vector.
		/// </remarks>
		/// <param name="drift">
		/// A <see cref="Vector"/> of time-dependent drifts of the Brownian
		/// random walks.
		/// </param>
		/// <param name="variance">
		/// A <see cref="Vector"/> of time-dependent variances.
		/// </param>
		/// <param name="times">
		/// A <see cref="Vector"/> of explicitly specified times at which the
		/// path will be sampled. 
		/// The initial time is assumed to be zero and must 
		/// <b>not</b> be included in the passed vector.
		/// </param>
		public SinglePathGenerator( IContinousRng generator,
			Vector drift, Vector variance, Vector times) 
			: base(1, times.Count)
		{
			if( times[0] < 0.0 )
				throw new ArgumentException( Strings.Get( GetType(), 
					"TODO: First time can not be negative (times[0] must be >= 0.0)." ), "times" );
			if( variance.Count != TimeSteps )
				throw new ArgumentException( Strings.Get( GetType(), 
					"TODO: Size mismatch between variance and time vectors." ), "variance" );
			if( drift.Count != TimeSteps )
				throw new ArgumentException( Strings.Get( GetType(), 
					"TODO: Size mismatch between drift and time vectors." ), "drift" );

			this.times = times;
			timeDelays = adjacent_difference(times);
			this.drift[0] = timeDelays * drift;

			this.arrayGenerator = new RandomArrayGenerator( generator, 
				timeDelays * variance); 
		}

		/// <summary>
		/// Generate next path.
		/// </summary>
		/// <returns>
		/// An array containing a single <see cref="Path"/>, 
		/// wrapped in a (possibly weighted) <see cref="Sample"/>.
		/// </returns>
		public override Sample Next()
		{
			Path[] pathes = new Path[1];
			Sample sample = arrayGenerator.NextSample();  // get a weighted Vector
			pathes[0] = new Path(times, drift[0], (Vector)sample.value);
			return new Sample( pathes, sample.weight);
		}
	}
}
