// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: DiscretizedAsset.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:31  jens
 *  Initial import
 *
 *  Revision 1.4  2002/08/30 18:21:11  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.3  2002/07/24 17:04:26  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.2  2002/07/24 14:42:42  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using System.Collections;
using Stochastix.Utilities;
using Stochastix.QuantLib.Resources;

namespace Stochastix.QuantLib.NumericalMethods
{
	/// <summary>
	/// Discretized asset class used by numerical methods.
	/// </summary>
	/// <remarks>
	/// This abstract base class is a representation of the price of a
	/// derivative at a specific time. It is roughly an array of values, 
	/// each value being associated to a state of the underlying stochastic
	/// variables. For the moment, it is only used when working with trees, 
	/// but it should be quite easy to make a use of it in finite-differences
	/// methods. 
	/// The two main points, when deriving classes, are:
	/// <list type="number">
	/// <item><description>
	/// Define the initialisation procedure 
	/// (e.g. terminal payoff for european stock options).
	/// </description></item>
	/// <item><description>
	/// Define the method adjusting values, when necessary, at each time steps 
	/// (e.g. apply the step condition for american or bermudan options).
	/// </description></item>
	/// </list>
	/// </remarks>
	[ RcsId("$Id: DiscretizedAsset.cs,v 1.1.1.1 2003/02/01 19:01:31 jens Exp $") ] 
	[ ComVisible(false) ]
	abstract public class DiscretizedAsset : IDiscretizedAsset
	{

		private const double EPSILON = Double.Epsilon;	// TODO: maybe this is to small!

		public DiscretizedAsset(INumericalMethod method)
		{
			this.method = method;
		}

		private INumericalMethod method;
		protected double[] values_;
		protected double time_;
		
		#region IDiscretizedAsset implementation

		public double Time
		{
			get	{ return time_; }
			set { this.time_ = value; }
		}

		public double[] Values 
		{
			get { return values_; }
			set { this.values_ = value; }
		}

		public double this[int i]
		{ 
			get 
			{ 
				if ( values_ == null )
					throw new InvalidOperationException(
						Strings.Get( GetType(), "NumDiscrAsVal"));
				return values_[i]; 
			}
		}

		public INumericalMethod Method
		{ 
			get { return method; }
		}

		abstract public void Reset(int size);

		virtual public void AdjustValues() 
		{}

		virtual public void AddTimes(/*std::list<Time>&*/ IList times) 
		{}

		#endregion

		protected bool IsOnTime(double time)
		{
			ITimeGrid grid = this.Method.TimeGrid;
			double gridTime = grid[grid.FindIndex(time)];
			return ( Math.Abs(gridTime - this.Time) < EPSILON);
		}

	}
}
