// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: EndCriteria.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:31  jens
 *  Initial import
 *
 *  Revision 1.2  2002/08/30 18:21:11  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.1  2002/08/07 19:14:59  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Utilities;

namespace Stochastix.QuantLib.Optimization
{
	/// <summary>
	/// Class to gather criteria to end optimization process.
	/// </summary>
	/// <remarks>
	/// Possible end criterias are:
	/// <list type="bullet">
	///		<item>
	///			<description>stationary point</description> 
	///		</item>
	///		<item>
	///			<description>stationary gradient</description> 
	///		</item>
	///		<item>
	///			<description>maximum number of iterations</description> 
	///		</item>
	///		<item>
	///			<description>...</description>
	///		</item>
	/// </list>
	/// </remarks>
	[ RcsId("$Id: EndCriteria.cs,v 1.1.1.1 2003/02/01 19:01:31 jens Exp $") ] 
	[ ComVisible(false) ]
	public class EndCriteria : ICloneable
	{
		public enum CriteriaType { none, maxIter, statPt, statGd };

		/// <summary>
		/// Default constructor.
		/// </summary>
		public EndCriteria() : this(100, 1e-8)
		{}

		/// <summary>
		/// Initialization constructor.
		/// </summary>
		/// <param name="maxIteration"></param>
		/// <param name="epsilon"></param>
		public EndCriteria (int maxIteration, double epsilon)
		{
			this.maxIteration = maxIteration;
			this.functionEpsilon = epsilon;
			this.gradientEpsilon = epsilon;
			this.maxIterStatPt = maxIteration/10;
			// this.statState = 0;
			// this.endCriteria = CriteriaType.none;
			// this.positiveOptimization = false;
		}

		public EndCriteria Clone()
		{
			return (EndCriteria)this.MemberwiseClone();
		}

		
		//! Maximum number of iterations
		private int maxIteration;
		//! function and gradient epsilons
		private double functionEpsilon, gradientEpsilon;
		//! Maximum number of iterations in stationnary state
		private int maxIterStatPt, statState;
		private CriteriaType endCriteria;
		private bool positiveOptimization;

		public bool PositiveOptimization
		{
			get { return positiveOptimization; }
			set { positiveOptimization = value; }
		}

		/// <summary>
		/// Maximum iterations.
		/// </summary>
		public int MaxIterations
		{
			get { return maxIteration; }
			set { maxIteration = value; }
		}

		/// <summary>
		/// The end criteria type.
		/// </summary>
		public CriteriaType Criteria
		{
			get { return endCriteria; }
		}

		/// <summary>
		/// Test if the number of iteration is not too big and if we don't
		/// raise a stationary point.
		/// </summary>
		/// <param name="iteration"></param>
		/// <param name="fold"></param>
		/// <param name="normgold"></param>
		/// <param name="fnew"></param>
		/// <param name="normgnew"></param>
		/// <param name="normdiff"></param>
		/// <returns></returns>
		public bool Check(int iteration,
			double fold,
			double normgold,
			double fnew,
			double normgnew,
			double normdiff)
		{
			return (
				CheckIterationNumber(iteration) ||
				CheckStationaryValue(fold, fnew) ||
				CheckAccuracyValue(fnew) ||
				CheckAccuracyValue(fold) ||
				CheckAccuracyGradientNorm(normgnew) ||
				CheckAccuracyGradientNorm(normgold)
				);
		}

		public bool CheckIterationNumber(int iteration) 
		{
			bool test = iteration >= maxIteration;
			if (test)
				endCriteria = CriteriaType.maxIter;
			return test;
		}

		public bool CheckStationaryValue(double fold, double fnew) 
		{
			bool test = Math.Abs(fold-fnew) < functionEpsilon;
			if (test) 
			{
				if (++statState > maxIterStatPt) 
					endCriteria = CriteriaType.statPt;
			} 
			else 
			{
				if (statState != 0)
					statState = 0;
			}
			return test && (statState > maxIterStatPt);
		}

		public bool CheckAccuracyValue(double f) 
		{
			bool test = positiveOptimization && (f < functionEpsilon);
			if (test) 
				endCriteria = CriteriaType.statPt;
			return test;
		}

		public bool CheckStationnaryGradientNorm (double normDiff) 
		{
			bool test = normDiff < gradientEpsilon;
			if (test)
				endCriteria = CriteriaType.statGd;
			return test;
		}

		public bool CheckAccuracyGradientNorm(double norm) 
		{
			bool test = norm < gradientEpsilon;
			if (test)
				endCriteria = CriteriaType.statGd;
			return test;
		}


		#region Implementation (ICloneable)

		object ICloneable.Clone()
		{
			return this.Clone();
		}

		#endregion


	}
}
