// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: LeastSquareFunction.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:31  jens
 *  Initial import
 *
 *  Revision 1.4  2002/10/15 13:20:40  stochastix
 *  now using dotNum
 *
 *  Revision 1.3  2002/08/30 18:21:11  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.2  2002/08/15 18:14:20  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.1  2002/08/07 19:14:59  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Numerics;
using Stochastix.Utilities;


namespace Stochastix.QuantLib.Optimization
{
	/// <summary>
	/// Design a least square function as a cost function using the
	/// interface provided by LeastSquareProblem class.
	/// </summary>
	/// <remarks>
	/// Array vector class requires function DotProduct() that computes
	/// dot product and - operator.
	/// M matrix class requires function transpose() that computes
	/// transpose and * operator with vector class.
	/// </remarks>
	[ RcsId("$Id: LeastSquareFunction.cs,v 1.1.1.1 2003/02/01 19:01:31 jens Exp $") ] 
	[ ComVisible(false) ]
	public class LeastSquareFunction : CostFunction 
	{
		//! Default constructor
		public LeastSquareFunction (LeastSquareProblem lsp)
		{
			this.lsp = lsp;
		}

		//! least square problem
		private LeastSquareProblem lsp;

		/// <summary>
		/// Compute value of the least square function value in x.
		/// </summary>
		/// <param name="x"></param>
		/// <returns></returns>
		override public double Value(Vector x)
		{
			// size of target and function to fit vectors
			Vector target = new Vector(lsp.Count);
			Vector fct2fit = new Vector(lsp.Count);
			// compute its values
			lsp.TargetAndValue(x, target, fct2fit);
			// do the difference
			target.Subtract(fct2fit);
			// and compute the scalar product (square of the norm)
			return Vector.DotProduct(target, target);
		}

		/// <summary>
		/// Compute grad_f, the first derivative of
		///	the least square function with respect to x.
		/// </summary>
		/// <param name="grad_f">On return, it contains the gradient.</param>
		override public void Gradient(Vector grad_f, /*const*/ Vector x)
		{
            // size of target and function to fit vectors
			Vector target = new Vector(lsp.Count);
			Vector fct2fit = new Vector(lsp.Count);
            // size of gradient matrix
            Matrix grad_fct2fit = new Matrix(lsp.Count, x.Count);
            // compute its values
            lsp.TargetValueAndGradient(x, grad_fct2fit, target, fct2fit);
            // do the difference
            target.Subtract(fct2fit);
            // compute derivative
            grad_f = -2.0 * (Matrix.Transpose(grad_fct2fit) * target);
		}

		/// <summary>
		/// Compute value and gradient of the least square function.
		/// </summary>
		/// <param name="grad_f"></param>
		/// <param name="x"></param>
		/// <returns></returns>
		override public double ValueAndGradient(Vector grad_f,
			/*const*/ Vector x)
		{
			// size of target and function to fit vectors
			Vector target = new Vector(lsp.Count);
			Vector fct2fit = new Vector(lsp.Count);
			// size of gradient matrix
			Matrix grad_fct2fit = new Matrix(lsp.Count, x.Count);
			// compute its values
			lsp.TargetValueAndGradient(x, grad_fct2fit, target, fct2fit);
			// do the difference
			target.Subtract(fct2fit);
			// compute derivative
			grad_f = -2.0 * (Matrix.Transpose(grad_fct2fit) * target);
            // and compute the scalar product (square of the norm)
            return Vector.DotProduct(target, target);
		}
	}
}
