// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: SteepestDescent.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:32  jens
 *  Initial import
 *
 *  Revision 1.4  2002/10/15 13:20:40  stochastix
 *  now using dotNum
 *
 *  Revision 1.3  2002/08/30 18:21:11  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.2  2002/08/15 18:14:20  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.1  2002/08/07 19:14:59  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Numerics;
using Stochastix.Utilities;
using Stochastix.QuantLib.Resources;


namespace Stochastix.QuantLib.Optimization
{
	/// <summary>
	/// Multi-dimensional Steepest Descent class.
	/// </summary>
	/// <remarks>
	/// User has to provide line-search method and
	/// optimization end criteria.
	/// 
	/// Search direction = - f'(x)
	/// </remarks>
	[ RcsId("$Id: SteepestDescent.cs,v 1.1.1.1 2003/02/01 19:01:32 jens Exp $") ] 
	[ ComVisible(false) ]
	[ Description("Multi-dimensionnal Steepest Descent class.") ]
	public class SteepestDescent : Method 
	{
		//! 
		/// <summary>
		/// Default constructor.
		/// </summary>
		public SteepestDescent()
			: this( new ArmijoLineSearch() )
		{}

		/// <summary>
		/// Initialization constructor.
		/// </summary>
		/// <param name="lineSearch">Line search algorithm.</param>
		public SteepestDescent(LineSearch lineSearch)
			: base()
		{
			this.lineSearch = lineSearch;
		}

		/// <summary>
		/// Line search algorithm.
		/// </summary>
		private LineSearch lineSearch;

		/// <summary>
		/// Minimize the optimization problem <paramref name="P"/>.
		/// </summary>
		/// <param name="P"></param>
		override public void Minimize(Problem P)
		{
			// classical initial value for line-search step
			double t = 1.0;

			// Set gold at the size of the optimization problem search direction
			Vector gold = new Vector(SearchDirection.Count);
			Vector gdiff = new Vector(SearchDirection.Count);
			FunctionValue = P.ValueAndGradient(gold, X);
			SearchDirection = -gold;
			GradientNormValue = Vector.DotProduct(gold, gold);
			// function and squared norm of gradient values;
			double normdiff = Math.Sqrt(GradientNormValue);

			for( bool end=true; !end; Iterations++)
			{
				// Linesearch
				t = lineSearch.Search(P, t);

				if (!lineSearch.Succeeded) 
					throw new ApplicationException(
						Strings.Get( GetType(), "OptLineSrch" ) );

				// End criteria
				end = EndCriteria.Check(
					Iterations, FunctionValue, 
					Math.Sqrt(GradientNormValue), 
					lineSearch.LastFunctionValue,
					Math.Sqrt(lineSearch.LastGradientNorm2), 
					normdiff);

				// Updates
				// New point
				X = lineSearch.LastX;
				// New function value
				FunctionValue = lineSearch.LastFunctionValue;
				// New gradient and search direction vectors
				gdiff = gold - lineSearch.LastGradient;
				normdiff = Math.Sqrt(Vector.DotProduct(gdiff, gdiff));
				gold = lineSearch.LastGradient.Clone();
				SearchDirection = -gold;
				// New gradient squared norm
				GradientNormValue = lineSearch.LastGradientNorm2;
			}
		}
	}
}
