// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: EuropeanAnalyticalEngine.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:32  jens
 *  Initial import
 *
 *  Revision 1.3  2002/08/30 18:21:12  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.2  2002/08/15 18:14:21  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.1  2002/08/07 19:14:59  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using System.Data;
using System.Diagnostics;
using Stochastix.Utilities;
using Stochastix.QuantLib.Instruments;
using CUM = Stochastix.Numerics.Distributions.CumulativeNormalDistribution;

namespace Stochastix.QuantLib.PricingEngines
{
	/// <summary>
	/// European option engine using analytic formulas.
	/// </summary>
	[ RcsId("$Id: EuropeanAnalyticalEngine.cs,v 1.1.1.1 2003/02/01 19:01:32 jens Exp $") ] 
	[ ComVisible(false) ]
	[ Description("European option engine using analytic formulas.") ]
	public class EuropeanAnalyticalEngine : VanillaEngine
	{

		#region Implementation of IPricingEngine
		public override string UniqueId
		{
			get { return GetType().Name; }
		}

		public override void Calculate(DataRow args, DataRow results)
		{
			// parse Arguments from DataRow
			VanillaOptionArguments voa = new VanillaOptionArguments(args);

			double dividendDiscount = Math.Exp(
				- voa.dividendYield * voa.residualTime );
			double riskFreeDiscount = Math.Exp(
				- voa.riskFreeRate * voa.residualTime );
			double stdDev = voa.volatility * Math.Sqrt(voa.residualTime);

			double D1 = Math.Log( voa.underlying/voa.strike) / stdDev +
				stdDev/2.0 + (voa.riskFreeRate-voa.dividendYield) *
				voa.residualTime/stdDev;
			double D2 = D1-stdDev;

			double alpha, beta, NID1;
			switch (voa.optionType) 
			{
				case Option.Type.Call:
					alpha = CUM.Function(D1);
					beta  = CUM.Function(D2);
					NID1  = CUM.FunctionDerivative(D1);
					break;
				case Option.Type.Put:
					alpha = CUM.Function(D1)-1.0;
					beta  = CUM.Function(D2)-1.0;
					NID1  = CUM.FunctionDerivative(D1);
					break;
				case Option.Type.Straddle:
					alpha = 2.0*CUM.Function(D1)-1.0;
					beta  = 2.0*CUM.Function(D2)-1.0;
					NID1  = 2.0*CUM.FunctionDerivative(D1);
					break;
				default:
					// can't happen since args are validated before
					Debug.Fail("Illegal option type");
					return;
			}

			results["value"] = voa.underlying * dividendDiscount * alpha -
				voa.strike * riskFreeDiscount * beta;

			//  only fill if column is in result row
			if( results.Table.Columns.IndexOf("delta") >= 0)
				results["delta"] = dividendDiscount * alpha;
			if( results.Table.Columns.IndexOf("gamma") >= 0)
				results["gamma"] = NID1 * dividendDiscount /
					(voa.underlying * stdDev);
			if( results.Table.Columns.IndexOf("theta") >= 0)
				results["theta"] = - voa.underlying * NID1 *
					voa.volatility * dividendDiscount /
					(2.0 * Math.Sqrt(voa.residualTime)) +
					voa.dividendYield * voa.underlying *
					alpha * dividendDiscount -
					voa.riskFreeRate * voa.strike *
					riskFreeDiscount * beta;
			if( results.Table.Columns.IndexOf("rho") >= 0)
				results["rho"] = voa.residualTime * riskFreeDiscount *
					voa.strike * beta;
			if( results.Table.Columns.IndexOf("dividendRho") >= 0)
				results["dividendRho"] = - voa.residualTime *
					dividendDiscount * voa.underlying * alpha;
			if( results.Table.Columns.IndexOf("vega") >= 0)
				results["vega"] = voa.underlying * NID1 *
					dividendDiscount * Math.Sqrt(voa.residualTime);
		}
		#endregion

	}
}
