// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: DiscountCurve.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:37  jens
 *  Initial import
 *
 *  Revision 1.7  2002/08/30 18:21:12  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.6  2002/08/15 18:14:21  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.5  2002/08/07 19:15:00  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.4  2002/07/31 18:47:47  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.3  2002/07/24 17:04:26  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.2  2002/07/24 14:42:42  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Collections;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Numerics.Solvers1D;
using Stochastix.Business;
using Stochastix.Utilities;
using Stochastix.QuantLib.Resources;
using Stochastix.Numerics.Interpolations;

namespace Stochastix.QuantLib.TermStructures
{
	/// <summary>
	/// Term structure based on loglinear interpolation of discount factors.
	/// </summary>
	/// <remarks>
	/// Loglinear interpolation guarantees piecewise constant forward
	/// rates.
	/// <para>
	/// Rates are assumed to be annual continuos compounding.
	/// </para>
	/// </remarks>
	[ RcsId("$Id: DiscountCurve.cs,v 1.1.1.1 2003/02/01 19:01:37 jens Exp $") ] 
	[ ComVisible(false) ]
	public class DiscountCurve : DiscountStructure, ITermStructure
	{
		// data members
		private IDayCounter dayCounter;
		private DateTime todaysDate;
		private IList dates;
		private double[] times;
		private IList discounts;
		private Interpolation interpolation;

		public DiscountCurve(DateTime todaysDate, IList dates, IList discounts,
			IDayCounter dayCounter)
		{
			if( dates.Count <= 1 )
				throw new ArgumentException( Strings.Get( GetType(),
					"TODO: too few dates"), "dates");

			if( dates.Count != discounts.Count)
				throw new ArgumentException( Strings.Get( GetType(),
					"TODO: dates/Discounts mismatch") );

			if( (double)discounts[0] != 1.0 )
				throw new ArgumentException( Strings.Get( GetType(),
					"TODO: the first discount must be == 1.0 " +
					"to flag the corrsponding date as settlement date" ), "discounts" );

			this.todaysDate = todaysDate.Date;	// normalize dates
			this.dayCounter = dayCounter;
			this.dates = dates;
			this.discounts = discounts;

			this.times = new double[dates.Count];
			for( int i=1; i<times.Length; i++)
			{
				if( (DateTime)dates[i] <= (DateTime)dates[i-1] )
					throw new ArgumentException( Strings.Get( GetType(),
						"TODO: invalid date" ), "dates" );
				
				// this check is repeated in LogLinearInterpolation
				if( (double)discounts[i] <= 0.0 )
					throw new ArgumentException( Strings.Get( GetType(),
						"TODO: invalid discount, must be positive." ), "discounts" );
				
				times[i] = dayCounter.YearFraction( (DateTime)dates[0], (DateTime)dates[i] );
			}

			interpolation = new LogLinearInterpolation( times, discounts);

		}

		public int Count 
		{ 
			get { return times.Length; }
		}

		public IList Times 
		{ 
			get { return times; }
		}

		public IList Dates 
		{ 
			get { return dates; }
		}


		/// <summary>
		/// Calculates the discount factor.
		/// </summary>
		/// <param name="time">Time from reference.</param>
		/// <param name="extrapolate"></param>
		/// <returns>The discount factor at the given time.</returns>
		override protected double discountImpl(double time, bool extrapolate)
		{
			if( time<0.0 )
				throw new ArgumentException( Strings.Get( GetType(),
					"TODO: negative time not allowed" ), "time" );
			return interpolation.Value(time, extrapolate);
		}

		/// <summary>
		/// A stringified representation of this term structure.
		/// </summary>
		/// <returns>A String representing the object.</returns>
		override public String ToString() 
		{
			return String.Format(
				"DiscountCurve (todaysDate={0:d} #segments={1})",	
				TodaysDate, times.Length-1
				);
		}

		/// <summary>
		/// The latest time for which the curve can return rates.
		/// </summary>
		override public double MaxTime 
		{ 
			get { return times[times.Length-1]; }
		}

		/// <summary>
		/// The latest date for which the curve can return rates.
		/// </summary>
		override public DateTime MaxDate 
		{ 
			get { return (DateTime)dates[dates.Count-1]; }
		}

		/// <summary>
		/// Today's date or better the "as-of date" of this curve.
		/// </summary>
		override public DateTime TodaysDate 
		{
			get { return todaysDate; }
		}

		/// <summary>
		/// The reference date of this curve.
		/// </summary>
		override public DateTime ReferenceDate 
		{
			get { return (DateTime)dates[0]; }
		}

		/// <summary>
		/// The <see cref="IDayCounter"/> used for internal date/time conversions.
		/// </summary>
		override public IDayCounter DayCounter 
		{
			get { return dayCounter; }
		}

	}
}
