// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: ForwardRateStructure.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:37  jens
 *  Initial import
 *
 *  Revision 1.4  2002/08/30 18:21:12  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.3  2002/07/24 17:04:26  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.2  2002/07/24 14:42:42  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using Stochastix.Utilities;
using Stochastix.QuantLib.Resources;

namespace Stochastix.QuantLib.TermStructures
{
	/// <summary>
	/// This abstract class acts as an adapter to TermStructure allowing the
	/// programmer to implement only the <c>forwardImpl()</c> 
	/// method in derived classes.
	/// </summary>
	/// <remarks>
	/// Rates are assumed to be annual continuos compounding.
	/// </remarks>
	[ RcsId("$Id: ForwardRateStructure.cs,v 1.1.1.1 2003/02/01 19:01:37 jens Exp $") ] 
	[ ComVisible(false) ]
	public abstract class ForwardRateStructure : TermStructure 
	{
		/// <summary>
		/// The zero yield rate calculated from the instantaneous forward rate.
		/// </summary>
		/// <remarks>
		/// This is just a default, highly inefficient implementation. 
		/// Derived classes should implement their own zeroYield method.
		/// </remarks>
		/// <param name="time">Time from reference.</param>
		/// <param name="extrapolate"></param>
		/// <returns>The zero yield at the given time</returns>
		override protected double zeroYieldImpl(double time, bool extrapolate) 
		{
			if( time<0.0 )
				throw new ArgumentException( Strings.Get( GetType(),
					"TODO: negative time not allowed" ), "time" );
			if (time == 0.0) return forwardImpl(0.0, false);
			double sum = 0.5*forwardImpl(0.0, false);
			const int N = 1000;
			double dt = time/N;
			for (double i=dt; i<time; i+=dt)
				sum += forwardImpl(i, extrapolate);
			sum += 0.5*forwardImpl(time, extrapolate);
			return sum*dt/time;
		}

		/// <summary>
		/// The discount factor calculated from the instantaneous forward rate.
		/// </summary>
		/// <param name="time">Time from reference.</param>
		/// <param name="extrapolate"></param>
		/// <returns>The discount factor at the given time</returns>
		override protected double discountImpl(double time, bool extrapolate) 
		{
			double r = zeroYieldImpl(time, extrapolate);
			return Math.Exp(-r*time);
		}

	}

}
