// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: ImpliedTermStructure.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:37  jens
 *  Initial import
 *
 *  Revision 1.5  2002/08/30 18:21:12  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.4  2002/07/24 17:04:26  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.3  2002/07/24 14:42:42  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;
using Stochastix.QuantLib.MarketElements;
using Stochastix.Business;
using Stochastix.Utilities;

namespace Stochastix.QuantLib.TermStructures
{
	
	/// <summary>
	/// Implied term structure at a given date in the future.
	/// </summary>
	/// <remarks>
	/// This term structure will remain linked to the original structure,
	/// i.e., any changes in the latter will be reflected in this 
	/// structure as well.
	/// </remarks>
	[ RcsId("$Id: ImpliedTermStructure.cs,v 1.1.1.1 2003/02/01 19:01:37 jens Exp $") ] 
	[ ComVisible(true) ]
	[ ClassInterface(ClassInterfaceType.None) ]
	public class ImpliedTermStructure : DiscountStructure
	{
		/// <summary>
		/// The original <see cref="TermStructure"/> which will
		/// remain linked.
		/// </summary>
		private ITermStructure originalCurve;

		private DateTime todaysDate, referenceDate;

		/// <summary>
		/// Link this ImpliedTermStructure to another
		/// <see cref="TermStructure"/> with a future date.
		/// </summary>
		/// <param name="originalCurve">The original <see cref="TermStructure"/>.</param>
		public ImpliedTermStructure( ITermStructure originalCurve, 
			DateTime todaysDate, DateTime referenceDate) 
		{
			if( originalCurve == null)
				throw new ArgumentNullException("originalCurve", 
					"The original curve can not be null.");
			this.originalCurve = originalCurve;
			RegisterWith(originalCurve);

			// normalize dates
			this.todaysDate = todaysDate.Date;	
			this.referenceDate = referenceDate.Date;
			// TODO: check if referenceDate <= orinalCurve.MaxDate
		}

		/// <summary>
		/// The linked <see cref="TermStructure"/>.
		/// </summary>
		public ITermStructure OriginalCurve 
		{
			get { return originalCurve; }
/*	allowing this  means we have to also re-check the new dates etc.
			set 
			{
				if( value == null)
					throw new ArgumentNullException(); 
				UnregisterWith(originalCurve); // detach event handler
				originalCurve = value;
				RegisterWith(marketElement); // re-register
				OnChanged();
			}
*/
		}

		/// <summary>
		/// Calculates the discount factor.
		/// </summary>
		/// <param name="time">Time from reference.</param>
		/// <param name="extrapolate"></param>
		/// <returns>The discount factor at the given time</returns>
		override protected double discountImpl(double time, bool extrapolate)
		{
			/* t is relative to the current reference date
			   and needs to be converted to the time relative
			   to the reference date of the original curve */
			time = time + DayCounter.YearFraction(
				originalCurve.ReferenceDate, referenceDate);
			// evaluationDate cannot be an extrapolation
			/* discount at evaluation date cannot be cached
			   since the original curve could change between
			   invocations of this method */
			return originalCurve.Discount(time, extrapolate) /
				originalCurve.Discount(referenceDate, false);
		}

		/// <summary>
		/// A stringified representation of this term structure.
		/// </summary>
		/// <remarks>
		/// This method is used for output and comparison between indexes.
		///	It is <b>not</b> meant to be used for writing switch-on-type code.
		/// </remarks>
		/// <returns>A String representing the object.</returns>
		override public String ToString() 
		{
			return String.Format(
				"ImpliedTermStructure (new referenceDate={0:d}, originalCurve='{1}')",
				this.ReferenceDate, this.OriginalCurve
				);
		}

		/// <summary>
		/// The latest date for which the curve can return rates.
		/// </summary>
		override public DateTime MaxDate 
		{ 
			get { return originalCurve.MaxDate; }
		}

		/// <summary>
		/// The latest time for which the curve can return rates.
		/// </summary>
		override public double MaxTime
		{ 
			get 
			{
				return DayCounter.YearFraction(
					  referenceDate, originalCurve.MaxDate); 
			}
		}

		/// <summary>
		/// Today's date or better the "as-of date" of this curve.
		/// </summary>
		override public DateTime TodaysDate 
		{
			get { return todaysDate; }
		}

		/// <summary>
		/// The reference date of this curve.
		/// </summary>
		override public DateTime ReferenceDate 
		{
			get { return referenceDate; }
		}

		/// <summary>
		/// The <see cref="IDayCounter"/> used for internal date/time conversions.
		/// </summary>
		override public IDayCounter DayCounter 
		{
			get { return originalCurve.DayCounter; }
		}

	};


}
