// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: PiecewiseFlatForward.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:38  jens
 *  Initial import
 *
 *  Revision 1.7  2002/08/30 18:21:12  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.6  2002/08/15 18:14:21  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.5  2002/08/07 19:15:00  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.4  2002/07/31 18:47:47  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.3  2002/07/24 17:04:26  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.2  2002/07/24 14:42:42  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Collections;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Numerics.Solvers1D;
using Stochastix.Business;
using Stochastix.Utilities;
using Stochastix.QuantLib.Resources;

namespace Stochastix.QuantLib.TermStructures
{
	/// <summary>
	/// Piecewise flat forward term structure.
	/// </summary>
	/// <remarks>
	/// This term structure is bootstrapped on a number of interest rate
	/// instruments which are passed as a vector of RateHelper
	/// instances. Their maturities mark the boundaries of the flat
	/// forward segments.
	/// 
	/// The values of the forward rates for each segment are determined
	/// sequentially starting from the earliest period to the latest.
	/// The value for each segment is chosen so that the instrument whose
	/// maturity marks the end of such segment is correctly repriced on
	/// the curve.
	/// 
	/// Rates are assumed to be annual continuos compounding.
	/// 
	/// The bootstrapping algorithm will raise an exception if
	/// any two instruments have the same maturity date.
	/// </remarks>
	[ RcsId("$Id: PiecewiseFlatForward.cs,v 1.1.1.1 2003/02/01 19:01:38 jens Exp $") ] 
	[ ComVisible(false) ]
	public class PiecewiseFlatForward : TermStructure, ITermStructure
	{
		// This class is derived directly from term structure since we are
		// rewriting all of forward, discount and zeroYield to take advantage
		// of its own internal structure. 

		// data members
		private IDayCounter dayCounter;
		private DateTime todaysDate;
		private DateTime referenceDate;
		private bool needsBootstrap;
		private IList instruments;
		private int segments=0;		// bootstrapped segments
		private DateTime[] dates;
		private double[] forwards;
		private double[] times;
		private double[] discounts;
		private double[] zeroYields;
		private double accuracy;

		/// <summary>
		///	Built a piecewise flat forward from a SORTED
		///	<see cref="System.Collections.IList"/> of 
		///	<see cref="RateHelper"/>s.
		/// </summary>
		/// <param name="todaysDate"></param>
		/// <param name="referenceDate"></param>
		/// <param name="instruments">
		///		A <b>sorted</b> <see cref="System.Collections.IList"/>
		///		of <see cref="RateHelper"/>s.
		///	</param>
		/// <param name="dayCounter"></param>
		/// <param name="accuracy">eg. 1.0e-12</param>
		// sorted IList of instruments (ArrayList, Array, ...)
		public PiecewiseFlatForward(DateTime todaysDate, DateTime referenceDate,
			IList instruments, IDayCounter dayCounter, double accuracy)
		{
			if (instruments.Count==0) 
				throw new ArgumentException( 
					Strings.Get(GetType(), "PieceNoInst"), "instruments" );

			this.todaysDate = todaysDate.Date;	// normalize dates
			this.referenceDate = referenceDate.Date;
			this.instruments = instruments;
			this.needsBootstrap = true;
			this.dayCounter = dayCounter;
			this.accuracy = accuracy;

			// sort rate helpers
			foreach( IRateHelper rateHelper in instruments )
				rateHelper.TermStructure = this;
			if( instruments is ArrayList )			// TODO: worth the check???
				((ArrayList)instruments).Sort();
			else
				ArrayList.Adapter(instruments).Sort();

#if DEBUG
			// check that there is no instruments with the same maturity
			// RateHelper.IComparable already enforces strict ordering
			DateTime dt = DateTime.MinValue; // a low value
			foreach( IRateHelper rateHelper in instruments )
			{
				DateTime maturity = rateHelper.Maturity();
				if( maturity > dt )
					dt = maturity;
				else
					throw new ArgumentException( 
						Strings.Format(GetType(), "PieceMaturity", maturity, dt ), "instruments");
			}
#endif

			// observe instruments
			RegisterWith(instruments);

		}

		PiecewiseFlatForward( DateTime todaysDate, 
			DateTime[] dates, double[] forwards, IDayCounter dayCounter)
		{
			this.todaysDate = todaysDate.Date;	
			this.referenceDate = ((DateTime)dates[0]).Date;

			if( dates.Length == 0 ) 
				throw new ArgumentException( 
					Strings.Get(GetType(), "PieceNoDates" ), "dates");
			if( dates.Length != forwards.Length ) 
				throw new ApplicationException( Strings.Get(GetType(), "PieceMismatch" ));

			this.segments = dates.Length-1;
			this.dates = (DateTime[])dates.Clone();
			this.forwards = (double[])forwards.Clone();
			this.needsBootstrap = false;
			this.dayCounter = dayCounter;

			times = new double[dates.Length];
			times[0] = 0.0;
			discounts = new double[dates.Length];
			discounts[0] = 1.0;
			zeroYields = new double[dates.Length];
			zeroYields[0] = (double)forwards[0];

			for (int i=1; i<dates.Length; i++) 
			{
				times[i] = dayCounter.YearFraction(referenceDate, 
					(DateTime)dates[i]);
				discounts[i] = discounts[i-1] * 
					Math.Exp( -(double)forwards[i] * (times[i]-times[i-1]) );
				zeroYields[i] = -Math.Log(discounts[i]) / times[i];
			}
		}

		public int Count 
		{ 
			get { return segments+1; }
		}

		public IList Times 
		{ 
			get 
			{
				if (needsBootstrap) bootstrap();
				return (IList)times.Clone();
			}
		}

		public IList Dates 
		{ 
			get 
			{
				if (needsBootstrap) bootstrap();
				return (IList)dates.Clone();
				}
		}

        private void bootstrap() 
		{
            // prevent recursively calling bootstrap() when the
            // term structure methods are called by the rate helpers
            needsBootstrap = false;
            try {
				int size = instruments.Count+1 ;
				dates = new DateTime[size];
				dates[0] = referenceDate;
				times = new double[size];
				// times[0] = 0.0;
				discounts = new double[size];
				discounts[0] = 1.0;
                forwards = new double[size];
				zeroYields = new double[size];
				// dummy values at reference (origin: FFObjFunction ctor)
				// forwards[0] = 0.0;
				// zeroYields[0] = 0.0;

                // the choice of the solver determines whether the
                // accuracy is on the discount or the instrument rate
                ISolver1D solver = new Brent();

                // bootstrapping loop
				foreach( IRateHelper instrument in instruments )
				{	
					// already done in ctor
					// instrument.TermStructure = this;

                    double guess = instrument.DiscountGuess();
                    if (guess == double.MaxValue) {
                        if (segments > 0) {    // we can extrapolate
                            guess = Discount( instrument.Maturity(), true);
                        } else {        // any guess will do
                            guess = 0.9;
                        }
                    }
                    // bracket
                    double min = accuracy*10e-4;
					double max = discounts[segments];
					segments++;
                    try
					{
                        solver.Solve(
							new FFObjFunction(this, instrument, segments),
                            accuracy, guess, min, max
						);
                    } catch (System.Exception innerException) {
					
						// PieceBoot = Could not bootstrap curve 
						//	(segment {0} of {1}, last forward = {2}, 
						//	last discount = {3}, last zero-yield = {4}, 
						//	last guess was {5}, error generated by '{6}').
						throw new ApplicationException( Strings.Format (
							GetType(), "PieceBoot", new Object[] 
								{ 
									segments, size-1,
									forwards[segments-1], discounts[segments-1], 
									zeroYields[segments-1], guess, innerException.Message
								}
							), innerException);
                    }
                }
            } catch {
                // signal incomplete state
                needsBootstrap = true;
                throw;	// rethrow
            }
        }

		/// <summary>
		/// Returns times.BinarySearch() with an additional check for interpolation.
		/// NOTE: This is different from QuantLib implementation!!
		/// </summary>
		private int referenceNode(double time, bool extrapolate) 
		{
			// this is just a shortcut for a value also returned by BinarySearch()
			if( time == 0.0) return 0;

			double tmax = times[segments]; // [times.Length-1]
			if( time < 0.0 || ( time > tmax && !extrapolate) ) 
				throw new ArgumentOutOfRangeException(
					"time", time, Strings.Format( GetType(), "PieceRange", time, 0.0, tmax) );

			if (time > tmax)
				return ~(segments);	// force interpolation above last segment

			// j = times.BinarySearch(time) returns the 0-based index position for
			// an exact match, or the in-between position as binary complement.
			// eg. j == 2  -->  time = times[2]
			//    ~j == 2  -->  times[1] < time < times[2]
			//    ~j == times.Count  --> times > times[i] for all i
			// see also QuantLib.Tests.TestPiecewiseFlatForward.TestBinarySearch()
			return Array.BinarySearch(times, 0, segments+1, time ); 
			// return ( j>=0 ? j : ~j );
/*
			std::vector<Time>::const_iterator i=times_.begin(),
							   j=times_.end(), k;
			while (j-i > 1) 
			{
				k = i+(j-i)/2;
				if (t <= *k)
					j = k;
				else
					i = k;
			}
			return (j-times.begin());
*/
		}


		/// <summary>
		/// Calculates the zero yield rate.
		/// </summary>
		/// <param name="time">Time from reference.</param>
		/// <param name="extrapolate"></param>
		/// <returns>The zero yield at the given time.</returns>
		override protected double zeroYieldImpl(double time, bool extrapolate) 
		{
			if (needsBootstrap) bootstrap();

			int n = referenceNode(time, extrapolate);
			if( n>=0 )	// exact match
			{
				return (double)zeroYields[n];
			} 
			else 
			{
				n = ~n;	// n is now the index of the next-larger value
				double tn = (double)times[n-1];
				return ( (double)zeroYields[n-1]*tn + 
							(double)forwards[n]*(time-tn) ) / time;
			}
		}

		/// <summary>
		/// Calculates the discount factor.
		/// </summary>
		/// <param name="time">Time from reference.</param>
		/// <param name="extrapolate"></param>
		/// <returns>The discount factor at the given time.</returns>
		override protected double discountImpl(double time, bool extrapolate)
		{
			if (needsBootstrap) bootstrap();

			int n = referenceNode(time, extrapolate);
			if (n>=0) 
			{
				return (double)discounts[n];
			} 
			else 
			{
				n = ~n;	// n is now the index of the next-larger value
				return (double)discounts[n-1] *
					Math.Exp(-(double)forwards[n] * (time-(double)times[n-1]));
			}
		}

		/// <summary>
		/// Calculates the instantaneous forward rate.
		/// </summary>
		/// <param name="time">Time from reference.</param>
		/// <param name="extrapolate"></param>
		/// <returns>The forward rate at the given time.</returns>
		override protected double forwardImpl(double time, bool extrapolate) 
		{
			if (needsBootstrap) bootstrap();
			int n = referenceNode(time, extrapolate);
			return (double)forwards[ ( n>=0 ? n : ~n ) ];
		}

		/// <summary>
		/// A stringified representation of this term structure.
		/// </summary>
		/// <returns>A String representing the object.</returns>
		override public String ToString() 
		{
			return String.Format(
				"PiecewiseFlatForward (todaysDate={0:d} #segments={1})",	
				TodaysDate, segments
				);
		}

		/// <summary>
		/// The latest time for which the curve can return rates.
		/// </summary>
		override public double MaxTime 
		{ 
			get 
			{
				if (needsBootstrap) bootstrap();
				return (double)times[segments];
			}
		}

		/// <summary>
		/// The latest date for which the curve can return rates.
		/// </summary>
		override public DateTime MaxDate 
		{ 
			get 
			{
				if (needsBootstrap) bootstrap();
				return (DateTime)dates[segments];
			}
		}

		/// <summary>
		/// Today's date or better the "as-of date" of this curve.
		/// </summary>
		override public DateTime TodaysDate 
		{
			get { return todaysDate; }
		}

		/// <summary>
		/// The reference date of this curve.
		/// </summary>
		override public DateTime ReferenceDate 
		{
			get { return referenceDate; }
		}

		/// <summary>
		/// The <see cref="IDayCounter"/> used for internal date/time conversions.
		/// </summary>
		override public IDayCounter DayCounter 
		{
			get { return dayCounter; }
		}

		override protected void Notify(object sender, EventArgs e) 
		{ 
			needsBootstrap = true;
			base.Notify(sender, e);
		}


		/// <summary>
		/// Objective function for solver (inner class).
		/// </summary>
		private class FFObjFunction : ObjectiveFunction , IObjectiveFunction
		{
			private PiecewiseFlatForward curve;
			private IRateHelper rateHelper;
			private int segment;

			public FFObjFunction(PiecewiseFlatForward curve,
				IRateHelper rateHelper, int segment)
			{
				this.curve = curve;
				this.rateHelper = rateHelper;
				this.segment = segment;
            
				// extend curve to next point
				curve.dates[segment] = rateHelper.Maturity();
                curve.times[segment] = curve.DayCounter.YearFraction(
					curve.ReferenceDate, curve.dates[segment]);
				// dummy values - will be reset by operator()
				// we skip these since arrays[] are initialized per default
				// curve.discounts[segment] = 0.0;
				// curve.forwards[segment] = 0.0;
				// curve.zeroYields[segment] = 0.0;
			}

			override public double Value(double discount)
			{
				curve.discounts[segment] = discount;
				curve.zeroYields[segment] =
					-Math.Log(discount) / curve.times[segment];
				curve.forwards[segment] =
					Math.Log(curve.discounts[segment-1]/discount) /
					(curve.times[segment] - curve.times[segment-1]);
				if (segment == 1) 
				{
					curve.forwards[0] = curve.zeroYields[0] =
						curve.forwards[1];
				}
				return rateHelper.QuoteError();
			}

		}

	}
}
