// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: DepositRateHelper.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:38  jens
 *  Initial import
 *
 *  Revision 1.5  2002/08/30 18:21:12  stochastix
 *  license and namespace tweaks
 *
 *  Revision 1.4  2002/07/24 17:04:26  stochastix
 *  Preparing for preview release
 *
 *  Revision 1.3  2002/07/24 14:42:42  stochastix
 *  Preparing for preview release
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;
using Stochastix.QuantLib.MarketElements;
using Stochastix.Business;
using Stochastix.Utilities;

namespace Stochastix.QuantLib.TermStructures
{
	/// <summary>
	/// Deposit rate.
	/// </summary>
	/// <remarks>
	/// This class assumes that today's date does not change
	/// between calls of setTermStructure().
	/// </remarks>
	[ RcsId("$Id: DepositRateHelper.cs,v 1.1.1.1 2003/02/01 19:01:38 jens Exp $") ] 
	[ ComVisible(false) ]
	public class DepositRateHelper : RateHelper, IRateHelper
	{

		public DepositRateHelper( double quote,
			int tenor, TimeUnit units, int settlementDays,
			IBusinessCalendar calendar, RollingConvention convention,
			IDayCounter dayCounter) : this(
			new SimpleMarketElement(quote), tenor, units, 
			settlementDays, calendar, convention, dayCounter)
		{}

		public DepositRateHelper( IMarketElement marketElement,
			int tenor, TimeUnit units, int settlementDays, 
			IBusinessCalendar calendar, RollingConvention convention,
			IDayCounter dayCounter) : base(marketElement)
		{
			this.tenor = tenor;
			this.units = units;
			this.settlementDays = settlementDays;
			this.calendar = calendar;
			this.convention = convention;
			this.dayCounter = dayCounter;
		}

		private int settlementDays;
		private int tenor;
		private TimeUnit units;
		private IBusinessCalendar calendar;
		private RollingConvention convention;
		private IDayCounter dayCounter;
		private DateTime settlementDate;
		private DateTime maturityDate;
		private double yearFraction;

		/// <summary>
		/// A stringified representation of the rate helper.
		/// </summary>
		/// <returns>A String representing the object.</returns>
		override public string ToString()
		{
			return String.Format(
				"DepositRateHelper (settlementDate={0:d}, tenor={1}, rate={2})",
				settlementDate, new Period(tenor, units), Quote);
		}


		/// <summary>
		/// Called everytime a new term structure is set. 
		/// </summary>
		/// <remarks>
		/// The new term structure is not null.
		/// </remarks>
		/// <param name="termStructure">The new term structure.</param>
		override protected void initializeHelper(ITermStructure termStructure)
		{
			settlementDate = calendar.Advance(
				termStructure.TodaysDate, settlementDays, TimeUnit.Days,
				RollingConvention.Following // TODO: always 'Following' ???
				);
			maturityDate = calendar.Advance(
				settlementDate, tenor, units, convention);
			yearFraction = dayCounter.YearFraction(settlementDate, maturityDate);
		}

		override public double ImpliedQuote() 
		{
			return ( TermStructure.Discount(settlementDate, false) /
					 TermStructure.Discount(maturityDate, false) - 1.0 )  / 
				yearFraction;
		}

		override public double DiscountGuess()
		{
			// extrapolation shouldn't be needed if the input 
			// makes sense but we'll play it safe
			return TermStructure.Discount(settlementDate, true) /
				(1.0 + Quote * yearFraction);
		}

		override public DateTime Maturity()
		{
			if( TermStructure == null) 
				; // throws an error if not
			return maturityDate;
		}

	}
}
