// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: BlackConstantVolatility.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:39  jens
 *  Initial import
 *
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Business;
using Stochastix.Business.DayCounters;
using Stochastix.Utilities;
using Stochastix.QuantLib.Resources;
using Stochastix.QuantLib.MarketElements;

namespace Stochastix.QuantLib.Volatilities
{

	// TODO: make COM available, late init: DC, MarketElement + vola assignable

	/// <summary>
	/// Constant Black volatility, no time-strike dependence.
	/// </summary>
	/// <remarks>
	/// This class implements the <see cref="IBlackVolTermStructure"/>
	/// interface for a constant Black volatility (no time/strike 
	/// dependence).
	/// <para>
	/// Volatilities are assumed to be expressed on an annual basis.
	/// </para>
	/// </remarks>
	[ RcsId("$Id: BlackConstantVolatility.cs,v 1.1.1.1 2003/02/01 19:01:39 jens Exp $") ] 
	[ ComVisible(true) ]
	public abstract class BlackConstantVolatility : BlackVolatilityTermStructure
	{
		/// <summary>
		/// Constructor for BlackConstantVolatility class.
		/// </summary>
		public BlackConstantVolatility(DateTime referenceDate, double volatility) 
			: this(referenceDate, volatility, Actual365.Instance)
		{}

		/// <summary>
		/// Constructor for BlackConstantVolatility class.
		/// </summary>
		public BlackConstantVolatility(DateTime referenceDate, double volatility, 
			IDayCounter dayCounter) : this(referenceDate, 
			new SimpleMarketElement(volatility), dayCounter)
		{}

		/// <summary>
		/// Constructor for BlackConstantVolatility class.
		/// </summary>
		public BlackConstantVolatility(DateTime referenceDate, 
			IMarketElement volatility, IDayCounter dayCounter) : base()
		{
			this.referenceDate = referenceDate;
			this.volatility = volatility;
			this.dayCounter = dayCounter;
			RegisterWith(volatility);
		}

		private DateTime referenceDate;
		private IMarketElement volatility;
		private IDayCounter dayCounter;


		/// <summary>
		/// Black present (a.k.a spot) volatility at a given time from reference.
		/// </summary>
		/// <param name="maturity">Time from reference.</param>
		/// <param name="strike"></param>
		/// <param name="extrapolate"></param>
		/// <returns>Volatility at the given time.</returns>
		public override double Volatility(double maturity, double strike, 
			bool extrapolate)
		{
			if( maturity < 0.0 )
				throw new ArgumentException( Strings.Get( GetType(),
					"TODO: negative time not allowed." ), "maturity" );
			return volatility.Value;
		}

		/// <summary>
		/// The latest date for which the term structure can return vols.
		/// </summary>
		/// <value><see cref="DateTime"/></value>
		public override DateTime MaxDate 
		{
			get { return DateTime.MaxValue; }
		}

		/// <summary>
		/// The reference date of this curve for which t=0.
		/// </summary>
		/// <value><see cref="DateTime"/></value>
		public override DateTime ReferenceDate 
		{
			get { return referenceDate; }
		}

		/// <summary>
		/// The <see cref="IDayCounter"/> used for internal date/time conversions.
		/// </summary>
		public override IDayCounter DayCounter 
		{
			get { return dayCounter; }
		}


	}
}
