// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: BlackVolTermStructure.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:39  jens
 *  Initial import
 *
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Business;
using Stochastix.Utilities;
using Stochastix.QuantLib.Resources;

namespace Stochastix.QuantLib.Volatilities
{

	/// <summary>
	/// Black volatility term structure abstract base class.
	/// </summary>
	/// <remarks>
	/// This abstract class defines the base of concrete
	/// black volatility term structures which will be derived from this one.
	/// <para>Volatilities are assumed to be expressed on an annual basis.</para>
	/// </remarks>
	[ RcsId("$Id: BlackVolTermStructure.cs,v 1.1.1.1 2003/02/01 19:01:39 jens Exp $") ] 
	[ ComVisible(false) ]
	public abstract class BlackVolTermStructure : Observable, IBlackVolTermStructure
	{
		/// <summary>
		/// Protected constructor for abstract base class.
		/// </summary>
		protected BlackVolTermStructure()
		{}


		/// <summary>
		/// Black present (a.k.a spot) volatility at a given time from reference.
		/// </summary>
		/// <remarks>
		/// This method must be overridden to implement the actual volatility
		/// calculation in derived classes.
		/// </remarks>
		/// <overloads>
		/// Black present (a.k.a spot) volatility.
		/// </overloads>
		/// <param name="maturity">Time from reference.</param>
		/// <param name="strike"></param>
		/// <param name="extrapolate"></param>
		/// <returns>Volatility at the given time.</returns>
		public abstract double Volatility( double maturity, double strike, bool extrapolate);

		/// <summary>
		/// Black present (a.k.a spot) variance at a given time from reference.
		/// </summary>
		/// <remarks>
		/// This method must be overridden to implement the actual variance
		/// calculation in derived classes.
		/// </remarks>
		/// <overloads>
		/// Black present (a.k.a spot) variance.
		/// </overloads>
		/// <param name="maturity">Time from reference.</param>
		/// <param name="strike"></param>
		/// <param name="extrapolate"></param>
		/// <returns>Variance at the given time.</returns>
		public abstract double Variance( double maturity, double strike, bool extrapolate);

		/// <summary>
		/// Black present (a.k.a spot) volatility at a given date.
		/// </summary>
		/// <param name="maturity">The <see cref="DateTime"/> to use for calculations.</param>
		/// <param name="strike"></param>
		/// <param name="extrapolate"></param>
		/// <returns>Volatility at the given date.</returns>
		public double Volatility( DateTime maturity, double strike, bool extrapolate)
		{
			return Volatility( date2time(maturity), strike, extrapolate);
		}

		/// <summary>
		/// Black present (a.k.a spot) variance at a given date.
		/// </summary>
		/// <param name="maturity">The <see cref="DateTime"/> to use for calculations.</param>
		/// <param name="strike"></param>
		/// <param name="extrapolate"></param>
		/// <returns>Variance at the given date.</returns>
		public double Variance( DateTime maturity, double strike, bool extrapolate)
		{
			return Variance( date2time(maturity), strike, extrapolate);
		}


		/// <summary>
		/// Convert a <see cref="DateTime"/> to continous time
		/// since reference.
		/// </summary>
		/// <param name="date">The <see cref="DateTime"/> to convert.</param>
		/// <returns>Continous time since reference date.</returns>
		protected double date2time(DateTime date) 
		{
			// TODO: check for valid dates eg. reference <= d <= maxdatetime
			// TODO: interpret certain dates (eg. serial <= 100) as time
			return DayCounter.YearFraction(ReferenceDate, date);
		}


		/// <summary>
		/// Black future (a.k.a. forward) volatility between given dates.
		/// </summary>
		/// <overloads>
		/// Black future (a.k.a. forward) volatility.
		/// </overloads>
		/// <param name="date1">The start date of the future period.</param>
		/// <param name="date2">The end date of the future period.</param>
		/// <param name="strike"></param>
		/// <param name="extrapolate"></param>
		/// <returns>Volatility between given dates.</returns>
		public double ForwardVolatility( DateTime date1, DateTime date2, 
			double strike, bool extrapolate)
		{
			return ForwardVolatility( date2time(date1), date2time(date1), 
				strike, extrapolate);
		}

		/// <summary>
		/// Black future (a.k.a. forward) volatility between given times.
		/// </summary>
		/// <param name="time1">The start time of the future period.</param>
		/// <param name="time2">The end time of the future period.</param>
		/// <param name="strike"></param>
		/// <param name="extrapolate"></param>
		/// <returns>Volatility between given times.</returns>
		public double ForwardVolatility( double time1, double time2, double strike, bool extrapolate)
		{
			if( time1 > time2 )
				throw new ArgumentException( Strings.Get( GetType(),
					"TODO: time can not flow backwards." ) );
			double var1 = Variance(time1, strike, extrapolate);
			if (time2==time1)
				time2 = time1 + 0.00001; // a small "time epsilon"
			return Math.Sqrt(
				( Variance(time1, strike, extrapolate) - var1 ) /
				( time2 - time1 ) );
		}


		/// <summary>
		/// Black future (a.k.a. forward) variance between given dates.
		/// </summary>
		/// <overloads>
		/// Black future (a.k.a. forward) variance.
		/// </overloads>
		/// <param name="date1">The start date of the future period.</param>
		/// <param name="date2">The end date of the future period.</param>
		/// <param name="strike"></param>
		/// <param name="extrapolate"></param>
		/// <returns>Variance between given dates.</returns>
		public double ForwardVariance( DateTime date1, DateTime date2, 
			double strike, bool extrapolate)
		{
			return ForwardVariance( date2time(date1), date2time(date1), 
				strike, extrapolate);
		}

		/// <summary>
		/// Black future (a.k.a. forward) variance between given times.
		/// </summary>
		/// <param name="time1">The start time of the future period.</param>
		/// <param name="time2">The end time of the future period.</param>
		/// <param name="strike"></param>
		/// <param name="extrapolate"></param>
		/// <returns>Volatility between given times.</returns>
		public double ForwardVariance( double time1, double time2, double strike, bool extrapolate)
		{
			if( time1 > time2 )
				throw new ArgumentException( Strings.Get( GetType(),
					"TODO: time can not flow backwards." ) );
			return Variance(time2, strike, extrapolate)
				 - Variance(time1, strike, extrapolate);
		}


		/// <summary>
		/// Partial derivative of Black volatility with respect to time.
		/// </summary>
		/// <param name="time"></param>
		/// <param name="strike"></param>
		/// <param name="extrapolate"></param>
		/// <returns></returns>
		public double TimeDerivative(double time, double strike, bool extrapolate)
		{
			const double dT = 1.0/365.0;
			return ( Volatility(time+dT, strike, extrapolate) -
				Volatility(time, strike, extrapolate) ) / dT;
		}
        
		/// <summary>
		/// Partial derivative of Black volatility with respect to strike.
		/// </summary>
		/// <param name="time"></param>
		/// <param name="strike"></param>
		/// <param name="extrapolate"></param>
		/// <returns></returns>
		public double StrikeDerivative(double time, double strike, bool extrapolate)
		{
			double dK = 1.0e-3 * strike;
			return ( Volatility(time, strike+dK, extrapolate) -
				Volatility(time, strike-dK, extrapolate) ) / (2*dK);
		}
                   
		/// <summary>
		/// Second-order derivative of Black volatility with respect to strike.
		/// </summary>
		/// <param name="time"></param>
		/// <param name="strike"></param>
		/// <param name="extrapolate"></param>
		/// <returns></returns>
		public double StrikeSecondDerivative(double time, double strike, bool extrapolate)
		{
			double dK = 1.0e-3 * strike;
			return ( Volatility(time, strike+dK, extrapolate) +
				Volatility(time, strike-dK, extrapolate) -
				2*Volatility(time, strike, extrapolate) ) / (dK*dK);
		}




		/// <summary>
		/// The latest time for which the term structure can return vols.
		/// </summary>
		public virtual double MaxTime 
		{
			get{ return date2time(MaxDate); }
		}
		
		/// <summary>
		/// The latest date for which the term structure can return vols.
		/// </summary>
		/// <value><see cref="DateTime"/></value>
		public abstract DateTime MaxDate 
		{
			get;
		}

		/// <summary>
		/// The reference date of this curve for which t=0.
		/// </summary>
		/// <value><see cref="DateTime"/></value>
		public abstract DateTime ReferenceDate 
		{
			get;
		}

		/// <summary>
		/// The <see cref="IDayCounter"/> used for internal date/time conversions.
		/// </summary>
		public abstract IDayCounter DayCounter 
		{
			get;
		}


	}
}
