// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: CapFlatVolatilityStructure.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:39  jens
 *  Initial import
 *
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Business;
using Stochastix.Utilities;

namespace Stochastix.QuantLib.Volatilities
{

	/// <summary>
	/// Cap/floor flat volatility structure.
	/// </summary>
	/// <remarks>
	/// This class is purely abstract and defines the interface of
	/// concrete structures which will be derived from this one.
	/// <para>Volatilities are assumed to be expressed on an annual basis.</para>
	/// </remarks>
	[ RcsId("$Id: CapFlatVolatilityStructure.cs,v 1.1.1.1 2003/02/01 19:01:39 jens Exp $") ] 
	[ ComVisible(false) ]
	public abstract class CapFlatVolatilityStructure : Observable  // , ISwaptionVolatilityStructure
	{
		/// <summary>
		/// Protected constructor for abstract base class.
		/// </summary>
		protected CapFlatVolatilityStructure()
		{}

		/// <summary>
		/// Volatility for a given end time and strike rate.
		/// </summary>
		/// <overloads>
		/// Volatility.
		/// </overloads>
		/// <remarks>
		/// This method must be overridden to implement the actual volatility
		/// calculation in derived classes.
		/// </remarks>
		/// <param name="time">End time from settlement date.</param>
		/// <param name="strike"></param>
		/// <returns>The volatility expressed on an annual basis.</returns>
		[ ComVisible(false) ]
		[ Description("Volatility for a given end time and strike rate.") ]
		public abstract double Volatility(double time, double strike);

		/// <summary>
		/// Volatility for a given end date and strike rate.
		/// </summary>
		/// <overloads>
		/// Volatility.
		/// </overloads>
		/// <param name="end">End date.</param>
		/// <param name="strike"></param>
		/// <returns>The volatility expressed on an annual basis.</returns>
		[ ComVisible(true) ]
		[ Description("Volatility for a given end date and strike rate.") ]
		public double Volatility(DateTime end, double strike)
		{
			return Volatility( date2time(end), strike);
		}

		/// <summary>
		/// Volatility for a given cap/floor length and strike rate.
		/// </summary>
		/// <overloads>
		/// Volatility.
		/// </overloads>
		/// <param name="length">Cap/floor <see cref="Period"/>.</param>
		/// <param name="strike"></param>
		/// <returns>The volatility expressed on an annual basis.</returns>
		[ ComVisible(true) ]
		[ Description("Volatility for a given cap/floor length and strike rate.") ]
		public double Volatility(Period length, double strike)
		{
			return Volatility( date2time(SettlementDate+length), strike);
		}


		/// <summary>
		/// Convert a <see cref="DateTime"/> to continous time
		/// since settlement date.
		/// </summary>
		/// <param name="date">The <see cref="DateTime"/> to convert.</param>
		/// <returns>Continous time since settlement date.</returns>
		protected double date2time(DateTime date) 
		{
			// TODO: check for valid dates eg. reference <= d <= maxdatetime
			// TODO: interpret certain dates (eg. serial <= 100) as time
			return DayCounter.YearFraction(SettlementDate, date);
		}

//		/// <summary>
//		/// The latest time for which the term structure can return vols.
//		/// </summary>
//		public virtual double MaxTime 
//		{
//			get{ return date2time(MaxDate); }
//		}
//		
//		/// <summary>
//		/// The latest date for which the term structure can return vols.
//		/// </summary>
//		/// <value><see cref="DateTime"/></value>
//		public abstract DateTime MaxDate 
//		{
//			get;
//		}
//
		/// <summary>
		/// Today's date.
		/// </summary>
		/// <value><see cref="DateTime"/></value>
		public abstract DateTime TodaysDate 
		{
			get;
		}

		/// <summary>
		/// The settlement date.
		/// </summary>
		/// <value><see cref="DateTime"/></value>
		public abstract DateTime SettlementDate 
		{
			get;
		}


		/// <summary>
		/// The <see cref="IDayCounter"/> used for internal date/time conversions.
		/// </summary>
		public abstract IDayCounter DayCounter 
		{
			get;
		}


	}
}
