// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: IBlackVolTermStructure.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:39  jens
 *  Initial import
 *
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Business;
using Stochastix.Utilities;

namespace Stochastix.QuantLib.Volatilities
{

	/// <summary>
	/// Black volatility term structure interface.
	/// </summary>
	/// <remarks>
	/// Interface for concrete black volatility term structures.
	/// <para>Volatilities are assumed to be expressed on an annual basis.</para>
	/// </remarks>
	[ RcsId("$Id: IBlackVolTermStructure.cs,v 1.1.1.1 2003/02/01 19:01:39 jens Exp $") ] 
	[ ComVisible(true) ]
	[ Description("Interface for concrete black volatility term structures. " +
		  "Volatilities are assumed to be expressed on an annual basis.") ]
	[ InterfaceTypeAttribute(ComInterfaceType.InterfaceIsDual) ]
	public interface IBlackVolTermStructure : IObservable 
	{

		/// <summary>
		/// A stringified representation of this object.
		/// </summary>
		/// <returns>A String representing the object.</returns>
		[
		Description("A stringified representation of this object."),
		DispId(0)	// DISPID_VALUE
		]
		string ToString();


		/// <summary>
		/// Black present (a.k.a spot) volatility at a given date.
		/// </summary>
		/// <overloads>
		/// Black present (a.k.a spot) volatility.
		/// </overloads>
		/// <param name="maturity">The <see cref="DateTime"/> to use for calculations.</param>
		/// <param name="strike"></param>
		/// <param name="extrapolate"></param>
		/// <returns>Volatility at the given date.</returns>
		[ Description("Black present (a.k.a spot) volatility at a given date.") ]
		double Volatility( DateTime maturity, double strike, bool extrapolate);

		/// <summary>
		/// Black present (a.k.a spot) volatility at a given time from reference.
		/// </summary>
		/// <param name="maturity">Time from reference.</param>
		/// <param name="strike"></param>
		/// <param name="extrapolate"></param>
		/// <returns>Volatility at the given time.</returns>
		[ ComVisible(false) ]
		[ Description("Black present (a.k.a spot) volatility at a given time.") ]
		double Volatility( double maturity, double strike, bool extrapolate);


		/// <summary>
		/// Black present (a.k.a spot) variance at a given date.
		/// </summary>
		/// <overloads>
		/// Black present (a.k.a spot) variance.
		/// </overloads>
		/// <param name="maturity">The <see cref="DateTime"/> to use for calculations.</param>
		/// <param name="strike"></param>
		/// <param name="extrapolate"></param>
		/// <returns>Variance at the given date.</returns>
		[ Description("Black present (a.k.a spot) variance at a given date.") ]
		double Variance( DateTime maturity, double strike, bool extrapolate);

		/// <summary>
		/// Black present (a.k.a spot) variance at a given time from reference.
		/// </summary>
		/// <param name="maturity">Time from reference.</param>
		/// <param name="strike"></param>
		/// <param name="extrapolate"></param>
		/// <returns>Variance at the given time.</returns>
		[ ComVisible(false) ]
		[ Description("Black present (a.k.a spot) variance at a given time.") ]
		double Variance( double maturity, double strike, bool extrapolate);


		/// <summary>
		/// Black future (a.k.a. forward) volatility between given dates.
		/// </summary>
		/// <overloads>
		/// Black future (a.k.a. forward) volatility.
		/// </overloads>
		/// <param name="date1">The start date of the future period.</param>
		/// <param name="date2">The end date of the future period.</param>
		/// <param name="strike"></param>
		/// <param name="extrapolate"></param>
		/// <returns>Volatility between given dates.</returns>
		[ Description("Black future (a.k.a. forward) volatility between given dates.") ]
		double ForwardVolatility( DateTime date1, DateTime date2, double strike, bool extrapolate);

		/// <summary>
		/// Black future (a.k.a. forward) volatility between given times.
		/// </summary>
		/// <param name="time1">The start time of the future period.</param>
		/// <param name="time2">The end time of the future period.</param>
		/// <param name="strike"></param>
		/// <param name="extrapolate"></param>
		/// <returns>Volatility between given times.</returns>
		[ Description("Black future (a.k.a. forward) volatility between given times.") ]
		[ ComVisible(false) ]
		double ForwardVolatility( double time1, double time2, double strike, bool extrapolate);


		/// <summary>
		/// Black future (a.k.a. forward) variance between given dates.
		/// </summary>
		/// <overloads>
		/// Black future (a.k.a. forward) variance.
		/// </overloads>
		/// <param name="date1">The start date of the future period.</param>
		/// <param name="date2">The end date of the future period.</param>
		/// <param name="strike"></param>
		/// <param name="extrapolate"></param>
		/// <returns>Variance between given dates.</returns>
		[ Description("Black future (a.k.a. forward) variance between given dates.") ]
		double ForwardVariance( DateTime date1, DateTime date2, double strike, bool extrapolate);

		/// <summary>
		/// Black future (a.k.a. forward) variance between given times.
		/// </summary>
		/// <param name="time1">The start time of the future period.</param>
		/// <param name="time2">The end time of the future period.</param>
		/// <param name="strike"></param>
		/// <param name="extrapolate"></param>
		/// <returns>Volatility between given times.</returns>
		[ Description("Black future (a.k.a. forward) variance between given times.") ]
		[ ComVisible(false) ]
		double ForwardVariance( double time1, double time2, double strike, bool extrapolate);


		/// <summary>
		/// Partial derivative of Black volatility with respect to time.
		/// </summary>
		/// <param name="time"></param>
		/// <param name="strike"></param>
		/// <param name="extrapolate"></param>
		/// <returns></returns>
		[ Description("Partial derivative of Black volatility with respect to time.") ]
		double TimeDerivative(double time, double strike, bool extrapolate);
        
		/// <summary>
		/// Partial derivative of Black volatility with respect to strike.
		/// </summary>
		/// <param name="time"></param>
		/// <param name="strike"></param>
		/// <param name="extrapolate"></param>
		/// <returns></returns>
		[ Description("Partial derivative of Black volatility with respect to strike.") ]
		double StrikeDerivative(double time, double strike, bool extrapolate);
                   
		/// <summary>
		/// Second-order derivative of Black volatility with respect to strike.
		/// </summary>
		/// <param name="time"></param>
		/// <param name="strike"></param>
		/// <param name="extrapolate"></param>
		/// <returns></returns>
		[ Description("Second-order derivative of Black volatility with respect to strike.") ]
		double StrikeSecondDerivative(double time, double strike, bool extrapolate);


		/// <summary>
		/// The latest time for which the term structure can return vols.
		/// </summary>
		double MaxTime 
		{
			get;
		}
		
		/// <summary>
		/// The latest date for which the term structure can return vols.
		/// </summary>
		/// <value><see cref="DateTime"/></value>
		DateTime MaxDate 
		{
			get;
		}

		/// <summary>
		/// The reference date of this curve for which t=0.
		/// </summary>
		/// <value><see cref="DateTime"/></value>
		DateTime ReferenceDate 
		{
			get;
		}

		/// <summary>
		/// The <see cref="IDayCounter"/> used for internal date/time conversions.
		/// </summary>
		IDayCounter DayCounter 
		{
			get;
		}


	}

}