// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: SwaptionVolatilityMatrix.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:01:40  jens
 *  Initial import
 *
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using System.Collections;
using Stochastix.Business;
using Stochastix.Business.DayCounters;
using Stochastix.Utilities;
using Stochastix.QuantLib.Resources;
using Stochastix.QuantLib.MarketElements;
using Stochastix.Numerics;
using Stochastix.Numerics.Interpolations;

namespace Stochastix.QuantLib.Volatilities
{

	/// <summary>
	/// Swaption at-the-money volatility matrix.
	/// </summary>
	/// <remarks>
	/// This class provides the at-the-money volatility for a given 
	/// swaption by interpolating a volatility matrix whose elements are
	/// the market volatilities of a set of swaption with given exercise 
	/// date and length.
	/// <para>
	/// The calculation is performed interpolating on the volatility surface.
	/// </para>
	/// <para>
	/// Volatilities are assumed to be expressed on an annual basis.
	/// </para>
	/// </remarks>
	[ RcsId("$Id: SwaptionVolatilityMatrix.cs,v 1.1.1.1 2003/02/01 19:01:40 jens Exp $") ] 
	[ ComVisible(true) ]
	public class SwaptionVolatilityMatrix : SwaptionVolatilityStructure
	{

		/// <summary>
		/// Constructor for SwaptionVolatilityMatrix class.
		/// </summary>
		public SwaptionVolatilityMatrix(DateTime todaysDate,
			IList exerciseDates, IList lengths, Matrix volatilities) 
			: this(todaysDate, exerciseDates, lengths, volatilities, 
				Thirty360US.Instance )
		{}

		/// <summary>
		/// Constructor for SwaptionVolatilityMatrix class.
		/// </summary>
		public SwaptionVolatilityMatrix(DateTime todaysDate,
			IList exerciseDates, IList lengths, Matrix volatilities,
			IDayCounter dayCounter) : base()
		{
			if( exerciseDates.Count != volatilities.Columns )
				throw new ArgumentException( Strings.Get( GetType(),
					"TODO: size mismatch between date vector and vol matrix colums" ), 
					"volatilities" );
			if( lengths.Count != volatilities.Rows )
				throw new ArgumentException( Strings.Get( GetType(),
					"TODO: size mismatch between lengths vector and vol matrix rows" ), 
					"volatilities" );

//            // cannot have dates[0]==todaysDate, since the
//            // value of the vol at dates[0] would be lost
//            // (variance at todaysDate must be zero)
//			if( (DateTime)dates[0] <= todaysDate )
//				throw new ArgumentException( Strings.Get( GetType(),
//					"TODO: cannot have dates[0]<=todaysDate" ), "dates" );
//
			this.todaysDate = todaysDate;
			this.dayCounter = dayCounter;

			this.times     = new double[ exerciseDates.Count ];
			for( int i=0; i<this.times.Length; i++)
				this.times[i] = date2time( (DateTime)exerciseDates[i] );

			this.lengths   = new double[ lengths.Count ];
			DateTime startDate = (DateTime)exerciseDates[0]; // as good as any
			for( int i=0; i<this.lengths.Length; i++)
				this.lengths[i] = DayCounter.YearFraction( startDate,
					startDate+(Period)lengths[i] );

			// this.maxDate = (DateTime)dates[dates.Count-1];

			this.interpolation = new BilinearInterpolation(
				this.times, this.lengths, volatilities);
		}

		private DateTime todaysDate;
		private IDayCounter dayCounter;
		// private DateTime maxDate;
		private double[] times = null;
		private double[] lengths = null;
		private Interpolation2D interpolation;


		/// <summary>
		/// Swaption volatility for a given starting time and length.
		/// </summary>
		/// <overloads>
		/// Swaption volatility.
		/// </overloads>
		/// <param name="startTime">Start time from today's date.</param>
		/// <param name="length"></param>
		/// <param name="strike"></param>
		/// <returns>The swaption volatility expressed on an annual basis.</returns>
		public override double Volatility(double startTime, double length, double strike)
		{
//			if( startTime < 0.0 )
//				throw new ArgumentException( Strings.Get( GetType(),
//					"TODO: negative time not allowed." ), "startTime" );
			return interpolation.Value(startTime, length, false);
		}



//		/// <summary>
//		/// The latest date for which the term structure can return vols.
//		/// </summary>
//		/// <value><see cref="DateTime"/></value>
//		public override DateTime MaxDate 
//		{
//			get { return maxDate; }
//		}

		/// <summary>
		/// The date of this curve for which t=0.
		/// </summary>
		/// <value><see cref="DateTime"/></value>
		public override DateTime TodaysDate 
		{
			get { return todaysDate; }
		}

		/// <summary>
		/// The <see cref="IDayCounter"/> used for internal date/time conversions.
		/// </summary>
		public override IDayCounter DayCounter 
		{
			get { return dayCounter; }
		}


	}
}
