// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: CachedContinousRng.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:02:13  jens
 *  Initial import
 *
 *  Revision 1.2  2002/12/05 15:57:51  stochastix
 *  synced with my current sandbox
 *
 *  Revision 1.1  2002/10/15 12:51:08  stochastix
 *  Initial implementation of a PRNG framework
 *  with support for Intel MKL/VSL.
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Utilities;

namespace Stochastix.Random
{
	/// <summary>
	/// Abstract base for a cached continous random number generator.
	/// </summary>
	/// <remarks>
	/// Derived classes must override and supply the concrete RNG 
	/// implementation in <see cref="Generate"/>.
	/// </remarks>
	[ RcsId("$Id: CachedContinousRng.cs,v 1.1.1.1 2003/02/01 19:02:13 jens Exp $") ] 
	[ ComVisible(false) ]
	public abstract class CachedContinousRng : IContinousRng

	{
		/// <summary>
		/// Initialize a cached continous RNG with the default cache size (256 numbers).
		/// </summary>
		/// <overloads>
		/// Initialize a cached continous random number generator.
		/// </overloads>
		protected CachedContinousRng() : this(256)
		{}

		/// <summary>
		/// Initialize a cached continous RNG with the given cache size..
		/// </summary>
		/// <param name="cacheSize">Size of the cache for random numbers.</param>
		/// <remarks>
		/// Adjust the <paramref name="cacheSize"/> to optimize the performance of the
		/// underlying library.
		/// </remarks>
		protected CachedContinousRng(int cacheSize)
		{
			cache = new Double[cacheSize];
		}

		/// <summary>
		/// Array acting as a cache for random numbers.
		/// </summary>
		double[] cache;

		/// <summary>
		/// Index to first "fresh" random number in <see cref="cache"/>.
		/// </summary>
		int current = int.MaxValue;


		/// <summary>
		/// Draw random samples from the generator.
		/// </summary>
		/// <remarks>
		/// Must be overridden by derived classes with a concrete RNG implementation.
		/// </remarks>
		/// <param name="r">
		///		An <see cref="Array"/> to be filled with double-precision floating point 
		///		numbers.</param>
		/// <param name="start">
		///		The (zero-based) index of the first array element to fill.
		/// </param>
		/// <param name="length">
		///		The number of random samples to generate.
		/// </param>
		abstract protected void Generate(double[] r, int start, int length);

		#region Implementation of IContinousRng

		/// <summary>
		/// Returns a random number between 0.0 and 1.0.
		/// </summary>
		/// <returns>
		/// A double-precision floating point number greater than or equal to 0.0, 
		/// and less than 1.0.
		/// </returns>
		/// <remarks>
		/// The value is returned from a cache that is filled by the underlying library.
		/// </remarks>
		public double NextDouble()
		{
			if( current >= cache.Length )
			{
				Generate(cache, 0, cache.Length);
				current = 0;
			}
			return cache[current++];
		}

		/// <summary>
		/// Returns an <see cref="Array"/> of random number between 0.0 and 1.0.
		/// </summary>
		/// <param name="r">
		/// A double-precision floating point <see cref="Array"/> to be filled
		/// with random numbers number greater than or equal to 0.0, and less than 1.0.
		/// </param>
		/// <remarks>
		/// The value can be returned from a cache that is filled by the underlying library.
		/// <para>
		/// When the requested number of samples is larger than the size of the cache, the
		/// underlying library is used to fill the passed array directly. If you want to make
		/// sure that random numbers are returned in the same order as generated by the
		/// underlying RNG make the cache size at least the same size as the
		/// largest number of samples requested in one call.
		/// </para>
		/// </remarks>
		public void Next(double[] r)
		{
			Next(r, r.GetLowerBound(0), r.Length);
		}

		/// <summary>
		/// Returns an <see cref="Array"/> of random number between 0.0 and 1.0.
		/// </summary>
		/// <param name="r">
		/// A double-precision floating point <see cref="Array"/> to be filled
		/// with random numbers number greater than or equal to 0.0, and less than 1.0.
		/// </param>
		/// <param name="start">Index of the first elemnt to fill.</param>
		/// <remarks>
		/// The value can be returned from a cache that is filled by the underlying library.
		/// <para>
		/// When the requested number of samples is larger than the size of the cache, the
		/// underlying library is used to fill the passed array directly. If you want to make
		/// sure that random numbers are returned in the same order as generated by the
		/// underlying RNG make the cache size at least the same size as the
		/// largest number of samples requested in one call.
		/// </para>
		/// </remarks>
		public void Next(double[] r, int start)
		{
			Next( r, start, r.GetUpperBound(0)-start+1);
		}

		/// <summary>
		/// Returns an <see cref="Array"/> of random number between 0.0 and 1.0.
		/// </summary>
		/// <param name="r">
		/// A double-precision floating point <see cref="Array"/> to be filled
		/// with random numbers number greater than or equal to 0.0, and less than 1.0.
		/// </param>
		/// <param name="start">Index of the first elemnt to fill.</param>
		/// <param name="length">Number of random numbers to generate.</param>
		/// <remarks>
		/// The value can be returned from a cache that is filled by the underlying library.
		/// <para>
		/// When the requested number of samples is larger than the size of the cache, the
		/// underlying library is used to fill the passed array directly. If you want to make
		/// sure that random numbers are returned in the same order as generated by the
		/// underlying RNG make the cache size at least the same size as the
		/// largest number of samples requested in one call.
		/// </para>
		/// </remarks>
		public virtual void Next(double[] r, int start, int length)
		{
			if( current >= cache.Length ) 
			{
				Generate( r, start, length);	// no cache
				return;						// quick exit
			}

			// use elements from cache if available
			int cached = Math.Min(length, cache.Length - current);
			if( cached > 0 )	
			{
				Array.Copy(cache, current, r, start, cached);
				current += cached;
			}
			// generate remaining elements
			if( cached < length )
				Generate( r, start+cached, length-cached);
		
		}

		#endregion

	}
}
