// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: R250.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:02:14  jens
 *  Initial import
 *
 *  Revision 1.2  2002/12/05 15:57:54  stochastix
 *  synced with my current sandbox
 *
 *  Revision 1.1  2002/10/15 12:51:08  stochastix
 *  Initial implementation of a PRNG framework
 *  with support for Intel MKL/VSL.
 *
 *
 *********************************************************************
 */
#endregion

using System;

namespace Stochastix.Random.Generators
{
	/* This is a shift-register random number generator. The sequence is

	   x_n = x_{n-103} ^ x_{n-250}        ("^" means XOR)

	   defined on 32-bit words.

	   The first 250 elements x_1 .. x_250 are first initialized as x_n =
	   s_n, where s_n = (69069*s_{n-1}) mod 2^32 and s_0=s is the
	   user-supplied seed. To ensure that the sequence does not lie on a
	   subspace we force 32 of the entries to be linearly independent.  We
	   take the 32 elements x[3], x[10], x[17], x[24], ..., 213 and apply
	   the following operations,

	   x[3]   &= 11111111111111111111111111111111
	   x[3]   |= 10000000000000000000000000000000 
	   x[10]  &= 01111111111111111111111111111111
	   x[10]  |= 01000000000000000000000000000000 
	   x[17]  &= 00111111111111111111111111111111
	   x[17]  |= 00100000000000000000000000000000 
	   ....      ...
	   x[206] &= 00000000000000000000000000000111
	   x[206] |= 00000000000000000000000000000100 
	   x[213] &= 00000000000000000000000000000011
	   x[213] |= 00000000000000000000000000000010 
	   x[220] &= 00000000000000000000000000000001
	   x[220] |= 00000000000000000000000000000001 

	   i.e. if we consider the bits of the 32 elements as forming a 32x32
	   array then we are setting the diagonal bits of the array to one and
	   masking the lower triangle below the diagonal to zero.

	   With this initialization procedure the theoretical value of
	   x_{10001} is 1100653588 for s = 1 (Actually I got this by running
	   the original code). The subscript 10001 means (1) seed the
	   generator with s = 1 and then do 10000 actual iterations.

	   The period of this generator is about 2^250.

	   The algorithm works for any number of bits. It is implemented here
	   for 32 bits.

	   From: S. Kirkpatrick and E. Stoll, "A very fast shift-register
	   sequence random number generator", Journal of Computational Physics,
	   40, 517-526 (1981). 

	*/

	/// <summary>
	/// A shift-register random number generator.
	/// </summary>
	public sealed class R250 : BasicRng
	{
		private int i = 0;
		private uint[] x = new uint[250];

		override public object Clone()
		{
			// Perform a shallow copy first
			R250 clone = (R250) base.Clone();
			// and deep clone the seeds
			clone.x = (uint[])this.x.Clone();
			return clone;
		}

		public R250() : this(1)	// default seed is 1
		{}

		public R250(int seed)
		{
			uint s = (uint)seed;

			for (int j=0; j<250; j++) // Fill the buffer
			{
				//	#define LCG(n) ((69069 * n) & 0xffffffffUL)
				//	s = LCG (s);
				s = (69069*s) & 0xffffffffU;	// bitwise AND
				this.x[j] = s;
			}

			// make linearly independent -- see remarks
			uint msb = 0x80000000U;
            uint mask = 0xffffffffU;
			for (int k=3; k<=220; k+=7)
			{
				this.x[k] &= mask;	// Turn off bits left of the diagonal
				this.x[k] |= msb;	// Turn on the diagonal bit
				mask >>= 1;
				msb >>= 1;
			}
		}

		uint NextUInt32()
		{
			int j = i;
			if (j >= 147) j -= 147; else j += 103;

			uint k = x[i] ^ x[j];	// bitwise XOR
			x[i] = k;

			if (i >= 249) i = 0; else i++;

			return k;
		}

		/// <summary>
		/// Draw a random sample for the generator.
		/// </summary>
		/// <returns>
		/// A double-precision floating point number greater than 0.0 
		/// and less than 1.0.
		/// </returns>
		override protected double Sample()
		{
			// UInt32.MaxValue == 4294967295;
			// resulting in [0.0..1.0[
            return NextUInt32() / 4294967296.0 ;	
		}


	}
}


