// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: TestRandom.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:02:15  jens
 *  Initial import
 *
 *  Revision 1.2  2002/12/05 15:57:51  stochastix
 *  synced with my current sandbox
 *
 *  Revision 1.1  2002/10/15 12:53:47  stochastix
 *  added tests for random numbers and risk statistics
 *
 *
 *********************************************************************
 */
#endregion

#undef SHOW_RANDOM_NUMBERS

namespace Stochastix.Random.Tests
{
	using System;
	using System.Collections;
	using NUnit.Framework;
	using Stochastix.Random;
	using Stochastix.Numerics.Distributions;
	using Org.Mentalis.Utilities;

	/// <summary>
	/// 
	/// </summary>
	public class TestRandom: TestCase 
	{
		/// <summary>
		/// Just a constructor. I think you can pass the name of the
		/// test (==method) to run.
		/// </summary>
		/// <param name="name">Name of the test to run.</param>
		public TestRandom(String name): base(name) {}

		// declare private members here used for test setup

		/// <summary>
		/// Initialize this test case.
		/// </summary>
		protected override void SetUp() 
		{
		}

		private const double average = 0.0;
		private const double sigma = 1.0;
		private const int N = 10000;
		private const int SEED = 1;

		private string[] basicRngs = { "MCG31", "R250", "MRG32k3a" };

		/// <summary>
		/// Free all you resources.
		/// </summary>
		protected override void TearDown()
		{
		}

		private InvCumulativeNormalDistribution invCum = 
			new InvCumulativeNormalDistribution(average, sigma);

		public void TestUniformRng()
		{
			const double a=0.5,b=2.0;
			const double n=(double)N;

			double[] r = new double[N];

			/***** Theoretical moments *****/
			double tM=(b+a)/2.0;
			double tD=((b-a)*(b-a))/12.0;
			double tQ=((b-a)*(b-a)*(b-a)*(b-a))/80.0;

			foreach( String basicRng in basicRngs)
			{
				using( IRandomStream stream =
						   RandomStreamFactory.Instance.Create(basicRng, SEED) )
				{
					stream.UniformRng(a, b).Next(r);

					/***** Sample moments *****/
					double sum=0;
					double sum2=0;
					for(int i=0;i<N;i++) 
					{
						sum+=(double)r[i];
						sum2+=(double)r[i]*(double)r[i];
					}
					double sM=sum/n;
					double sD=sum2/n-(sM*sM);

					/***** Comparison of theoretical and sample moments *****/
					double tD2=tD*tD;
					double s=((tQ-tD2)/n)-(2*(tQ-2*tD2)/(n*n))+((tQ-3*tD2)/(n*n*n));

					double DeltaM=(tM-sM)/Math.Sqrt(tD/n);
					double DeltaD=(tD-sD)/Math.Sqrt(s);

					/***** Printing results *****/
#if SHOW_RANDOM_NUMBERS
					Console.WriteLine("Results (first 10 of 1000) for '{0}':", basicRng);
					for(int i=0;i<10;i++) 
						Console.WriteLine("r[{0}]={1}", i, r[i]);
#endif
					Assertion.Assert( basicRng + ": Sample moments are disagreed with theory",
						DeltaM<=3.0 && DeltaD<=3.0);
				}
			}
		}

		public void TestGaussianRng()
		{
			const double a=0.5, sigma=2.0;
			const double n=(double)N;

			double[] r = new double[N];

			/***** Theoretical moments *****/
			double tM=a;
			double tD=sigma*sigma;
			double tQ=720.0*sigma*sigma*sigma*sigma;

			foreach( String basicRng in basicRngs)
			{
				using( IRandomStream stream =
						   RandomStreamFactory.Instance.Create(basicRng, SEED) )
				{
					stream.GaussianRng(a, sigma).Next(r);

					/***** Sample moments *****/
					double sum=0;
					double sum2=0;
					for(int i=0;i<N;i++) 
					{
						sum+=(double)r[i];
						sum2+=(double)r[i]*(double)r[i];
					}
					double sM=sum/n;
					double sD=sum2/n-(sM*sM);

					/***** Comparison of theoretical and sample moments *****/
					double tD2=tD*tD;
					double s=((tQ-tD2)/n)-(2*(tQ-2*tD2)/(n*n))+((tQ-3*tD2)/(n*n*n));

					double DeltaM=(tM-sM)/Math.Sqrt(tD/n);
					double DeltaD=(tD-sD)/Math.Sqrt(s);

					/***** Printing results *****/
#if SHOW_RANDOM_NUMBERS
					Console.WriteLine("Results (first 10 of 1000) for '{0}':", basicRng);
					for(int i=0;i<10;i++) 
						Console.WriteLine("r[{0}]={1}", i, r[i]);
#endif
					Assertion.Assert( basicRng + ": Sample moments are disagreed with theory",
						DeltaM<=3.0 && DeltaD<=3.0);
				}
			}
		}

		public void TestVslPerformance()
		{
			int N = 100000;
			const int ROUNDS = 10;
			const double a=0.5, b=2.0;
			double[] r = new double[N];
			StopWatch s = new StopWatch();

			Console.WriteLine("Starting {0} rounds with {1} numbers", ROUNDS, N);

			foreach( String basicRng in basicRngs)
			{
				using( IRandomStream stream =
						   RandomStreamFactory.Instance.Create(basicRng, SEED) )
				{
					IContinousRng rng = stream.GaussianRng(a, b);
					GC.Collect();
					
					for( int i=0; i< ROUNDS+3; i++ )
					{
						if( i==3) s.Reset(); // three warmup rounds
						rng.Next(r);
					}
					Console.WriteLine("rng={0} (DotNum.Random), avg/100ms={1}", basicRng, s.Peek()/ROUNDS);
				}
			}
		}

	}
}
