// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: VSL.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:02:14  jens
 *  Initial import
 *
 *  Revision 1.2  2002/12/05 15:57:51  stochastix
 *  synced with my current sandbox
 *
 *  Revision 1.1  2002/10/15 12:51:08  stochastix
 *  Initial implementation of a PRNG framework
 *  with support for Intel MKL/VSL.
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using System.Diagnostics;
using System.Text;
using Stochastix.Utilities;

namespace Stochastix.Random
{

	/// <summary>
	/// Interface to the optimized Intel vector statistics library (VSL).
	/// </summary>
	[ RcsId("$Id: VSL.cs,v 1.1.1.1 2003/02/01 19:02:14 jens Exp $") ] 
	[ ComVisible(false) ]
	public sealed class VSL
	{
		/// <summary>
		/// VSL Version or null (Nothing in Visual Basic) when the library is not available.
		/// </summary>
		public static readonly string Version;

		#region Static Constructor and availability check

		private const string IMPORT_DLL = "Stochastix.Random.Native.dll";
		private const string DLL_Version = "GetVersionVSL";

		static VSL() 
		{
			StringBuilder buf = new StringBuilder(120);
			try 
			{
				GetVersion(buf, buf.Capacity);
//				oldmode = vmlGetMode();
//				vmlSetMode(Mode.VML_HA | Mode.VML_DOUBLE_CONSISTENT | 
//					Mode.VML_ERRMODE_ERRNO);
//				vmlClearErrStatus();
			} 
			catch (Exception e) 
			{
				Trace.WriteLine("VSL not loaded: " + e.Message);
				return;
			}
			Version = buf.ToString().Trim();
			Trace.WriteLine("VSL loaded '" + Version + "'");
		}

		[ DllImport(IMPORT_DLL, EntryPoint=DLL_Version) ]
		private static extern void GetVersion(StringBuilder buf, int bufSize);

		#endregion

		#region VSL Service Functions

		/// <summary>
		/// BRNG names.
		/// </summary>
		internal enum BasicRng
		{
			VSL_BRNG_MCG31		= 0,
			VSL_BRNG_R250		= 1,
			VSL_BRNG_MRG32K3A	= 2
		}

		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		private static extern void vslNewStream( ref IntPtr stream, int brng, uint seed );

//		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
//		private static extern void vslNewStreamEx( ref IntPtr stream , int brng, int nSeeds, uint* seeds );

		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		private static extern void vslDeleteStream( ref IntPtr stream);

		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		private static extern void vslCopyStream( out IntPtr newStream, IntPtr srcStream);

		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		private static extern void vslLeapfrogStream( IntPtr stream, int k, int nStreams);
		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		private static extern void vslSkipAheadStream( IntPtr stream, int k, int nStreams);

		[StructLayout(LayoutKind.Sequential)]
		private struct VSLBRngProperties 
		{
			int StreamStateSize;    /* Stream state size (in bytes) */
			int NSeeds;             /* Number of seeds */
			int IncludesZero;       /* Zero flag */
			int WordSize;           /* Size (in bytes) of base word */
			int NBits;              /* Number of actually used bits */
			IntPtr InitStream;		/* Pointer to InitStream func */
			IntPtr sBRng;			/* Pointer to S func */
			IntPtr dBRng;           /* Pointer to D func */
			IntPtr iBRng;           /* Pointer to I func */
		};

		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		private static extern void vslGetBrngProperties( int brng, ref VSLBRngProperties properties );

		#endregion

		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		internal static extern void vdRngUniform(int method, IntPtr stream, 
			int n, ref double r, double a, double b);

		[ DllImport(IMPORT_DLL, CallingConvention=CallingConvention.Cdecl) ]
		internal static extern void vdRngGaussian(int method, IntPtr stream, 
			int n, ref double r, double a, double b);


		internal static IntPtr NewStream(int brng, int seed )
		{
			IntPtr newStream= IntPtr.Zero;
			try 
			{
				vslNewStream(ref newStream, brng, (uint)seed);
			}
			catch(Exception e) 
			{
				Console.WriteLine(e.ToString());
			}
			return newStream;
		}

		internal static IntPtr CopyStream(IntPtr vslStreamStatePtr)
		{
			IntPtr newStream;
			vslCopyStream( out newStream, vslStreamStatePtr);
			return newStream;
		}

		internal static void DeleteStream(ref IntPtr vslStreamStatePtr)
		{
			vslDeleteStream(ref vslStreamStatePtr);
		}

	}
}
