// This file is part of Stochastix.NET, a set of open source assemblies 
// for the .NET framework. See http://stochastix.net/ for details.

#region Copyright  2001-2003 Jens Thiel and others. All rights reserved.
/*
 ***************************************************** \begincopyright
 *
 * Copyright (c) 2001-2003 Jens Thiel (http://www.jens-thiel.de/)
 * See LICENSE.TXT for details. All rights reserved. 
 *
 * The licensing policy in brief:
 * Our free software is 100% GPL, and if yours is also 100% GPL (or OSI 
 * compliant), then you never have to pay us for the licences. In all 
 * other instances, you are better served by our commercial licence. 
 *
 * See http://stochastix.net/license or included LICENSE.TXT for details.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ******************************************************* \endcopyright
 */
#endregion

#region RCS Log
/*
 *********************************************************** \internal
 *
 *  $Log: VslRandomStream.cs,v $
 *  Revision 1.1.1.1  2003/02/01 19:02:14  jens
 *  Initial import
 *
 *  Revision 1.2  2002/12/05 15:57:51  stochastix
 *  synced with my current sandbox
 *
 *  Revision 1.1  2002/10/15 12:51:08  stochastix
 *  Initial implementation of a PRNG framework
 *  with support for Intel MKL/VSL.
 *
 *
 *********************************************************************
 */
#endregion

using System;
using System.Runtime.InteropServices;	// COM interop attributes
using System.ComponentModel;			// some useful attributes
using Stochastix.Utilities;

namespace Stochastix.Random
{
	/// <summary>
	/// Summary description for VslRandomStream.
	/// </summary>
	[ RcsId("$Id: VslRandomStream.cs,v 1.1.1.1 2003/02/01 19:02:14 jens Exp $") ] 
	[ ComVisible(false) ]
	internal class VslRandomStream : IRandomStream
	{
		public VslRandomStream(IntPtr vslStreamStatePtr)
		{
			this.vslStreamStatePtr = vslStreamStatePtr;
		}

		private IntPtr vslStreamStatePtr;

		#region Implementation of ICloneable
		public IRandomStream Clone()
		{
			return new VslRandomStream( VSL.CopyStream( vslStreamStatePtr) );
		}
		object ICloneable.Clone()
		{
			return this.Clone();
		}
		#endregion

		#region Implementation of IDisposable
		public void Dispose()
		{
			if( vslStreamStatePtr != IntPtr.Zero)
			{
				VSL.DeleteStream(ref vslStreamStatePtr);
				vslStreamStatePtr = IntPtr.Zero;
			}
		}
		#endregion

		#region Implementation of IRandomStream

		public IContinousRng UniformRng(double minValue, double maxValue)
		{
			return new VslUniformRng( this, minValue, maxValue);
		}

		public IContinousRng GaussianRng( double mean, double sigma)
		{
			return new VslGaussianRng( this, mean, sigma);
		}

		#endregion

		#region VSL RNG classes

		/// <summary>
		/// Summary description for VslAbstractContinousRng.
		/// </summary>
		private abstract class VslAbstractContinousRng 
			: CachedContinousRng
		{
			public VslAbstractContinousRng( VslRandomStream stream)
			{
				this.stream = stream;
			}

			protected VslRandomStream stream;

		}

		/// <summary>
		/// Summary description for VslUniformRng.
		/// </summary>
		private class VslUniformRng : VslAbstractContinousRng
		{
			public VslUniformRng( VslRandomStream stream, double minValue, double maxValue)
				: base(stream)
			{
				if (minValue>maxValue) 
					throw new ArgumentOutOfRangeException("minValue");
				this.minValue = minValue;
				this.maxValue = maxValue;
			}

			private double minValue, maxValue;

			override protected void Generate(double[] r, int start, int n)
			{
				VSL.vdRngUniform(0, stream.vslStreamStatePtr, 
					n, ref r[start], minValue, maxValue);
			}
		}

		/// <summary>
		/// Summary description for VslGaussianRng.
		/// </summary>
		private class VslGaussianRng : VslAbstractContinousRng
		{
			public VslGaussianRng( VslRandomStream stream, double mean, double sigma)
				: base(stream)
			{
				this.mean = mean;
				this.sigma = sigma;
			}

			private double mean, sigma;

			override protected void Generate(double[] r, int start, int n)
			{
				VSL.vdRngGaussian(0, stream.vslStreamStatePtr, 
					n, ref r[start], mean, sigma);
			}
		}

		#endregion

	}
}
